/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.postit;

import java.awt.*;
import java.awt.event.*;
import java.util.ListResourceBundle;
import java.util.MissingResourceException;
import java.util.Locale;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;

import com.tivoli.uif.core.UFImageLoader;
import com.tivoli.twg.console.NLSUtilities;
import com.tivoli.twg.console.TWGHelpContext;
import com.tivoli.twg.console.TWGTaskFrame;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.guilibs.ImageLabel;
import com.tivoli.twg.guilibs.WindowAttributeEvent;
import com.tivoli.twg.guilibs.WindowAttributes;
import com.tivoli.twg.libs.*;

/*****************************************************************************
*                         Bob's Software House
*                    A Division of BobCo Enterprises
*****************************************************************************/
/****************************************************************************
* This is an example GUI frame that will provide the task's interface
* to the user to allow them to create post-it notes for a managed object.
* In this example, a task window is displayed, with a client area that
* allows the user to enter a note that is then attached to the managed 
* object the task was launched against.
*
* @see TWGTaskFrame
*****************************************************************************/
public class BCPostItFrame extends TWGTaskFrame
                           implements ActionListener
{
    /**
    * This is the constructor for creating the task frame for the user.
    * This is a default constructor, which is required by TWGTaskFrame
    * since it is loaded as a bean.
    */
    public BCPostItFrame()
    {
        // Construct the parent frame without a status bar.
        super( false );
    }


    /**
     * Initialization routine for setting up non-gui items.  Derived classes
     * override this method to perform their initialization.  This method
     * is called from setTaskActivator after the task activation instance has
     * been initialized (meaning task object information can be retrieved from
     * the task activation object).  Return true to have the frame continue
     * initialization and be displayed.  Return false to stop the frame from
     * coming up.
     */
    public boolean pInit( )
    {
        boolean brc = true;

        try
        {
            // Load in the resource bundle for the frame.
            msgResources = (ListResourceBundle)ListResourceBundle.getBundle(
                                                                           "com.BobCo.postit.BCPostItResources", Locale.getDefault());
        }
        catch ( MissingResourceException mrx )
        {
            brc = false;
        } 

        // Set the frame's title and icon.
        setTitle( msgResources.getString( "ReminderNotesTitle" ));
        setIconImage( UFImageLoader.getImage( "/com/BobCo/postit/images/postit16.gif" ));

        return brc;
    }


    /**
     * Called by the console when the server task activation for this instance of
     * the task frame has ended.  All termination processing should be done here,
     * and the main thread should be exited.
     *
     * @param endingType This indicates why the server task ended (i.e. completed
     * successfully, error occurred, etc.
     */
    public void serverTaskEnded( int endingType )
    {
    }


    /**
     * Create the client pane and return it.
     */
    public Container buildView()
    {
        // Construct the top level client area pane that all panes are
        // placed into for the user interface.
        clientPanel = new JPanel();
        clientPanel.setBorder( new EmptyBorder( 10, 10, 10, 10 ) );
        clientPanel.setLayout( new BorderLayout() );

        // Build the attached objects group list
        objectGroup = new JPanel();
        objectGroup.setLayout( new BoxLayout( objectGroup, BoxLayout.Y_AXIS) );
        titledBorder = new TitledBorder(
                                       msgResources.getString( "PostItObjectsPmt" ));
        titledBorder.setTitleFont( boldFont );
        objectGroup.setBorder( new CompoundBorder( titledBorder,
                                                   new EmptyBorder( 10, 10, 10, 10 )) );

        // Provide the field to show the association object names in.
        systemNames = new JTextField();
        objectGroup.add( systemNames );

        // Populate with the target names.
        String targets = "";
        systemIDs = getTaskActivator().getMoid();
        if (systemIDs != null)
        {
            for (int i=0; i < systemIDs.Length(); i++ )
            {
                // Add space before each name, except first one, and then add name.
                if ( i > 0 )
                    targets += " ";
                targets += systemIDs.GetValue(i);
            }
        }
        systemNames.setText( targets );
        systemNames.setEditable( false );
        clientPanel.add( objectGroup, BorderLayout.NORTH );

        // Build the message information group
        textGroup = new JPanel();
        textGroup.setLayout( new BoxLayout( textGroup, BoxLayout.Y_AXIS) );
        titledBorder = new TitledBorder( msgResources.getString( "PostItMessagePmt" ));
        titledBorder.setTitleFont( boldFont );
        textGroup.setBorder( new CompoundBorder( titledBorder,
                                                 new EmptyBorder( 10, 10, 10, 10 )) );

        // Place the text area into a scrolling area.
        JScrollPane spane = new JScrollPane( JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                             JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );

        // Construct the field for allowing the user to enter the message
        // to be sent to the user specified.  We should make this a panel
        // that will wrap text as we don't know how long the message will be.
        noteText = new JTextPane();

        try
        {
            DefaultStyledDocument doc = new DefaultStyledDocument();
            SimpleAttributeSet defaultStyle = new SimpleAttributeSet();
            StyleConstants.setFontSize( defaultStyle, 12 );
            StyleConstants.setFontFamily( defaultStyle, "Dialog" );
            doc.insertString( doc.getLength(), "", defaultStyle );
            noteText.setDocument( doc );
        }
        catch ( javax.swing.text.BadLocationException e)
        {
        }
        spane.setViewportView( noteText );
        textGroup.add( spane );
        clientPanel.add( textGroup, BorderLayout.CENTER );

        // Create the button group for placing the buttons into.
        btnGroup = new JPanel();
        btnGroup.setLayout( new BoxLayout( btnGroup, BoxLayout.X_AXIS ) );
        btnGroup.setBorder( new EmptyBorder( 10, 10, 10, 10 ) );
        btnGroup.add( Box.createHorizontalGlue() );

        // Add the ok button to the panel.
        okButton = new JButton( msgResources.getString( "ReminderNoteOkBtn" ));
        okButton.addActionListener( this );
        okButton.setActionCommand( "OKMessage" );
        btnGroup.add( okButton );
        btnGroup.add( Box.createHorizontalStrut( 10 ) );

        // Add the cancel button to the panel.
        cancelButton = new JButton( msgResources.getString( "ReminderNoteCancelBtn" ));
        cancelButton.addActionListener( this );
        cancelButton.setActionCommand( "CancelMessage" );
        btnGroup.add( cancelButton );
        btnGroup.add( Box.createHorizontalStrut( 10 ) );

        // Add the help button to the panel.
        helpButton = new JButton( msgResources.getString( "ReminderNoteHelpBtn" ));
        btnGroup.add( helpButton );

        clientPanel.add( btnGroup, BorderLayout.SOUTH );

        // Add a help topic to the client panel.  Whenever help is requested
        // for this frame via the F1 key, the help window will be launched
        // and the help panel loaded will be the one matching the topic
        // associated here.
        if ( getHelpContext() != null )
        {
            getHelpContext().addHelp( clientPanel, "BCPostItConsole.main" );
            getHelpContext().addHelp( helpButton, "BCPostItConsole.main" );
        }

        updateWindowAttributes( getTaskActivator().getMainConsole().getWindowAttributes() );

        return clientPanel;
    }


    /**
     * This function is called when the window attributes for the application
     * have changed.  The Task Frame should make changes to its appearance
     * based on the new interface attributes.
     */
    public void windowAttributesChanged( WindowAttributeEvent wae )
    {
       updateWindowAttributes( wae.getWindowAttributes() );
    }

    private void updateWindowAttributes( WindowAttributes wa )
    {
        // Update the window appearance
        JComponent consoles[] = {clientPanel};
        ImageLabel labels[] = {};
        JComponent panels[] = {textGroup,objectGroup,btnGroup};

        // NOTE: 'consoles' will respond to the 'shadowed' option while 'panels'
        // will not.
        wa.applyColors( consoles, labels, panels );

        // Titled border text is an example of something that
        // must be set separately. Since this one is on a panel,
        // we will have it use the PANEL_TEXT color.
        titledBorder.setTitleColor( wa.getAppearanceColor( WindowAttributes.PANEL_TEXT ) );

        invalidate();  
        validate();
        repaint();    
    }


    /**
     * Clean up routine to allow derived task frame classes to clean up
     * after themselves.  Derived classes override this method to perform 
     * their cleanup.  This method is called when the task frame is 
     * definitely closing (meaning any user verification was already 
     * completed and the user indicated it was okay to close).
     */
    public void pCleanUp( )
    {
        okButton.removeActionListener( this );
        cancelButton.removeActionListener( this );
    }


    /**
     * Override TWGTaskFrame method to set focus
     */
    public void windowOpened( WindowEvent wevt )
    {
        super.windowOpened( wevt );
        noteText.requestFocus();
    }


    /**
     * Command handler class for actions.  In this sample, we only
     * have two commands, ok message and cancel/close the frame.
     */
    public void actionPerformed( ActionEvent evt )
    {
        String actionCommand = evt.getActionCommand();

        // If the command is ok message, then you would extract
        // the information from the client area.
        // We just do the same as cancel since this is not a real
        // application.  A more robust implementation would actually
        // check the fields for validity.
        if ( actionCommand.equals( "OKMessage" ))
        {
            // Get the text entered in the note area
            StyledDocument doc = noteText.getStyledDocument();
            String messageToSend = null;
            try
            {
                messageToSend = doc.getText( 0, doc.getLength() );
            }
            catch ( BadLocationException e )
            {
            }
            // Create the command to send to create a note.
            BCCreatePostItNoteCmd cmd = new BCCreatePostItNoteCmd( messageToSend, systemIDs );

            // Send the create command
            try
            {
                // Get this task frame's ServiceNode
                ServiceNode sn = getTaskFrameServiceNode();
                // Send note create command
                sn.SendAsynchCommand( cmd );
            }
            catch (BadServiceNodeImplException snix)
            {
                System.out.println(snix);
            }
            catch (ServiceNodeClosedException sncx)
            {
                System.out.println(sncx);
            }
            catch (ServiceNodeException snx)
            {
                System.out.println(snx);
            }
            catch (InterruptedException ix)
            {
                System.out.println(ix);
            }
            // After creating the note, exit.
            consoleCancel( TWGTaskFrame.USER_REQUESTED_CLOSE );
        }
        // If the command was cancel, then close the task frame down.
        // Calling consoleCancel() is the method to use and note how
        // we pass a parameter to communicate who requested the frame
        // close.
        else if ( actionCommand.equals( "CancelMessage" ))
        {
            consoleCancel( TWGTaskFrame.USER_REQUESTED_CLOSE );
        }
    }


    //
    // private data
    //
    // Get bold font only for locales that support them - plain otherwise.
    private Font               boldFont = NLSUtilities.getBoldFont( new Font("Dialog", Font.BOLD, 12) );
    private ListResourceBundle msgResources;
    private JPanel             clientPanel;
    private JPanel             textGroup;
    private JPanel             objectGroup;
    private JPanel             btnGroup;
    private TitledBorder       titledBorder;
    private JTextField         systemNames;
    private JTextPane          noteText;
    private JButton            okButton;
    private JButton            cancelButton;
    private JButton            helpButton;
    private LongValueSet       systemIDs;

} // end class BCSendMsgFrame
