/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.postit;

import com.tivoli.twg.engine.TWGContentInfoObject;
import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.MultiLocaleBundle;
import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import com.tivoli.twg.log.TWGRas;

/**
 * Custom content info object.  This was created for the sample since each
 * of our content information object names needs to include the note number
 * that is associated with its console objects (managed objects and/or filter
 * objects).
 *
 * @author David Bernath
 * @version 1.1 - 6/8/05
 */
public class BCPostItNoteCIO extends TWGContentInfoObject
{
    // State indicating this is an urgent request
    public static final int URGENT_NOTE = 0x00001;

    // Specific note id this object represents.
    private int noteID;
    private boolean noteDeleted = false;

    // Image set resource list for post it note content info objects
    private static final String[] POSTIT_CIOBJECT_IMGS = 
    {
        "/com/BobCo/postit/images/postit16.gif",
        "/com/BobCo/postit/images/posti16u.gif",
        "/com/BobCo/postit/images/postit32.gif",
        "/com/BobCo/postit/images/posti32u.gif"
    };

    /**
     * Constructor for content info object instance : creates instance
     * of a content info object that is associated with the managed objects
     * specified.
     * @param name The name of the content info object (or the key)
     * @param name_rb The name of the resource bundle to use for finding the
     *                key in.
     * @param bean_id The Object ID for the content info bean object that
     *                represents the task's associated view.
     * @param oids The list of objects that this task view object is associated
     *             with.
     * @param note_id The unique note id that this object represents.
     * @param urgent If true, this note represents a note that has an urgent state.
     * @exception <code>InvalidObjectIDException</code> if invalid object
     */
    public BCPostItNoteCIO( String name, String name_rb, long bean_id, LongValueSet oids,
                            int note_id, boolean urgent ) throws InvalidObjectIDException
    {
        super ( name, name_rb, bean_id, oids, false );
        noteID = note_id;
        noteDeleted = false;
        if ( urgent )
            setState( getState() | URGENT_NOTE );
    }

    /**
     * Get name of content info object (using given locale)
     * @param loc - locale of request
     * @return name
     */
    public String getName( Locale loc )
    {
        if ( nls_bundle_name != null )
        {
            try
            {
                MultiLocaleBundle mlb = MultiLocaleBundle.getSharedBundle(nls_bundle_name);
                String[] parms = new String[1];
                parms[0] = Integer.toString( noteID );
                return MessageFormat.format( mlb.getString(loc, ci_name), parms );
            }
            catch (MissingResourceException mrx)
            {
            }
        }
        return ci_name;
    }

    /**
     * Get the note ID
     * @return noteID
     */
    public int getNoteID()
    {
        return noteID;
    }

    /**
     * Mark this note as deleted
     */
    public void setNoteDeleted()
    {
        noteDeleted = true;
    }

    /**
     * Report whether or not this note has been deleted
     */
    public boolean isNoteDeleted()
    {
        return noteDeleted;
    }
}
