/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved.                            *
 *****************************************************************************/
package com.BobCo.postit;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;

import com.tivoli.twg.engine.SetClearMOStatusCmd;
import com.tivoli.twg.engine.TWGContentInfoObject;
import com.tivoli.twg.engine.TWGObject;
import com.tivoli.twg.engine.TWGTask;
import com.tivoli.twg.engine.TWGTaskActivation;
import com.tivoli.twg.engine.TWGTaskServer;
import com.tivoli.twg.engine.TWGRemoteTaskManager;
import com.tivoli.twg.engine.UpdateContentInfoCmd;
import com.tivoli.twg.libs.IntKeyTable;
import com.tivoli.twg.libs.IntelByteBuffer;
import com.tivoli.twg.libs.IntValueSet;
import com.tivoli.twg.libs.LongKeyTable;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.ServiceThread;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeException;
import com.tivoli.twg.libs.ServiceNodeClosedException;
import com.tivoli.twg.libs.BadServiceNodeImplException;
import com.tivoli.twg.libs.ServiceNodeLocalImplFactory;
import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandReceivedListener;
import com.tivoli.twg.log.TWGRas;

/*****************************************************************************
*                         Bob's Software House
*                    A Division of BobCo Enterprises
*****************************************************************************/
/****************************************************************************
* This is an example server task providing the tracking of the reminder
* notes that are attached to managed objects/groups.
* In this example, ....
*****************************************************************************/

public class BCPostItNotebook implements TWGTaskServer,
                                         BCPostItConstants,
                                         CommandReceivedListener
{
    // Instance of the one server task allowed.
    private static BCPostItNotebook postItServerTask;

    // Unique id to be assigned to the next note created.
    private int noteID;

    // Look up table containing all of the reminder notes created.
    private IntKeyTable postItNoteMsgs;

    // Look up table containing the mapping of reminder notes to object ids.
    private LongKeyTable postItNoteMaps;

    // Post It service node for communicating with server.
    private ServiceNode postItServiceNode = null;

    private boolean keepGoing = false;

    /**
     * Constructor for the Reminder Notes Server Task
     */
    public BCPostItNotebook()
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItNotebook.ctor() starting...");
        Exception except = null;

        // Only one server task allowed.  Cache this away for later access.
        postItServerTask = this;
        noteID = 1;
        postItNoteMsgs = new IntKeyTable();
        postItNoteMaps = new LongKeyTable();

        // Bring up the service node
        // Set ServiceNode support to use local IPC implementation
        ServiceNodeLocalImplFactory snlif = new ServiceNodeLocalImplFactory();
        ServiceNode.SetServiceNodeFactory(snlif);
        try
        {
            postItServiceNode = new ServiceNode( POST_IT_SERVER );
            postItServiceNode.RegisterDefaultCommandListener( postItServerTask ); // Catch any commands not handled
                                                                                  // by TWGRemoteTaskManager
            keepGoing = postItServiceNode.Create();
            if (keepGoing)
            {
                TWGRas.debug( TWGRas.EX1,  "BCPostItNotebook.ctor() Local service node created, name = "+postItServiceNode.Name());
            }
            else
            {
                TWGRas.debug( TWGRas.EX1,  "BCPostItNotebook.ctor() Local service node create failed.");
            }
        }
        catch (BadServiceNodeImplException snix)
        {
            except = snix;
        }
        catch (ServiceNodeClosedException sncx)
        {
            except = sncx;
        }
        catch (ServiceNodeException snx)
        {
            except = snx;
        }

        if (except != null)
        {
            TWGRas.error( TWGRas.EX1,  "BCPostItNotebook.ctor() Exception encountered while creating service node", except );
            except.printStackTrace();
        }

        TWGRas.exit( TWGRas.EX1,  "BCPostItNotebook.ctor() complete");
    }

    /**
     * This method is called first so that the server task can perform any
     * initialization prior to run() getting called.  If there are any other
     * threads needed besides the thread that calls run() they should be created
     * here.  Since the thread calling serverActivate() is dedicated to the
     * activation process, it is expected that control will not be returned from
     * this call until server task activation has completed.
     *
     * @return true if server activation completed successfully. or false if it
     *         failed
     */
    public boolean serverActivate(TWGTask task)
    {
        TWGRas.entryExit(TWGRas.EX1, "BCPostItNotebook.serverActivate");
        return true;
    }

    /**
     * This method is called when the server needs to deactivate this server
     * task.  The server task should perform any necessary clean up including
     * shutting down any additional threads it may have started and cause the
     * thread running in the run() method to return.
     */
    public void serverDeactivate() {
        TWGRas.entry(TWGRas.EX1, "BCPostItNotebook.serverDeactivate");
        try
        {
            if ( postItServiceNode != null )
            {
                keepGoing = false;
                postItServiceNode.DeregisterDefaultCommandListener();
                postItServiceNode.Close();
                synchronized (this) {
                   notifyAll();
                }
            }
        }
        catch (ServiceNodeException snx)
        {
            TWGRas.error( TWGRas.EX1, "BCPostItNotebook.serverDeactivate: Exception encountered while closing service node " + snx );
        }
        TWGRas.exit(TWGRas.EX1, "BCPostItNotebook.serverDeactivate");
    }

    /**
     * This method is called when a subtask activation has been received from
     * the console for this task.  The thread calling this method is responsible
     * for perfoming all subtaskActivate() calls, so control should not be kept
     * here for long.  If subtask activation requires waiting for an asynchronous
     * state change or event to occur, then this method should return indicating
     * that the activation is still pending and complete the subtask activation
     * later by calling act.getTask().subtaskActivateComplete().
     *
     * @param  act  the TWGTaskActivation object for the subtask being activated.
     * @return      Use one of the following final values from TWGTask:
     *              <ul>
     *              <li>SUBTASK_ACT_COMPLETE - completed successfully
     *              <li>SUBTASK_ACT_FAILED   - subtask activation failed
     *              <li>SUBTASK_ACT_PENDING  - subtask activation will complete
     *                                         asynchronously
     *              </ul>
     */
    public int subtaskActivate(TWGTaskActivation act)
    {
       TWGRas.entryExit(TWGRas.EX1, "BCPostIt.subtaskActivate");
       return TWGTask.SUBTASK_ACT_COMPLETE;
    }

    /**
     * This method is called when a subtask deactivation has been received from
     * the console for this task.
     */
    public void subtaskDeactivate(TWGTaskActivation act)
    {
       TWGRas.entryExit(TWGRas.EX1, "BCPostIt.subtaskDeactivate");
    }



    public void run()
    {
        // set the thread name to something more unique than 'main'
        TWGRas.entry(TWGRas.EX1, "BCPostItNotebook.run() starting...");
        Thread.currentThread().setName("PostItNoteMainThread");
        try 
        {
            TWGRas.debug(TWGRas.EX1, "BCPostItNotebook.run() processing commands...");
            postItServiceNode.ProcessCommands();
            synchronized (this) 
            {
               try 
               {
                   // This is where the task's run thread would go to work
                   wait();
               } 
               catch (InterruptedException ix) 
               {
                   TWGRas.debug(TWGRas.EX1, "BCPostItNotebook.run() InterruptedException encountered during wait." );
               }
            }
        }
        catch (InterruptedException x) 
        {
            TWGRas.error(TWGRas.EX1,  "BCPostItNotebook.run() InterruptedException", x);
            try 
            {
                postItServiceNode.DeregisterDefaultCommandListener();
                postItServiceNode.Close();
            } catch (ServiceNodeException snx) 
            {
                TWGRas.error(TWGRas.EX1,  "BCPostItNotebook.run() exception", snx);
            }
        } 
        catch (ServiceNodeException y) 
        {
            TWGRas.error(TWGRas.EX1,  "BCPostItNotebook.run() ServiceNodeException", y);
            try 
            {
                postItServiceNode.DeregisterDefaultCommandListener();
                postItServiceNode.Close();
            } 
            catch (ServiceNodeException snx2) 
            {
                TWGRas.error(TWGRas.EX1,  "BCPostItNotebook.run() exception", snx2);
            }
        }

        TWGRas.exit(TWGRas.EX1,  "BCPostItNotebook.run() exiting...");
    }

    /**
     * Command received callback.  This listener method is called when a
     * Command is received matching one of the command codes registered for
     * the given listener
     * @param sn - ServiceNode instance which received Command
     * @param cmd - Instance of Command received
     * @return true if ServiceNode should stay active, false if it should
     * terminate
     */
    public boolean CommandReceived( ServiceNode sn, Command cmd )
    {
        TWGRas.entry(TWGRas.EX1, "BCPostItNotebook.CommandReceived CommandCode() " + cmd.CommandCode() );

        // Check which command code came in and route to the correct one.
        switch ((int)cmd.CommandCode())
        {
            // If create new post it command, route it
            case (int)CREATE_POST_IT_NOTE_CMD:
                handleCreatePostItRequest(cmd);
                break;

                // If remove post it command, route it
            case (int)REMOVE_POST_IT_NOTE_CMD:
                handleRemovePostItRequest(cmd);
                break;

                // If get post it list command, route it
            case (int)RETRIEVE_POST_IT_LIST_CMD:
                handleGetPostItListRequest(cmd);
                break;

                // If get post its for object command, route it
            case (int)GET_POST_ITS_FOR_OBJECT_CMD:
                handleGetPostItNotesForObjectRequest(cmd);
                break;

                // If get post it note text command, route it
            case (int)GET_POST_IT_NOTE_TEXT_CMD:
                handleGetPostItNoteTextRequest(cmd);
                break;

            default:
                cmd.SetReturnCode( RC_UNSUPPORTED_CMD );
                break;
        }
        TWGRas.exit(TWGRas.EX1, "BCPostItNotebook.CommandReceived CommandCode() " + cmd.CommandCode() );

        return true;
    }

    /**
     * This method returns the postit service node defined in the server.
     *
     * @return post it service node reference.
     */
    public ServiceNode getPostItServiceNode()
    {
        TWGRas.entryExit(TWGRas.EX1, "BCPostItNotebook.getPostItServiceNode()" );
        return postItServiceNode;
    }

    /**
     * This method returns the postit service node defined in the server.
     *
     * @return post it service node reference.
     */
    public static BCPostItNotebook getPostItServerTask()
    {
        TWGRas.entryExit(TWGRas.EX1, "BCPostItNotebook.getPostItServerTask()" );
        return postItServerTask;
    }

    /**
     * Get the list of all object ids that have notes attached to them.
     *
     * @return Array of object ids that have notes attached.
     */
    public synchronized long[] getAllObjectsWithNotesAttached()
    {
        TWGRas.entryExit( TWGRas.EX1,  "BCPostItNotebook.getAllObjectsWithNotesAttached()" );
        if ( postItNoteMaps.Count() > 0 )
            return postItNoteMaps.GetAllKeys();
        else
            return null;
    }

    /**
     * Get the list of notes attached to a specific object id.
     *
     * @return Array of object ids that have notes attached.
     */
    public synchronized IntValueSet getAllNotesForObject( long oid )
    {
        TWGRas.entryExit( TWGRas.EX1,  "BCPostItNotebook.getAllNotesForObject(" + oid + ")" );
        return(IntValueSet)postItNoteMaps.FindObject( oid );
    }


    /**
     * Create a post it note for the objects specified.
     *
     * @param noteText The text of the note attached to the objects.
     * @param moids The managed object ids of the systems the note is attached to.
     */
    public synchronized int createPostItNote( String noteText, LongValueSet moids )
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItNotebook.createPostItNote("+noteText+", oids="+moids+")" );
        // First create the postit and add it to the listing.
        int thisNote = noteID++;
        TWGRas.debug( TWGRas.EX1,  "BCPostItNotebook.createPostItNote - insert " + thisNote + " with text " + noteText );
        postItNoteMsgs.InsertObject( thisNote, noteText );

        // Now attach this note to each object id in the id set passed.
        for (int i=0; i<moids.Length(); i++)
        {
            // Check if this oid already has a note attached to it (or not)
            IntValueSet notes = (IntValueSet)postItNoteMaps.FindObject( moids.GetValue(i) );
            // If no notes, then create a set to hold them and attach to this oid.
            if ( notes == null )
            {
                notes = new IntValueSet();
            }
            // Add this note to the set for this oid (either the old one or the new one).
            postItNoteMaps.InsertObject( moids.GetValue(i), notes );
            notes.InsertValue( thisNote );
        }
        // Tell handler to update itself by sending a command to the handler.
        UpdateContentInfoCmd cmd = new UpdateContentInfoCmd("tivoli.postit.sample");
        try
        {
            getPostItServiceNode().SendAsynchCommand(cmd);
        }
        catch (ServiceNodeException snx)
        {
            TWGRas.error( TWGRas.EX1, "BCPostItNotebook.UpdateContentInfoCmd error"+snx );
        }
        catch (InterruptedException ix)
        {
            TWGRas.error( TWGRas.EX1, "BCPostItNotebook.UpdateContentInfoCmd error"+ix );
        }

        TWGRas.exit( TWGRas.EX1,  "BCPostItNotebook.createPostItNote()" );
        return thisNote;
    }

    /**
     * Remove the post it note from the specified systems.
     *
     * @param noteID The id of the note to remove from the objects.
     * @param moids The object ids of the systems the note is removed from.
     */
    public synchronized void removePostItNote( int noteID, LongValueSet moids )
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItNotebook.removePostItNote( noteid=" + noteID + ", moids=" + moids + ")" );
        // Get the postIt Note object for the specified noteID
        TWGObject obj = TWGObject.FindObject( noteID );
        BCPostItNoteCIO ciobj = null;

        if( obj instanceof BCPostItNoteCIO )
        {
            // The remove was requested against a postit note 
            ciobj = (BCPostItNoteCIO)obj;
            if ( (ciobj != null) && (postItNoteMsgs.FindObject( ciobj.getNoteID() ) != null) )
            {
                // Remove it from each specified managed object id
                for (int i=0; i<moids.Length(); i++)
                {
                    removeNote( ciobj, moids.GetValue(i) );
                }

                // Finally, tell handler to update itself by sending a command to the handler.
                UpdateContentInfoCmd cmd = new UpdateContentInfoCmd( "tivoli.postit.sample" );
                try
                {
                    getPostItServiceNode().SendAsynchCommand(cmd);
                }
                catch (ServiceNodeException snx)
                {
                    TWGRas.error( TWGRas.EX1, "BCPostItNotebook.UpdateContentInfoCmd error", snx );
                }
                catch (InterruptedException ix)
                {
                    TWGRas.error( TWGRas.EX1, "BCPostItNotebook.UpdateContentInfoCmd error", ix );
                }
            }
        }
        TWGRas.exit( TWGRas.EX1,  "BCPostItNotebook.removePostItNote()" );
    }

    /**
     * Remove a single postit note.
     *
     * @param ciobj The note object to be removed from 
     *              the managed objects identified by moids.
     * @param moids The object ids of the systems from which 
     *              the note object is to be removed.
     */
    private void removeNote( BCPostItNoteCIO ciobj, long moid )
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItNotebook.removeNote() for moid " + moid );
        // Make sure this managed object has this note attached
        IntValueSet notes = (IntValueSet)postItNoteMaps.FindObject( moid );
        if ( notes != null )
        {
            // Remove the note from the managed object's set.
            TWGRas.debug( TWGRas.EX1,  "BCPostItNotebook.removeNote() notes is " + notes + " noteID to remove is " + ciobj.getNoteID() );
            int noteid = ciobj.getNoteID();
            notes.RemoveValue( noteid );

            // If this is the last note for this managed object, 
            // then remove the managed object from the postIt 
            // managed object tracking list.
            if ( notes.Length() == 0 )
                postItNoteMaps.RemoveObject( moid );
            LongValueSet oids = ciobj.getAssociatedObjectIDs();
            oids.RemoveValue( moid );
            ciobj.setAssociatedObjectIDs( oids );
        }
        // Check if there are any managed objects that still have this 
        // noteID attached.
        boolean found = false;
        Enumeration e = postItNoteMaps.GetObjects();
        while (e.hasMoreElements() && !found)
        {
            IntValueSet notelist = (IntValueSet)e.nextElement();
            if ( notelist.Find( ciobj.getNoteID() ) != -1 )
                found = true;
        }

        // If no moids have this note attached, then delete the note and
        // the object.
        if (!found)
        {
            TWGRas.debug( TWGRas.EX1,  "BCPostItNotebook.removeNote() deleting object" );
            postItNoteMsgs.RemoveObject( ciobj.getNoteID() );
            ciobj.setNoteDeleted();
        }
        TWGRas.exit( TWGRas.EX1,  "BCPostItNotebook.removeNote()" );
    }

    /**
     * Command handler for create post it note command
     * @param cmd - command requested
     */
    public void handleCreatePostItRequest( Command cmd )
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItNotebook.handleCreatePostItRequest()" );
        // Parm 0 = string text and longvalueset of association objects
        // Check if enough parameters
        if ( cmd.NumInputParms() < 1 )
        {
            cmd.SetReturnCode( RC_INVALID_PARAMETER );
            return;
        }

        String postItText;
        LongValueSet oids;

        try
        {
            // Process parameter 0
            IntelByteBuffer iBB = new IntelByteBuffer( cmd.InputParm(0) );
            postItText = iBB.ReadCompUnicode();
            oids = iBB.ReadLongValueSet();
        }
        catch ( ArrayIndexOutOfBoundsException oobx )
        {
            cmd.SetReturnCode( RC_INVALID_PARAMETER );
            return;
        }

        // Now that we have the command parameters, let create the note.
        long notID = (long)createPostItNote( postItText, oids );

        try
        {
            postItServiceNode.SendAsynchCommand(new SetClearMOStatusCmd(
                                                                       "postit.status",
                                                                       "postitStatus",
                                                                       oids,
                                                                       true,
                                                                       SetClearMOStatusCmd.STATUS_NONPERSISTENT_ONLY));
        } //endtry
        catch (ServiceNodeException snx)
        {
            TWGRas.error( TWGRas.EX1,  "BCPostItNotebook.handleCreatePostItRequest: Exception updating system status ", snx );
        }
        catch (InterruptedException ix)
        {
            TWGRas.error( TWGRas.EX1,  "BCPostItNotebook.handleCreatePostItRequest: Exception updating system status ", ix );
        }

        // Return successful, with object ID of note
        byte[] op0 = new byte[IntelByteBuffer.SIZEOF_LONG64];
        IntelByteBuffer.WriteLong64(op0, notID, 0);
        cmd.AttachOutputParm( op0 );
        cmd.SetReturnCode( RC_OKAY );
        TWGRas.exit( TWGRas.EX1,  "BCPostItNotebook.handleCreatePostItRequest()" );
    }

    /**
     * Command handler for remove post it note command
     * @param cmd - command requested
     */
    public void handleRemovePostItRequest( Command cmd )
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItNotebook.handleRemovePostItRequest()" );
        // Parm 0 = note id and longvalueset of objects to disassociate from
        // Check if enough parameters
        if ( cmd.NumInputParms() < 1 )
        {
            cmd.SetReturnCode( RC_INVALID_PARAMETER );
            return;
        }

        long notID;
        LongValueSet oids;

        try
        {
            // Process parameter 0
            IntelByteBuffer iBB = new IntelByteBuffer( cmd.InputParm(0) );
            notID = iBB.ReadLong64();
            oids = iBB.ReadLongValueSet();
        }
        catch ( ArrayIndexOutOfBoundsException oobx )
        {
            cmd.SetReturnCode( RC_INVALID_PARAMETER );
            return;
        }

        // Now that we have the command parameters, let create it.
        removePostItNote( (int)notID, oids );

        try
        {
            postItServiceNode.SendAsynchCommand( new SetClearMOStatusCmd( "postit.status",
                                                                          "postitStatus",
                                                                          oids,
                                                                          false,
                                                                          SetClearMOStatusCmd.STATUS_NONPERSISTENT_ONLY ) );
        } //endtry
        catch (ServiceNodeException e)
        {
            TWGRas.error( TWGRas.EX1,  "BCPostItNotebook.handleRemovePostItRequest: Exception updating system status ", e );
        }
        catch (InterruptedException e)
        {
            TWGRas.error( TWGRas.EX1,  "BCPostItNotebook.handleRemovePostItRequest: Exception updating system status ", e );
        }

        // Return successful
        cmd.SetReturnCode( RC_OKAY );
        TWGRas.exit( TWGRas.EX1,  "BCPostItNotebook.handleRemovePostItRequest()" );
    }

    /**
     * Command handler for retrieving the list of all object ids with notes attached.
     * @param cmd - command requested
     */
    public void handleGetPostItListRequest( Command cmd )
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItNotebook.handleGetPostItRequest()" );

        // Return successful, with LongValueSet of association ids
        long[] ids = getAllObjectsWithNotesAttached();
        LongValueSet oids = new LongValueSet();
        if ( ids != null )
            oids = new LongValueSet( ids, 0, ids.length );

        byte[] op0 = new byte[IntelByteBuffer.GetLongValueSetLength(oids)];
        IntelByteBuffer.WriteLongValueSet(op0, oids, 0);
        cmd.AttachOutputParm( op0 );
        cmd.SetReturnCode( RC_OKAY );

        TWGRas.exit( TWGRas.EX1,  "BCPostItNotebook.handleGetPostItRequest()" );
    }

    /**
     * Command handler for retrieving the list of all object ids with notes attached.
     * @param cmd - command requested
     */
    public void handleGetPostItNotesForObjectRequest( Command cmd )
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItNotebook.handleGetPostItNotesForObjectRequest()" );

        // Parm 0 = object id to get notes for.
        // Check if enough parameters
        if ( cmd.NumInputParms() < 1 )
        {
            cmd.SetReturnCode( RC_INVALID_PARAMETER );
            return;
        }

        long oid;
        try
        {
            // Process parameter 0
            IntelByteBuffer iBB = new IntelByteBuffer( cmd.InputParm(0) );
            oid = iBB.ReadLong64();
        }
        catch ( ArrayIndexOutOfBoundsException oobx )
        {
            cmd.SetReturnCode( RC_INVALID_PARAMETER );
            return;
        }

        // Return successful, with LongValueSet of association ids
        IntValueSet ivs = getAllNotesForObject( oid );
        if ( ivs == null )
            ivs = new IntValueSet();
        byte[] op0 = new byte[IntelByteBuffer.GetIntValueSetLength(ivs)];
        IntelByteBuffer.WriteIntValueSet(op0, ivs, 0);
        cmd.AttachOutputParm( op0 );

        TWGRas.exit( TWGRas.EX1,  "BCPostItNotebook.handleGetPostItNotesForObjectRequest()" );
    }

    /**
     * Command handler for retrieving the text for a specific post it note.
     * @param cmd - command requested
     */
    public void handleGetPostItNoteTextRequest( Command cmd )
    {
        TWGRas.entry( TWGRas.EX1,  "BCPostItNotebook.handleGetPostItNoteTextRequest()" );

        // Parm 0 = note id to get text for.
        // Check if enough parameters
        if ( cmd.NumInputParms() < 1 )
        {
            cmd.SetReturnCode( RC_INVALID_PARAMETER );
            return;
        }

        long coid;
        int nid;
        try
        {
            // Process parameter 0
            IntelByteBuffer iBB = new IntelByteBuffer( cmd.InputParm(0) );
            coid = iBB.ReadLong64();
        }
        catch ( ArrayIndexOutOfBoundsException oobx )
        {
            cmd.SetReturnCode( RC_INVALID_PARAMETER );
            return;
        }

        // Return successful, with string for note text.
        TWGObject obj = (BCPostItNoteCIO)TWGObject.FindObject( coid );
        if( obj instanceof BCPostItNoteCIO )
        {
            BCPostItNoteCIO ciobj = (BCPostItNoteCIO)obj;
            nid = ciobj.getNoteID();
            String text = (String)postItNoteMsgs.FindObject( nid );
            TWGRas.debug( TWGRas.EX1,  "BCPostItNotebook.handleGetPostItNoteTextRequest("+nid+"),text="+text);
            if ( text == null )
                text = "";
            byte[] op0 = new byte[IntelByteBuffer.GetCompUnicodeLength(text)];
            IntelByteBuffer.WriteCompUnicode(op0, text, 0);
            cmd.AttachOutputParm( op0 );
            cmd.SetReturnCode( RC_OKAY );
        }

        TWGRas.exit( TWGRas.EX1,  "BCPostItNotebook.handleGetPostItNoteTextRequest()" );
    }
}
