/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.BobCo.webobject;

import java.awt.*;
import java.net.*;
import java.io.IOException;

import com.tivoli.twg.console.TWGTaskFrame;
import com.tivoli.twg.console.TWGMainGUI;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.guilibs.*;
import com.tivoli.twg.log.TWGRas;

/**
* This is an example GUI implementation that will launch a browser to
* display the default URL for a web server managed object.  In an actual
* application, a developer could use information provided by the targeted
* system to determine which URL to bring up, or could query the user
* from his browser-based GUI to determine the target of the task.
*/
public class BCWebServerBrowserTaskGUI extends TWGTaskFrame
                                       implements CommandCompleteListener
{
  protected ServiceNode serviceNode;
  private TWGTaskActivator ivTaskActivator  = null;

  /**
  * This is the constructor for creating the task frame that will
  * allow the user to launch a browser gui from the main console.
  */
  public BCWebServerBrowserTaskGUI()
  {
    super(true);
    TWGRas.entryExit(TWGRas.EX7, "BCWebServerBrowserTaskGUI:  In constructor.");
  }

  /**
   * Called by the console when the server task activation for this instance of
   * the task frame has ended.  All termination processing should be done here,
   * and the main thread should be exited.
   *
   * @param endingType This indicates why the server task ended (i.e. completed
   * successfully, error occurred, etc.
   */
  public void serverTaskEnded( int endingType )
  {
    TWGRas.entryExit(TWGRas.EX7, "BCWebServerBrowserTaskGUI: in serverTaskEnded()");
  }


  /**
   * Start the console and display the browser.
   */
  public boolean pInit()
  {
    TWGRas.entryExit(TWGRas.EX7, "BCWebServerBrowserTaskGUI:  In pInit().");
    // First, stop status tracking.
    if ( ivTaskActivator != null )
      ivTaskActivator.resetGuiStatus();

    // Set up a service node so that we can send requests to the server
    try
    {
      serviceNode = new ServiceNode();
      //serviceNode.RegisterDefaultCommandListener(this);
      if (serviceNode.Create() == false)
        return false;
    }
    catch (BadServiceNodeImplException bsnie)
    {
      return false;
    }
    catch (ServiceNodeClosedException snce)
    {
      return false;
    }
    catch (ServiceNodeException sne)
    {
      return false;
    }

    // Get the object id we are launching the panel for.  Because this is
    // a single target, we just need the first (and only) id in the set.
    LongValueSet lvs = getTaskActivator().getMoid();
    long moid = lvs.GetValue( 0 );

    TWGRas.debug(TWGRas.EX7, "BCWebServerBrowserTaskGUI.pInit():  object moid is: " + moid);

    // Now get the object for that ID and get the name of the system.
    TWGConObject obj = TWGConObject.FindObject( moid );
    if ( obj instanceof TWGConManagedObject )
    {
      // Just output the name to show the data is available.  An application could
      // look at any of the data for the object to configure the GUI about to be
      // launched accordingly.
      String moName = ((TWGConManagedObject)obj).getName();
      TWGRas.debug(TWGRas.EX7,  "BCWebServerBrowserTask.pInit(): Launching browser for system: "
                   + moName );

      TWGGetAttributeCommand twggac = new TWGGetAttributeCommand(moid,
                                        new String(TWGManagedObjectConstants.ATTRIB_IP_ADDRS));

      twggac.setCommandCompleteListener(this);

      try
      {
        // use our own service node to send the command
        serviceNode.SendAsynchCommand(twggac);
      }
      catch ( ServiceNodeException snx )
      {
        TWGRas.error( TWGRas.EX7, "BCWebServerBrowserTask.pInit(): Can't get the task server service node", snx );
      }
      catch ( InterruptedException ix )
      {
        TWGRas.error( TWGRas.CONSOLE, "BCWebServerBrowserTask.pInit(): Error sending TWGGetAttributeCommand", ix );
      }
    }
    return false;
  }

  /**
   * Call back method for service node
   * @param cmd - the command that was processed
   */
  public void CommandComplete( Command cmd )
  {
    // If this was the get attribute command
    if ( cmd instanceof TWGGetAttributeCommand )
    {
      // and the command was successful.
      if ( ( cmd.ReturnCode() == 0 ) && ( cmd.NumOutputParms() > 0 ) )
      {
        // Check the attribute returned, which should be host name of the server.
        TWGGetAttributeCommand twggac = (TWGGetAttributeCommand)cmd;
        DataValue value = twggac.getAttributeValue( twggac.getObjectID( 0 ),
                                                   twggac.getAttributeID( 0 ) );

        // Check if we got a valid IP host name
        if ( ( value != null ) &&
             ( value.getDataValueType() == DataValue.STRING_ARRAY_TYPE ) &&
             ( ((TWGStringArray) value).getLength() > 0 ) )
        {
          String ipAddress = ((TWGStringArray) value).getValueAt(0);
          TWGRas.debug(TWGRas.EX7,  "BCWebServerBrowserTask.pInit(): Got the IP address: " + ipAddress);

          // Send a request to get the IP address
          try
          {
            BrowserLauncher.launchURL(new URL("http://" +  ipAddress));
          }
          catch (IOException ioe)
          {
            TWGRas.error(TWGRas.EX7,  "Exception launching URL:" + ioe );
          }
        }
        else
        {
          TWGRas.debug(TWGRas.EX7,  "BCWebServerBrowserTask.pInit(): No IP address");
        }
      }
      else
      {
        TWGRas.error(TWGRas.EX7,  "BCWebServerBrowserTask.CommandComplete(): rc or parms are bad");
      }
    }
    else
    {
      TWGRas.error(TWGRas.EX7,  "BCWebServerBrowserTask.CommandComplete(): we got the wrong command");
    }
  }

  /**
   * Called by the console when this instance of the task frame is being
   * cancelled.  All termination processing should be done here, and the main
   * thread should be exited.  Note this does not cancel the browser.
   *
   * @param cancelType This indicates which type of cancellation is occurring.
   */
  public void consoleCancel( int cancelType )
  {
    // Let the task activator know we are closing down shop.
    if ( ivTaskActivator != null )
      ivTaskActivator.guiExiting();

    ivTaskActivator = null;
  }
}
