/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.BobCo.webobject;

import com.tivoli.twg.engine.AddressEntryFormatException;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.TWGManagedObject;
import com.tivoli.twg.engine.TWGNativeAddressEntryShadow;
import com.tivoli.twg.libs.QueueElement;
import com.tivoli.twg.log.TWGRas;

import java.io.IOException;
import java.lang.SecurityException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;


/**
 * BCWebServerPingRequest implements the QueueElement interface.
 * It contains a ping message that is sent to a message queue,
 * and then acts on the message by processing the ping request.
 *
 * @see com.tivoli.twg.libs.QueueElement
 */
public class BCWebServerPingRequest implements QueueElement
{
   /* instance variables */
   private BCWebServerManagedObject webserver = null;

   /**
    * Constructor
    *
    * @param Socket client socket
    */
   public BCWebServerPingRequest( BCWebServerManagedObject webserver )
   {
     TWGRas.entryExit(TWGRas.EX7, "BCWebServerPingRequest:  constructor entry");
     this.webserver = webserver;
   }


   /**
    * Process queue element
    *
    * @param int queue type (not used)
    */
   public void process( int queueType )
   {
     String ipAddress = null;

     // this is where we do our processing.
     TWGRas.entryExit(TWGRas.EX7, "BCWebServerPingRequest.process(), name = "
                      + webserver.getName() + " queueType = " +
                      (queueType == QTYPE_BATCH? "batch" : "timer") );
     InetAddress address = null;
     ipAddress = webserver.getAddress();
     TWGRas.debug(TWGRas.EX7, "BCWebServerPingRequest.process():  ipAddress = "
                  + ipAddress);

     try
     {
       address = InetAddress.getByName(ipAddress);
     }
     catch (UnknownHostException uhe)
     {
       TWGRas.error(TWGRas.EX7,
                    "BCWebServerPingRequest.process(): Unknown host for address: "
                    + ipAddress);
       return;
     }

     try
     {
       // find out if the managed object is listening to port 80.
       // if so, assume it is a web server.
       Socket serverSocket = new Socket(address, 80);
       if (serverSocket != null)
       {
         // It looks like the web server is running, now check its state.  If
         // the web server thinks its state to be anything but online, change it.
         if (webserver.getState() != TWGManagedObjectConstants.MOSTATE_NORMAL_ONLINE)
         {
           webserver.setState(TWGManagedObjectConstants.MOSTATE_NORMAL_ONLINE);
         }
       }
       else
       {
         // It looks like the web server isn't running, so set the state to offline
         if (webserver.getState() != TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE)
         {
           webserver.setState(TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE);
         }
       }
     }
     catch (IOException ioe)
     {
       // this is the likely path we will take if nothing is listening on port 80
       TWGRas.debug(TWGRas.EX7,
                    "BCWebServerPingRequest.process(): Socket io exception - "
                    + ioe.toString());
       // set the state to offline if it's changed
       if (webserver.getState() != TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE)
       {
         webserver.setState(TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE);
       }
     }
     catch (SecurityException se)
     {
       TWGRas.debug(TWGRas.EX7,
                    "BCWebServerPingRequest.process(): Socket security exception - "
                    + se.toString());
       if (webserver.getState() != TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE)
       {
         // set the state to offline for now
         webserver.setState(TWGManagedObjectConstants.MOSTATE_NORMAL_OFFLINE);
       }
     }

     try
     {
       webserver.save();
     }
     catch (TWGPersistentObjectSaveException pos)
     {
       TWGRas.error(TWGRas.EX7,
                    "BCWebServerPingRequest.process(): TWGPersistentObjectSave exception.");
     }
   }
} /* class BCWebServerPingRequest */
