/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 2000. All Rights Reserved.                            *
 *****************************************************************************/
package com.BobCo.inventory;

import com.tivoli.twg.engine.*;
import java.util.Vector;

/**
 * <P>SQL database table for Keyboard information
 *
 * <p>Primary key - MANAGED_OBJ_ID
 *
 *
 * <p>
 * <table width=450 border=5 cols=6>
 * <!- Table Name>
 * <tr><td colspan=6 align=center><strong>BOBCO_DEVICES</strong></td>
 * <!- Table description >
 * <tr><td><strong>Column Name</strong></td>
 * <td><strong>Data Type</strong></td>
 * <td><strong>Description</strong></td>
 * <!- Table rows >
 * <tr><td>MANAGED_OBJ_ID</td>         <td>INT</td>       <td>Managed object id</td>
 * <tr><td>DEVICE_INDEX                </td><td>INT</td>  <td>Index for table key                       </td>
 * <tr><td>DEVICE_TYPE             </td><td>CHAR(10)</td> <td>Type - enumerated column                  </td>
 * <tr><td>DEVICE_DATE             </td><td>DATE</td>     <td>Type - time Date column                   </td>
 * <tr><td>DEVICE_CHIPSET             </td><td>ENUM</td> <td>Type - enumerated column                   </td>
 * <tr><td>DEVICE_SIZE             </td><td>DOUBLE</td>   <td>Type - double column                      </td>
 * </table>
 */

public class BobCoDeviceTable extends TWGDatabaseTable
{
   /* SQL Table Name */
   public static final String TABLE_NAME = "BOBCO_DEVICE";

   /* Table Columns */
   public static final String MANAGED_OBJ_ID          = "MANAGED_OBJ_ID";
   public static final String DEVICE_INDEX            = "DEVICE_INDEX";
   public static final String DEVICE_TYPE             = "DEVICE_TYPE";
   public static final String DEVICE_DATE             = "DEVICE_DATE";
   public static final String DEVICE_CHIPSET          = "DEVICE_CHIPSET"; 
   public static final String DEVICE_SIZE             = "DEVICE_SIZE"; 

   /* table version */
   public static final String TABLE_VERSION           = "1.0";


   /* A few defines for the enumerated Device Chipset type. */
   public static final String DEVICE_CHIPSET_UNKNOWN  = "UNKNOWN";
   public static final String DEVICE_CHIPSET_MOTOROLA = "MOTOROLA";
   public static final String DEVICE_CHIPSET_INTEL    = "INTEL";
   public static final String DEVICE_CHIPSET_IBM      = "IBM";
   public static final String DEVICE_CHIPSET_SANYO    = "SANYO";



   public BobCoDeviceTable()
   {
      super();
   }
   public void create() throws TWGDbException
   {

      //- Director Database name for the table
      String db = TWGDatabase.TWG_DB_NAME;
      String tableName = TABLE_NAME;
      String tableTokenName = TABLE_NAME;


      //----------------------------------------------------------------------
      //-Create the columns
      //----------------------------------------------------------------------
      TWGDbColumn column;
      TWGDbCharColumn charColumn;
      TWGDbIntegerColumn intColumn;
      // Object to hold the collection of columns
      TWGDbColumnData column_data = new TWGDbColumnData();

      // define the managed object id column specifying real and token column names
      intColumn = new TWGDbIntegerColumn(MANAGED_OBJ_ID,MANAGED_OBJ_ID);

      // managed object id column should not be a selectable column when creating
      // dynamic groups (filters)
      intColumn.setFilterColumn(false);

      // add the column to the column data
      column_data.addColumn(intColumn);

      // device index column
      intColumn = new TWGDbIntegerColumn(DEVICE_INDEX,DEVICE_INDEX);
      column_data.addColumn(intColumn);

      // device type column
      charColumn = new TWGDbCharColumn(DEVICE_TYPE,DEVICE_TYPE,10);
      column_data.addColumn(charColumn);

      // device Date column
      column = new TWGDbDatetimeColumn(DEVICE_DATE,DEVICE_DATE);
      column_data.addColumn(column);

      // device chipset column
      column = new TWGDbCharColumn(DEVICE_CHIPSET,DEVICE_CHIPSET,64);
      column.setDefaultValue(DEVICE_CHIPSET_UNKNOWN);
      ((TWGDbCharColumn)column).setMappedColumn(true);
      column_data.addColumn(column);

      // The device size column
      column = new TWGDbDoubleColumn(DEVICE_SIZE, DEVICE_SIZE);
      column_data.addColumn(column);

      //---------------------------------------------------------------------
      // parms object to hold optional parms (key, additional indices)
      //---------------------------------------------------------------------
      TWGDbTableParms parms = new TWGDbTableParms();

      // key fields are set in the ordered
      parms.setKey(MANAGED_OBJ_ID + "," + DEVICE_INDEX);

      // add an additional index on the device type column.  By default we
      // already have an index on the table key
      parms.addIndex("DEVICE_TYPE_IX", DEVICE_TYPE);


      //---------------------------------------------------------------------
      // NLS bundle for the table
      //---------------------------------------------------------------------
      String tableBundle = "com.BobCo.inventory.BobCoTableResources";


      //---------------------------------------------------------------------
      // Negotiate the SQL database name
      //---------------------------------------------------------------------
      int i = 1;
      String saveTableName      = tableName;
      String saveTableTokenName = tableTokenName;

      boolean tableNegotiationComplete = false;

      while (tableNegotiationComplete == false)
      {
         try
         {
            this.TWGInitTable(db, tableTokenName, tableName, column_data, parms,
                              tableBundle);

            tableNegotiationComplete = true;
         }
         catch (TWGDbRealTableNameNotUniqueException e)
         {
            tableName = saveTableName + String.valueOf(i);
            tableTokenName = saveTableTokenName + String.valueOf(i);
            i++;
         }
         catch (TWGDbTokenTableNameNotUniqueException e)
         {
            tableName = saveTableName + String.valueOf(i);
            tableTokenName = saveTableTokenName + String.valueOf(i);
            i++;
         }
      }
   }

   /*
   * Get the version number for the table
   *
   * @return the table version
   *
   */
   public String getTableVersion()
   {
      return TABLE_VERSION;
   }
}
