package com.BobCo.FileExplorerJVM;
import  com.tivoli.twg.engine.AddressEntryFormatException;
import  com.tivoli.twg.engine.TWGNativeAddressEntryShadow;
import  com.tivoli.twg.engine.TWGRemoteTaskActivation;
import  com.tivoli.twg.engine.TWGRemoteTaskClients;
import  com.tivoli.twg.engine.TWGRemoteManagedObject;
import  com.tivoli.twg.libs.Command;
import  com.tivoli.twg.libs.BadServiceNodeImplException;
import  com.tivoli.twg.libs.CommandReceivedListener;
import  com.tivoli.twg.libs.IntelByteBuffer;
import  com.tivoli.twg.libs.LongKeyTable;
import  com.tivoli.twg.libs.ServiceNode;
import  com.tivoli.twg.libs.ServiceNodeClosedException;
import  com.tivoli.twg.libs.ServiceNodeException;
import  com.tivoli.twg.libs.ServiceNodeRemoteImplFactory;
import  java.util.Enumeration;

/**
 * This utility class is used to maintain the additional information
 * needed for task activation processing beyond what is provided in
 * TWGRemoteTaskActivation.
 */
class BobCoJVMTaskActivation implements BobCoFileExplorerJVMConstants {
   static ServiceNode sn = null;
   private static LongKeyTable activations = new LongKeyTable();
   private static TWGNativeAddressEntryShadow nativeAddrShadow = new TWGNativeAddressEntryShadow();
   TWGRemoteTaskActivation act;
   TWGRemoteManagedObject  rmo;
   String consoleAddress = null;

   /**
    * Constructor
    * @param act  the TWGRemoteTaskActivation object for this activation.
    */
   BobCoJVMTaskActivation(TWGRemoteTaskActivation act) {
      System.out.println("BobCoJVMTaskActivation() contructor"
                        +" to log\\com.tivoli.twg.engine.TWGServerMain.output");
      this.act = act;
      // Even though the task is set up to be drag-drop and only accepting
      // a single target, we must use the Enumeration returned by getClients()
      // to extract the TWGRemoteManagedObject that is the target.
      Enumeration e = act.getClients().enumerateClients();
      rmo = (TWGRemoteManagedObject)e.nextElement();
      System.out.println("BobCoJVMTaskActivation() contructor\n act.getEngActID()="+ act.getEngActID()
	                                                    +" act.getMoid().GetValue(0)="+ act.getMoid().GetValue(0));
      activations.InsertObject(act.getEngActID(), this);
   }

   /**
    * Find a BobCoJVMTaskActivation based on the activation ID
    * @param actID   the activation ID to find
    */
   public static final BobCoJVMTaskActivation findActivation(long actID) {
      System.out.println("BobCoJVMTaskActivation.findActivation() actID="+ actID);
      return (BobCoJVMTaskActivation)activations.FindObject(actID);
   }

   /**
    * Delete a BobCoJVMTaskActivation from the collection of active activations.
    * @param actID   the activation ID to delete
    */
   public static final void deleteActivation(long actID) {
      System.out.println("BobCoJVMTaskActivation.deleteActivation()");
      BobCoJVMTaskActivation myAct = (BobCoJVMTaskActivation)activations.RemoveObject(actID);
      if (myAct != null) {
         myAct.act = null;
      }
   }

   /**
    * Set the service node to use to communicate with the client.
    * @param SN a ServiceNode
    */
   public static void setServiceNode(ServiceNode SN) {
      System.out.println("BobCoJVMTaskActivation.setServiceNode()"
                        +" to log\\com.tivoli.twg.engine.TWGServerMain.output");
      sn = SN;
   }

   /**
    * Get the activation ID of this object.
    * @return long the activation ID
    */
   public final long getActID() {
      return act.getEngActID();
   }

   /**
    * A utility method to do a SendAsynch to the client in this activation.
    * @param  cmd     the Command to send to the client
    * @return boolean true if no errors were encountered during the SendAsynch
    */
   public final boolean sendAsynchToClient(Command cmd) {
      boolean rc = true;
      if (sn != null) {
         byte[] addrEntry = rmo.getAddressEntryRecord();
         if (addrEntry != null) {
            try {
               nativeAddrShadow.initAddressEntry(addrEntry, 0, addrEntry.length);
               String ipcPath = nativeAddrShadow.getIPCPath();
               System.out.println("Sending command, ipcPath="+ipcPath);
               cmd.SetDestinationAddress(ipcPath+clientAddressSuffix);
               try {
                  sn.SendAsynchCommand(cmd);
               } catch (ServiceNodeClosedException sncx) {
                  rc = false;
                  System.out.println("ServiceNodeClosedException while sending command.");
               } catch (ServiceNodeException snx) {
                  rc = false;
                  System.out.println("ServiceNodeException while sending command.");
               } catch (InterruptedException ix) {
                  rc = false;
                  System.out.println("InterruptedException while sending command.");
               }
            } catch (AddressEntryFormatException aefx) {
               System.out.println("Caught AddressEntryFormatException");
            }
         } else {
            System.out.println("Client's address entry is null.");
            rc = false;
         }
      } else {
         System.out.println("Service node has not been initialized.");
         rc = false;
      }
      return rc;
   }

   /**
    * A utility function to send a postponed reply back to the console.
    * @param  cmd     the Command to send the postponed reply for
    * @return boolean true if no errors were encountered during
    *                 SendPostponedReply()
    */
   public final boolean sendPostponedReply(Command cmd) {
      System.out.println("BobCoJVMTaskActivation.sendPostponedReply()");
      boolean rc = true;
      try {
         rc = sn.SendPostponedReply(cmd);
      } catch (ServiceNodeClosedException sncx) {
         rc = false;
         System.out.println("ServiceNodeClosedException while sending postponed reply to console.");
      } catch (ServiceNodeException snx) {
         rc = false;
         System.out.println("ServiceNodeException while sending postponed reply to console.");
      }
      return rc;
   }
}
