package com.ibm.sysmgt.sdk.CmdTask;

import com.tivoli.twg.libs.*;
import com.tivoli.twg.guilibs.*;
import java.io.IOException;
import java.io.*;
import java.util.*;
import com.tivoli.twg.log.TWGRas;


/**
 * This class is a simple extension of JTree that sets up the basic
 * display properties of the file explorer tree.  It also contains
 * an inner class that provides the tree cell renderer.
 */
public class CmdLauncher  {

    private String unixCmdLine = null;
    private String windowsCmdLine = null;

    private boolean needsShell = true;
    private Hashtable envTable = null;
    Vector env = null;
    //private String[] environment = null;
    private File cwdFile = null;

    private String urlType = null;
    private String urlPort = null;
    private String urlFile = null;

   /**
    * Constructor
    * @param gui   the GUI creating this tree
    * @param moid  the ID of the managed object to explore
    */
   public CmdLauncher() {
       /* Do this ONLY if a CmdLine gets set, otherwise not needed
       if (TWGEnvironment.isWindows()) {
           buildWindowsEnvironment();

       } else {
           buildUnixEnvironment();
       }
       */
   }

   public void setUnixCommand(String c) {
       unixCmdLine = c;
       buildUnixEnvironment();

   }
   public void setWindowsCommand(String c) {
       windowsCmdLine = c;
       buildWindowsEnvironment();
   }
   public void setShell(boolean b) {
       needsShell = b;
   }

   public void setEnvironment(Hashtable ht){
       envTable = ht;
   }

   public void setUrl(String type, String port, String file){
       urlType = type;
       urlPort = port;
       urlFile = file;
   }

   public void setCwd(String c){
       if(c != null)
           cwdFile = new File(c);
       else
           cwdFile = null;
   }

   public String getUnixCommand() {
       return unixCmdLine;
   }

   public String getWindowsCommand() {
       return windowsCmdLine;
   }


   public boolean launch() {
       boolean launchSuccessful = true;

       // Console Cmd
       if(windowsCmdLine != null || unixCmdLine != null) {
           if (!needsShell) {
               if (TWGEnvironment.isWindows()) {
                   launchSuccessful &= launchWindows();

               } else {
                   launchSuccessful &= launchUnix();
               }
           } else {
               if (TWGEnvironment.isWindows()) {
                   launchSuccessful &= launchWindowsShell();

               } else {
                   launchSuccessful &= launchUnixShell();
               }
           }
       }
       //<wait on Agent return (if possibly a console Daemon for URL cmd) >


       // URL Cmd
       if(urlType != null || urlPort != null || urlFile != null){
           //try{

               for(Enumeration keys = envTable.keys(); keys.hasMoreElements(); ){
                   String key = (String) keys.nextElement();
                   if(key.startsWith("CMDTASK_IP_ADDRESS")){
                       String tempIpAddress = (String) envTable.get(key);
                       String urlString =   (urlType != null ? urlType + "://" : "") + tempIpAddress +
                                            // The slash after the URL port is now part of the
                                            // file name.  This is to get around a problem where
                                            // the java URL class doesn't seem to automatically
                                            // add the forward slash in its constructor.
                                            //(urlPort != null ? ":" + urlPort : "") + "/" +
                                            (urlPort != null ? ":" + urlPort : "") +
                                            (urlFile != null ? urlFile : "");
                       TWGRas.debug(TWGRas.EX0,"CmdTask.CmdLauncher.launch urlFile = " + urlFile );
                       TWGRas.debug(TWGRas.EX0,"CmdTask.CmdLauncher.launch BrowserLauncher.launchURL( " + urlString + " )");
                       //System.out.println("CmdTask.CmdLauncher.launch BrowserLauncher.launchURL( " + urlString + " )");
                       BrowserLauncher bl = new BrowserLauncher();
                       try{
                           //for Compatibility with Director 3.1 BrowserLauncher
                           // There appears to be a problem with the way Java handles the
                           // following URL creation.  It omits the "/" between the port
                           // number and the URL file name.
                           bl.launchURL(new java.net.URL(urlType,tempIpAddress,Integer.parseInt(urlPort),urlFile));
                           //bl.launchURL(new java.net.URL(urlString));
                           //bl.launchURL(urlString);
                       } catch (IOException ioe) {
                           TWGRas.debug(TWGRas.EX0,"CmdTask.CmdLauncher.launch IOException while BrowserLauncher.launchURL( " + urlString + " ) : " + ioe.getMessage());
                           launchSuccessful = false;
                       }
                   }
               }
           //} catch (Exception e){
           //    TWGRas.debug(TWGRas.EX0,"Exception encountered while trying to use BrowserLauncher from CmdTask.CmdLauncher.launch() :"
           //                     + e.getMessage());
           //}
       }
       return launchSuccessful;
   }

   private boolean launchWindowsShell() {
       TWGRas.debug(TWGRas.EX0,"CmdTask:CmdLauncher.launchWindowsShell - Launching " + windowsCmdLine + " in Windows shell");
       if (windowsCmdLine != null) {
           String[] environment = augmentEnvironment();

           try{
               Runtime.getRuntime().exec("cmd /c start cmd.exe /k " + windowsCmdLine, environment, cwdFile);
               //Runtime.getRuntime().exec("start cmd.exe /k " + windowsCmdLine /*, null, new File("C:\\")*/);
           } catch (SecurityException se) {
               // if a security manager exists and its checkExec method doesn't allow creation of a subprocess.
               TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.launchWindowsShell SecurityException : " + se.getMessage());
               return false;
           } catch (IOException ioe) {
               // if an I/O error occurs
               TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.launchWindowsShell IOException : " + ioe.getMessage());
               return false;
           }
           return true;
       } else return false;
   }

   private boolean launchUnixShell() {
       TWGRas.debug(TWGRas.EX0,"CmdTask:CmdLauncher.launchUnixShell - Launching " + unixCmdLine + " in UNIX shell");
       if (unixCmdLine != null) {
           String[] environment = augmentEnvironment();

           try{
               Runtime.getRuntime().exec("bash " + unixCmdLine, environment, cwdFile);
           } catch (SecurityException se) {
               // if a security manager exists and its checkExec method doesn't allow creation of a subprocess.
               TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.launchUnixShell SecurityException : " + se.getMessage());
               return false;
           } catch (IOException ioe) {
               // if an I/O error occurs
               TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.launchUnixShell IOException : " + ioe.getMessage());
               return false;
           }
           return true;
       } else return false;
   }

   private boolean launchWindows() {
       TWGRas.debug(TWGRas.EX0,"CmdTask:CmdLauncher.launchWindows - Launching " + windowsCmdLine);
       if (windowsCmdLine != null) {
           String[] environment = augmentEnvironment();

           try{
               Runtime.getRuntime().exec("cmd /c " + windowsCmdLine, environment, cwdFile);
           } catch (SecurityException se) {
               // if a security manager exists and its checkExec method doesn't allow creation of a subprocess.
               TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.launchWindows SecurityException : " + se.getMessage());
               return false;
           } catch (IOException ioe) {
               // if an I/O error occurs
               TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.launchWindows IOException : " + ioe.getMessage());
               return false;
           }
           return true;
       } else return false;
   }

   private boolean launchUnix() {
       TWGRas.debug(TWGRas.EX0,"CmdTask:CmdLauncher.launchUnix - Launching " + unixCmdLine);
           if (unixCmdLine != null) {
               String[] environment = augmentEnvironment();

               try{
                   Runtime.getRuntime().exec(unixCmdLine, environment, cwdFile);
               } catch (SecurityException se) {
                   // if a security manager exists and its checkExec method doesn't allow creation of a subprocess.
                   TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.launchUnix SecurityException : " + se.getMessage());
                   return false;
               } catch (IOException ioe) {
                   // if an I/O error occurs
                   TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.launchUnix IOException : " + ioe.getMessage());
                   return false;
               }
               return true;
           } else return false;
   }

   private void buildWindowsEnvironment(){
       // Build Vector for CmdTask environment
       env = new Vector();

       // Add current local console environment variables
       try{
           Process p = Runtime.getRuntime().exec("cmd /c set");
           BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
           String line = null;
           while( (line = in.readLine()) != null){
               if (line.indexOf('=') > 0) {
                   env.add(line);
               }
           }
           in.close();
           in = null;
           p = null;

           /*Properties props = new Properties();
           props.load(p.getInputStream());
           for(Enumeration e = props.keys(); e.hasMoreElements(); ){
               String key = (String) e.nextElement();           // Doesn't work... flattens Windows File Separator ('\')
               env.add(new String(key + "=" + props.get(key)));
           }*/

       } catch (SecurityException se) {
           // if a security manager exists and its checkExec method doesn't allow creation of a subprocess.
           TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.buildWindowsEnvironment SecurityException : " + se.getMessage());
       } catch (IOException ioe) {
           // if an I/O error occurs
           TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.buildWindowsEnvironment IOException : " + ioe.getMessage());
       }
   }

   private void buildUnixEnvironment(){
       // Build Vector for CmdTask environment
       env = new Vector();

       // Add current local console environment variables
       try{
           Process p = Runtime.getRuntime().exec("env");
           BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));
           String line = null;
           while( (line = in.readLine()) != null){
               TWGRas.debug(TWGRas.EX0,line);
               if (line.indexOf('=') > 0) {
                   env.add(line);
               }
           }
           in.close();
           in = null;
           p = null;

       } catch (SecurityException se) {
           // if a security manager exists and its checkExec method doesn't allow creation of a subprocess.
           TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.buildWindowsEnvironment SecurityException : " + se.getMessage());
       } catch (IOException ioe) {
           // if an I/O error occurs
           TWGRas.debug(TWGRas.EX0,"CmdTask::CmdLauncher.buildWindowsEnvironment IOException : " + ioe.getMessage());
       }
   }

   private String[] augmentEnvironment(){
       // Add CMDTASK_* Environment variables
       Vector augmentedEnv = (Vector) env.clone();
       String key = null;
       for(Enumeration keys = envTable.keys(); keys.hasMoreElements(); ) {
           key = (String) keys.nextElement();
           augmentedEnv.addElement(new String(key + "=" + envTable.get(key)));
       }
       String[] environment = (String[])augmentedEnv.toArray(new String[0]);
       return environment;
   }


   public static void main(String argv[])
   {
       CmdLauncher cl = new CmdLauncher();
       Hashtable ht = new Hashtable();
       ht.put("CMDTASK_AUGMENT","Sample CMDTASK variable added to Environment");
       ht.put("CMDTASK_IP_ADDRESS","www.ibm.com");
       ht.put("CMDTASK_IP_ADDRESS0","www.google.com");
       cl.setUrl("http",null,null);
       cl.setEnvironment(ht);
       if(argv.length>0){
           cl.setWindowsCommand(argv[0]);
           cl.setUnixCommand(argv[0]);

           if(argv.length<2 || !argv[1].equalsIgnoreCase("0") &&
              !argv[1].equalsIgnoreCase("false") &&
              !argv[1].equalsIgnoreCase("no"))
               cl.setShell(true);
           else
               cl.setShell(false);

           cl.launch();
       } else {
           TWGRas.debug(TWGRas.EX0,"java CmdLauncher CommandString <shellRequired>");
       }
   }
}
