/////////////////////////////////////////////////////////
// Module		FindFrame.java
// Version		3.10
// Language		Java
// Author		Scott Riley
// History
// 06/09/00    Release 3.10
// 10/26/99    Initial release
//
// Copyright (C) 1999-2000 Tivoli Systems, an IBM Company  
//
////////////////////////////////////////////////////////

// First import all the standard packages that we need
import java.awt.*;
import java.awt.event.*;

class FindFrame extends Frame implements ActionListener
{
	// Data Members
	SDKFind sdkFind;

	// The three panels inside this Frame.
	SearchPanel searchPanel;
	OptionsPanel optionsPanel;
	MatchesPanel resultPanel;

	// The number of the page being currently searched
	// This should be lesser than sdkFind.totalPages
	int currentPageNumber; 

   //  Define an inner class to listen for only one window event
   class MyAdapter extends WindowAdapter implements WindowListener {
      public void WindowClosing (WindowEvent evt) {
         setVisible(false);
			dispose();
      }
   }

	// Constructor
	FindFrame(SDKFind cs, Frame frame, String title)
	{
		super(title);
		sdkFind = cs;

		// File menu and its elements
		Menu fileMenu = new Menu("File");
		fileMenu.addSeparator();
		fileMenu.add("Close");

		// Help menu and its elements
		Menu helpMenu = new Menu("Help");
		helpMenu.add("Help on searching...");
		helpMenu.addSeparator();
		helpMenu.add("About...");

		// The menu bar at the top
		MenuBar menuBar = new MenuBar();
		menuBar.add(fileMenu);
		menuBar.add(helpMenu);
		menuBar.setHelpMenu(helpMenu);
      fileMenu.addActionListener(this);
      helpMenu.addActionListener(this);
		
		// Get the bg and fg colors from the applet and 
		// set the bg and fg of this frame to be the same 
		setBackground(cs.getBackground());
		setForeground(cs.getForeground());

		// Add the menubar and its menu items to this frame
		setMenuBar(menuBar);

		// Create a BoxLayout 
		// and start adding the panels one by one
      setLayout(new BorderLayout());

      Panel searchContainer = new Panel(new BorderLayout(0,0));
      add( "North",searchContainer);


		// Create an instance of searchPanel and add it
		searchPanel = new SearchPanel(this);
		searchContainer.add("North",searchPanel);
		
		// Constraints of optionsPanel nearly the same as those of searchPanel
		// Create an instance of optionsPanel and add it
		optionsPanel = new OptionsPanel(this);
		searchContainer.add("South", optionsPanel);

		// Create an instance of a MatchesPanel and add it
		resultPanel = new MatchesPanel(this);
		add("Center",resultPanel);
      // Establish an adapter for window actions.
      WindowListener adapter = new MyAdapter();
      addWindowListener(adapter);
      this.doLayout();
	}
	
	// Start painting the chrome
	public void paint(Graphics g)
	{
		Dimension d = getSize();
		g.setColor(new Color(0,0,255));	// Blue

		// Draw a nice 3-D box around the applet
		g.draw3DRect(0,0,d.width-2,d.height-2,true);
		g.draw3DRect(2,2,d.width-5,d.height-5,false);
	}
	
	// The padding for this frame
 /*  public Insets getInsets()
	{
		return new Insets(3,3,3,3);
	}   */

    // Event Handler
	// If 'Close' selected, set this frame to invisible.
	// If 'Help on searching...' selected, create the helpDialog.
	// If 'About' selected, create the aboutDialog.
	// The three panels will take care of their own events.
   public void actionPerformed(ActionEvent evt) {
      String what = evt.getActionCommand();
      if (what.equals("Close")) {
         setVisible(false);
      } else if (what.equals("Help on searching...")) {
         HelpDialog helpDialog = new HelpDialog(this);
      } else if (what.equals("About...")) {
         AboutDialog aboutDialog = new AboutDialog(this);
      }
   }

	public void processEvent(AWTEvent evt)  {
      if(evt.getID() == WindowEvent.WINDOW_CLOSING){
         setVisible(false);
      }
   }
} 
