
package com.ibm.sysmgt.sdk.CmdTask;

import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.IntelByteBuffer;
import java.io.ObjectStreamException;
import java.io.NotSerializableException;
import com.tivoli.twg.log.TWGRas;


/**
 * Command class used to send events to Director Server.
 *
 * @version 1.00 1997-12-05
 */

public class GetCmdStringCommand extends Command

  { /* class GetCmdStringCommand */

  public static final int Windows = 1;
  public static final int Unix = 2;

  private String storeTaskID = null;
  private Integer storePlatform = null;

  /** Command to get commandstring. */
  public static final int CMDTASK_GET_COMMANDSTRING =     0x0C088A7D;

  /**
   * Constructor.
   *
   * @param event TWGBaseEvent object to be sent.
   */


    

  public GetCmdStringCommand( Command cmd ) {
      // PROVIDER
      super(cmd);
      TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand PROVIDER constructor ");
  }

  public GetCmdStringCommand( ) {
      // REQUESTER

      super(CMDTASK_GET_COMMANDSTRING);
      SetDestinationAddress( CmdTaskConstants.CMDSERVERNAME );
      storeTaskID 	 = new String();
      storePlatform 	 = new Integer(-1);
      setResetInputParms();
  }

  public GetCmdStringCommand( String taskID ) {
    // REQUESTER

      super(CMDTASK_GET_COMMANDSTRING);
      SetDestinationAddress( CmdTaskConstants.CMDSERVERNAME );
      storeTaskID 	 = taskID;
      storePlatform 	 = new Integer(-1);
      setResetInputParms();
  }

  public GetCmdStringCommand( String taskID, int platform ) { 
    // REQUESTER

      super(CMDTASK_GET_COMMANDSTRING);
      SetDestinationAddress( CmdTaskConstants.CMDSERVERNAME );
      storeTaskID 	 = taskID;
      storePlatform 	 = new Integer(platform);
      setResetInputParms();
  }

  public void setTaskID (String taskID){
      // REQUESTER

      storeTaskID = taskID;
      setResetInputParms();
  }

  public void setPlatform (int platform){
      // REQUESTER

      storePlatform = new Integer(platform);
      setResetInputParms();
  }

  private void setResetInputParms(){
      ResetInput();
      SetCommandCode(CMDTASK_GET_COMMANDSTRING);
      SetDestinationAddress( CmdTaskConstants.CMDSERVERNAME );
      try{
	  AddObjectAsInputParm(storeTaskID);
	  AddObjectAsInputParm(storePlatform);
      } catch (NotSerializableException nse) {
	  TWGRas.debug(TWGRas.EX0,"GetCmdStringCommand.setResetInputParms NotSerializableException : " + nse.getMessage());
      }
  }

  
  
  public String getCommandString() {
        // REQUESTER
        
        String c = null;
	try{
	    c = (String) this.GetObjectFromOutputParm(0);
	    c = c.equals("") ? null : c;
	} catch (ObjectStreamException ose){
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getCommandString ObjectStreamException : " + ose.getMessage()); 
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getCommandString ClassNotFoundException : " + cnfe.getMessage()); 
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getCommandString : " + c);
	return c;
    }

  public boolean getShellRequired() {
      // REQUESTER
      
      boolean b = false;
      try{
	b = ( (Boolean) this.GetObjectFromOutputParm(1) ).booleanValue();
      } catch (ObjectStreamException ose){
	  TWGRas.debug(TWGRas.EX0,"GetCmdStringCommand.getShellRequired ObjectStreamException : " + ose.getMessage());
      } catch (ClassNotFoundException cnfe) {
	  TWGRas.debug(TWGRas.EX0,"GetCmdStringCommand ClassNotFoundException : " + cnfe.getMessage()); 
      }
      TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getShellRequired : " + b);
      return b;
  }

  public String getCwd() {
      // REQUESTER

      String c = null;
	try{
	    c = (String) this.GetObjectFromOutputParm(2);
	    c = c.equals("") ? null : c;
	} catch (ObjectStreamException ose){
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getCwd ObjectStreamException : " + ose.getMessage()); 
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getCwd ClassNotFoundException : " + cnfe.getMessage()); 
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getCwd : " + c);
	return c;
  }

  public String getUrlType(){
    // REQUESTER

	String c = null;
	try{
	    c = (String) this.GetObjectFromOutputParm(3);
	    c = c.equals("") ? null : c;
	} catch (ObjectStreamException ose){
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getUrlType ObjectStreamException : " + ose.getMessage()); 
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.UrlType ClassNotFoundException : " + cnfe.getMessage()); 
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getUrlType : " + c);
	return c;
  }

  public String getUrlPort(){
  // REQUESTER
      
      String c = null;
	try{
	    c = (String) this.GetObjectFromOutputParm(4);
	    c = c.equals("") ? null : c;
	} catch (ObjectStreamException ose){
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getUrlPort ObjectStreamException : " + ose.getMessage()); 
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getUrlPort ClassNotFoundException : " + cnfe.getMessage()); 
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getUrlPort : " + c);
	return c;
  }

  public String getUrlFile(){
  // REQUESTER

      String c = null;
	try{
	    c = (String) this.GetObjectFromOutputParm(5);
	    c = c.equals("") ? null : c;
	} catch (ObjectStreamException ose){
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getUrlFile ObjectStreamException : " + ose.getMessage()); 
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getUrlFile ClassNotFoundException : " + cnfe.getMessage()); 
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getUrlFile : " + c);
	return c;
  }

  public String getAgentCommandStringWindows() {
        // REQUESTER
        
        String c = null;
	try{
	    c = (String) this.GetObjectFromOutputParm(6);
	    c = c.equals("") ? null : c;
	} catch (ObjectStreamException ose){
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCommandStringWindows ObjectStreamException : " + ose.getMessage()); 
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCommandStringWindows ClassNotFoundException : " + cnfe.getMessage()); 
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCommandStringWindows : " + c);
	return c;
    }

  public String getAgentCommandStringUnix() {
      // REQUESTER
        
        String c = null;
	try{
	    c = (String) this.GetObjectFromOutputParm(7);
	    c = c.equals("") ? null : c;
	} catch (ObjectStreamException ose){
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCommandStringUnix ObjectStreamException : " + ose.getMessage()); 
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCommandStringUnix ClassNotFoundException : " + cnfe.getMessage()); 
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCommandStringUnix : " + c);
	return c;
  }

  public int getAgentTimeout() {
      // REQUESTER
      
      int i = Integer.MAX_VALUE;
      try{
	i = ( (Integer) this.GetObjectFromOutputParm(8) ).intValue();
      } catch (ObjectStreamException ose){
	  TWGRas.debug(TWGRas.EX0,"GetCmdStringCommand.getAgentTimeout ObjectStreamException : " + ose.getMessage());
      } catch (ClassNotFoundException cnfe) {
	  TWGRas.debug(TWGRas.EX0,"GetCmdStringCommand ClassNotFoundException : " + cnfe.getMessage()); 
      }
      TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentTimeout : " + i);
      return i;
  }
   
  public String getAgentCwdWindows() {
      // REQUESTER
      
      String c = null;
	try{
	    c = (String) this.GetObjectFromOutputParm(9);
	    c = c.equals("") ? null : c;
	} catch (ObjectStreamException ose){
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCwdWindows ObjectStreamException : " + ose.getMessage()); 
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCwdWindows ClassNotFoundException : " + cnfe.getMessage()); 
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCwdWindows : " + c);
	return c;
  }

  public String getAgentCwdUnix() {
      // REQUESTER
      
      String c = null;
	try{
	    c = (String) this.GetObjectFromOutputParm(10);
	    c = c.equals("") ? null : c;
	} catch (ObjectStreamException ose){
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCwdUnix ObjectStreamException : " + ose.getMessage()); 
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCwdUnix ClassNotFoundException : " + cnfe.getMessage()); 
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getAgentCwdUnix : " + c);
	return c;
  }


  public static void setCommandString(Command cmd, String commandString, boolean shellRequired, String cwd){
      // PROVIDER
      
	  TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand SETTING Command output parms to : " + commandString + "," + shellRequired + "," + cwd);
	  try{
	      cmd.AddObjectAsOutputParm(commandString);			// 0
	      cmd.AddObjectAsOutputParm(new Boolean(shellRequired));	// 1
	      cmd.AddObjectAsOutputParm(cwd);				// 2
	  } catch (NotSerializableException nse) {
	      TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.setCommandString NotSerializableException : " + nse.getMessage());
	  }
      

  }

  
  public static void setUrlString(Command cmd, String type, String port, String file){
      // PROVIDER
      
	  TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand SETTING URL output parms to : " + type + "," + port + "," + file);
	  try{
	      cmd.AddObjectAsOutputParm(type);				// 3
	      cmd.AddObjectAsOutputParm(port);				// 4
	      cmd.AddObjectAsOutputParm(file);				// 5
	      
	  } catch (NotSerializableException nse) {
	      TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.setUrlString NotSerializableException : " + nse.getMessage());
	  }
      
  }

  public static void setAgentCommandStrings(Command cmd, String commandStringWindows, String commandStringUnix,
					     int timeout, String windowsAgentCwd, String unixAgentCwd){
      // PROVIDER
       
	  TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand SETTING AgentCommand output parms to : " + commandStringWindows + 
			     "," + commandStringUnix + "," + timeout + "," + windowsAgentCwd + "," + unixAgentCwd);
	  try{
	      cmd.AddObjectAsOutputParm(commandStringWindows);		// 6
	      cmd.AddObjectAsOutputParm(commandStringUnix);		// 7
	      cmd.AddObjectAsOutputParm(new Integer(timeout));		// 8
	      cmd.AddObjectAsOutputParm(windowsAgentCwd);		// 9
	      cmd.AddObjectAsOutputParm(unixAgentCwd);			// 10
	  } catch (NotSerializableException nse) {
	      TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.setAgentCommandString NotSerializableException : " + nse.getMessage());
	  }
       
  }

  public String getTaskID() {
    // PROVIDER

	String c = null;
    
	try{
	    c = (String) GetObjectFromInputParm(0);
	} catch (ObjectStreamException ose) {
	    TWGRas.debug(TWGRas.EX0, "ObjectStreamException in GetCmdStringCommand.getTaskID : " +
				ose.getMessage() );
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0, "ClassNotFoundException in GetCmdStringCommand.getTaskID : " +
				cnfe.getMessage() );
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getTaskID : " + c);
	return c;
      }

  public int getPlatform() {
    // PROVIDER

	int i = 1;
	try{
	    i =  ((Integer) GetObjectFromInputParm(1)).intValue();
	} catch (ObjectStreamException ose) {
	    TWGRas.debug(TWGRas.EX0, "ObjectStreamException in GetCmdStringCommand.getPlatform : " +
				ose.getMessage() );
	} catch (ClassNotFoundException cnfe) {
	    TWGRas.debug(TWGRas.EX0, "ClassNotFoundException in GetCmdStringCommand.getPlatform : " +
				cnfe.getMessage() );
	}
	TWGRas.debug(TWGRas.EX0,"CmdTask::GetCmdStringCommand.getPlatform : " + i);
	return i;
  }

} /* class GetCmdStringCommand */

