/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.communication;

import com.tivoli.twg.libs.Command;
import com.tivoli.twg.libs.CommandCompleteListener;

/**
 * LogMsgCommand is the derived from the Command class for log message
 * commands.  Log message commands are sent to the msglogr program.
 *
 * @link com.tivoli.twg.libs.Command
 */
public class LogMsgCommand extends Command {

   /** Command code for log message commands. */
   static final int LOG_MSG        = 0x5001;
   static final int LOG_MSG_ASYNCH = 0x5002;

   /**
    * This constructor calls parent constructor with the log message
    * command code value.
    */
   LogMsgCommand() {

      super(LOG_MSG);
   }

   /**
    * Constructor which sets the command code value and the command complete
    * listener which allows the class calling this constructor to override
    * the CommandComplete method in this class.
    */
   LogMsgCommand(CommandCompleteListener ccl) {

      super(LOG_MSG, ccl);
   }

   /**
    * Processes the CommandComplete for this command.  Over-rides the
    * CommandComplete method of the Command class to display a success or
    * failure message based on completion status.
    */
   public void CommandComplete() {

      // Zero return code means success.
      if (ReturnCode() == 0) {
         System.out.println("Req: Message logged");
      }
      else {
         System.out.println("Req: Message log request failed.  RC = "+
                            Long.toHexString(ReturnCode()));
      }
   }
}
