/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/

package com.BobCo.snapins;

import java.awt.*;
import java.beans.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.cim.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.guilibs.*;
import com.tivoli.twg.console.TWGConsoleColor;

/**
 * The LogicalDiskSnapIn class is a sample CIM property browser.
 * It implements the TWGCIMPropertiesBrowserInterface and is
 * registered to display instances of Win32_LogicalDisk.
 *
 * @version 1.1 - 6/8/05
 */
public class LogicalDiskSnapIn implements TWGCIMPropertiesBrowserInterface,
                                          Runnable
{

   /* Private data */
   private ImagePanel              image_panel      = null;
   private JScrollPane             scroll_pane      = null;
   private JPanel                  panel            = null;
   private int                     status           = TWGCIMConstants.UNKNOWN;
   private String                  statusText       = null;
   private aCIMClass               cimClass         = null;
   private aCIMInstance            cimInstance      = null;
   private WindowAttributes        windowAttributes = null;
   private PropertyChangeSupport   prop             = null;
   private ListResourceBundle      bundle           = null;
   private ImageIcon               removableIcon    = null;
   private ImageIcon               fixedIcon        = null;
   private ImageIcon               cdromIcon        = null;
   private ImageIcon               networkIcon      = null;
   private ImageIcon               icon;
   private JLabel                  icon_panel;
   private JLabel                  label_text;
   private JLabel                  label_value;
   private JLabel                  type_text;
   private JLabel                  type_value;
   private JLabel                  filesys_text;
   private JLabel                  filesys_value;
   private JPanel                  used_space_color;
   private JLabel                  used_space_text;
   private JLabel                  used_space_bytes;
   private JLabel                  used_space_gb;
   private JPanel                  free_space_color;
   private JLabel                  free_space_text;
   private JLabel                  free_space_bytes;
   private JLabel                  free_space_gb;
   private JLabel                  separator;
   private JLabel                  capacity_text;
   private JLabel                  capacity_bytes;
   private JLabel                  capacity_gb;
   private PieChart                chart;
   private JLabel                  drive;
   private boolean                 view_built = false;
   private int                     drive_type;
   private long                    used_space;
   private long                    free_space;
   private long                    capacity;
   private String                  file_sys;
   private String                  volume_label;
   private String                  name;
   private LogicalDiskResponse     response = null;
   private static final String     removableIconPath = "com/BobCo/snapins/images/remdsk16.gif";
   private static final String     fixedIconPath     = "com/BobCo/snapins/images/fixdsk16.gif";
   private static final String     cdromIconPath     = "com/BobCo/snapins/images/cdrom16.gif";
   private static final String     networkIconPath   = "com/BobCo/snapins/images/netdsk16.gif";

   /**
    * Constructor.  The snap-in interface requires a default
    * constructor that does not have any parameters.
    */
   public LogicalDiskSnapIn ()
   {
      /* Create a Java Bean property change support object */
      /* to manage the property change listeners           */
      prop = new PropertyChangeSupport( this );
   }

   /**
    * The getDisplayWindow method is used to return a JPanel 
    * that will be displayed by the caller.  The snap-in is  
    * responsible for the contents and appearance of the panel.
    */
   public JPanel getDisplayWindow ()
   {
      /* If we have already created a panel, just re-use it*/
      if (image_panel == null)
      {
         /* Load the NLS resource bundle for display strings */
         if (bundle == null)
         {
            bundle = (ListResourceBundle)ListResourceBundle.getBundle
                     ( "com.BobCo.snapins.BobCoSnapInsResources",
                     Locale.getDefault() );
         }

         /* Create the panel with a Border layout. (Note that  */
         /* we actual create an ImagePanel which is a subclass */
         /* of JPanel).                                        */
         image_panel = new ImagePanel();
         image_panel.setLayout(new BorderLayout());

         /* Apply any window attributes */
         if (windowAttributes != null)
         {
            JComponent panels[] = {image_panel};
            ImageLabel labels[] = {};
            JComponent others[] = {};
            windowAttributes.applyColors(panels, labels, others);
            image_panel.invalidate();
            image_panel.validate();
            image_panel.repaint();
         }
      }

      /* Return the panel */
      return image_panel;
   }

   /**
    * Retrieve the current status of the snap-in.  Status values 
    * should be one of the constants defined in TWGCIMConstants.
    */
   public int getStatus()
   {
      return status;
   }

   /**
    * Set the current status of the snap-in.  Status values 
    * should be one of the constants defined in TWGCIMConstants.
    * After changing the status, any property change listeners
    * are notified of the change.
    */
   public void setStatus( int newStatus )
   {
      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* Take actions based on the new status */
      switch ( newStatus )
      {
         /* If we are closing, then remove all references to    */
         /* our data objects so that they are garbage collected */
         /* by the Java run-time.                               */
         case TWGCIMConstants.CLOSE:
            if (response != null)
            {
               /* If we are in the middle of a CIM response, then */
               /* cancel the response processing.                 */
               response.cancel();
               response = null;
            }
            panel = null;
            bundle = null;
            statusText = null;
            cimInstance = null;
            windowAttributes = null;
            break;
            
         /* If we are in-progress, then update the status text */
         case TWGCIMConstants.IN_PROGRESS:
            setStatusText(bundle.getString("ReadingDisk"));
            break;
            
         /* Otherwise we must be ready, so update the status text */
         default:
            setStatusText(bundle.getString("Ready"));
            break;
      }
      
      /* Notify any property change listeners */
      int oldStatus = status;
      status = newStatus;
      if (newStatus != TWGCIMConstants.CLOSE)
      {
         prop.firePropertyChange( "statusChange",
                                  new Integer(oldStatus),
                                  new Integer(newStatus));
      }
   }

   /**
    * Retrieve a text string that describes the current status
    * of the snap-in.  This string may be used by the parent of
    * the snap-in to update the status area at the bottom of
    * the window.
    */
   public String getStatusText()
   {
      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* If we have never set the status text, then assume */
      /* we are ready                                      */
      if ( statusText == null )
      {
         statusText = bundle.getString("Ready");
      }
      
      /* Return the latest status text string */
      return statusText;
   }

   /**
    * Set the current status text message for the snap-in.
    */
   public void setStatusText( String newStatusText )
   {
      statusText = newStatusText;
   }

   /**
    * Retrieve the CIM instance object that is being displayed 
    * by the snap-in. 
    */
   public aCIMInstance getCIMInstance()
   {
      return cimInstance;
   }

   /**
    * Set the CIM instance that the snap-in will display.
    * Calling this method will start the process of getting
    * the instance data and populating the display.
    */
   public void setCIMInstance( aCIMClass newCIMClass, aCIMInstance newCIMInstance )
   {
      /* Save the class and instance objects */
      cimClass = newCIMClass;
      cimInstance = newCIMInstance;

      /* Be sure that we have a display window to update */
      getDisplayWindow();

      /* Build new sub-components of our display window */
      buildView();

      /* Build a CIM object to get the property data that is */
      /* needed for the instance.                            */
      CIMAccessor     acc  = new CIMAccessor(((CIMInstance)cimInstance).GetMOID());
      CIMProperty[]   prop = new CIMProperty[6];
      prop[0] = new CIMProperty(cimInstance.GetNameSpace(),"DriveType");
      prop[1] = new CIMProperty(cimInstance.GetNameSpace(),"FileSystem");
      prop[2] = new CIMProperty(cimInstance.GetNameSpace(),"FreeSpace");
      prop[3] = new CIMProperty(cimInstance.GetNameSpace(),"Name");
      prop[4] = new CIMProperty(cimInstance.GetNameSpace(),"VolumeName");
      prop[5] = new CIMProperty(cimInstance.GetNameSpace(),"Size");
      CIMAccessPair[] ap   = new CIMAccessPair[1];
      ap[0] = new CIMAccessPair(cimInstance,prop);

      /* Reset the return data area */
      Object[] val = null;

      /* If a previous CIM enumeration response is still */
      /* in progress, then we need to cancel it.         */
      if (response != null)
      {
         response.cancel();
      }

      /* Set the status to in-progress */
      setStatus(TWGCIMConstants.IN_PROGRESS);

      /* Create a new response object */
      response = new LogicalDiskResponse(cimInstance);

      /* Try to get the property values */
      if (!acc.GetValue(ap,response))
      {
         /* If we had an error, then just cancel the operation */
         /* and set the status to ready                        */
         response = null;
         setStatus(TWGCIMConstants.READY);
      }
   }

   /**
    * This method is called by the parent of the snap-in to
    * update the window attributes so that they match the
    * rest of the current user console.
    */
   public void setWindowAttributes( WindowAttributes wa )
   {
      /* Save the new attributes */
      windowAttributes = wa;

      /* Only appy the attributes if we have an image_panel */
      if (image_panel != null)
      {
         /* Apply the new attributes to our window */
         JComponent panels[] = {image_panel};
         ImageLabel labels[] = {};
         JComponent others[] = {};
         windowAttributes.applyColors(panels, labels, others);

         /* Repaint the window */
         image_panel.invalidate();
         image_panel.validate();
         image_panel.repaint();
      }
   }

   /**
    * Retrieve the current window attributes 
    */
   public WindowAttributes getWindowAttributes()
   {
      return windowAttributes;
   }

   /**
    * Add a property change listener to the snap-in.
    */
   public void addPropertyChangeListener( PropertyChangeListener listener )
   {
      prop.addPropertyChangeListener( listener );
   }

   /**
    * Remove a property change listener from the snap-in.
    */
   public void removePropertyChangeListener( PropertyChangeListener listener )
   {
      prop.removePropertyChangeListener( listener );
   }

   /**
    * This method is used to get an icon image to display for
    * a CIM instance.
    */
   public ImageIcon getIconForCIMInstance ( aCIMInstance inst )
   {
      /* Create a response object */
      LogicalDiskResponse resp = new LogicalDiskResponse(inst);

      /* Get the drive type using our internal "synchronous" */
      /* implementation.                                     */
      Object[] val = resp.getPropertyValueArray("DriveType");
      Integer drive_type = new Integer(3);
      if ( val[0] instanceof Integer )
      {
         drive_type = (Integer) val[0];
      }

      /* Return the correct icon for the drive type */
      return getIconForDriveType(drive_type.intValue());
   }

   /**
    * This internal method is used to get an icon image for
    * a particular drive type
    */
   private ImageIcon getIconForDriveType ( int dt )
   {
      /* Return the correct icon for the drive type */
      switch (dt)
      {
         case 2: /* Removable Media */
            if (removableIcon == null)
            {
               removableIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(removableIconPath));
            }
            return removableIcon;
         case 3: /* Fixed Disk */
            if (fixedIcon == null)
            {
               fixedIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(fixedIconPath));
            }
            return fixedIcon;
         case 4: /* Network Drive */
            if (networkIcon == null)
            {
               networkIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(networkIconPath));
            }
            return networkIcon;
         case 5: /* CD-ROM */
            if (cdromIcon == null)
            {
               cdromIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(cdromIconPath));
            }
            return cdromIcon;
      }

      /* If it was not a recognized drive type, then return */
      /* the icon for a fixed disk.                         */
      if (fixedIcon == null)
      {
         fixedIcon = new ImageIcon(TWGRemoteSnapInManager.getImage(fixedIconPath));
      }
      return fixedIcon;
   }

   /**
    * This internal method is used to build the sub-components
    * of the display window.
    */
   private void buildView()
   {
      /* If we previously built a view, then remove all of the */
      /* old subcomponents (let them be garbage collected)     */
      if (view_built)
      {
         image_panel.removeAll();
      }

      /* Load the NLS resource bundle for display strings */
      if (bundle == null)
      {
         bundle = (ListResourceBundle)ListResourceBundle.getBundle
                  ( "com.BobCo.snapins.BobCoSnapInsResources",
                  Locale.getDefault() );
      }

      /* Create sub-components */
      panel = new JPanel();
      panel.setLayout(new GridBagLayout());
      scroll_pane = new JScrollPane();
      scroll_pane.setHorizontalScrollBarPolicy
                  (ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      scroll_pane.setVerticalScrollBarPolicy
                  (ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
      icon_panel = new JLabel();
      label_text = new JLabel(bundle.getString("Label"));
      label_value = new JLabel();
      type_text = new JLabel(bundle.getString("Type"));
      type_value = new JLabel();
      filesys_text = new JLabel(bundle.getString("FileSystem"));
      filesys_value = new JLabel();
      used_space_color = new JPanel();
      used_space_color.setBorder(new BevelBorder(BevelBorder.LOWERED));
      used_space_color.setBackground(TWGConsoleColor.getBlueColor());
      used_space_text = new JLabel(bundle.getString("UsedSpace"));
      used_space_bytes = new JLabel();
      used_space_bytes.setHorizontalAlignment(SwingConstants.RIGHT);
      used_space_gb = new JLabel();
      used_space_gb.setHorizontalAlignment(SwingConstants.RIGHT);
      free_space_color = new JPanel();
      free_space_color.setBorder(new BevelBorder(BevelBorder.LOWERED));
      free_space_color.setBackground(TWGConsoleColor.getMagentaColor());
      free_space_text = new JLabel(bundle.getString("FreeSpace"));
      free_space_bytes = new JLabel();
      free_space_bytes.setHorizontalAlignment(SwingConstants.RIGHT);
      free_space_gb = new JLabel();
      free_space_gb.setHorizontalAlignment(SwingConstants.RIGHT);
      separator = new JLabel();
      separator.setBorder(new BevelBorder(BevelBorder.LOWERED));
      capacity_text = new JLabel(bundle.getString("Capacity"));
      capacity_bytes = new JLabel();
      capacity_bytes.setHorizontalAlignment(SwingConstants.RIGHT);
      capacity_gb = new JLabel();
      capacity_gb.setHorizontalAlignment(SwingConstants.RIGHT);
      drive = new JLabel();
      drive.setHorizontalAlignment(SwingConstants.CENTER);

      /* Set flag to indicate the view has been built */
      view_built = true;
   }

   /**
    * This internal method is called to update the view
    * with data from the instance attributes.
    */
   private void updateView ()
   {
      /* Get the locale specific number format */
      NumberFormat nf = NumberFormat.getInstance();

      /* Get the icon for this instance */
      icon_panel.setIcon(getIconForDriveType(drive_type));

      /* Set the volume label */
      label_value.setText(volume_label);

      /* Set the drive type (local or network) */
      switch (drive_type)
      {
         case 4:
            type_value.setText(bundle.getString("NetworkDisk"));
            break;
         default:
            type_value.setText(bundle.getString("LocalDisk"));
            break;
      }

      /* Set the file system */
      filesys_value.setText(file_sys);

      /* Set the used space (in bytes) */
      used_space_bytes.setText(nf.format(used_space)+" "+bundle.getString("Bytes"));

      /* Set the free space (in bytes) */
      free_space_bytes.setText(nf.format(free_space)+" "+bundle.getString("Bytes"));

      /* Set the capacity (in bytes) */
      capacity_bytes.setText(nf.format(capacity)+" "+bundle.getString("Bytes"));

      /* Change the number format to be limited to 2 decimal places */
      if (nf instanceof DecimalFormat)
      {
        ((DecimalFormat)nf).applyPattern("0.00");
      }

      /* Set the used space (in MB or GB) */
      double tmp;
      tmp = used_space;
      tmp = tmp / 1048576L;
      if (tmp > 1024)
      {
         tmp = tmp / 1024L;
         used_space_gb.setText(nf.format(tmp)+" "+bundle.getString("GB"));
      }
      else
      {
         used_space_gb.setText(nf.format(tmp)+" "+bundle.getString("MB"));
      }

      /* Set the free space (in MB or GB) */
      tmp = free_space;
      tmp = tmp / 1048576L;
      if (tmp > 1024)
      {
         tmp = tmp / 1024L;
         free_space_gb.setText(nf.format(tmp)+" "+bundle.getString("GB"));
      }
      else
      {
         free_space_gb.setText(nf.format(tmp)+" "+bundle.getString("MB"));
      }

      /* Set the capacity (in MB or GB) */
      tmp = capacity;
      tmp = tmp / 1048576L;
      if (tmp > 1024)
      {
         tmp = tmp / 1024L;
         capacity_gb.setText(nf.format(tmp)+" "+bundle.getString("GB"));
      }
      else
      {
         capacity_gb.setText(nf.format(tmp)+" "+bundle.getString("MB"));
      }

      /* Set the drive name */
      drive.setText(bundle.getString("Drive")+" "+name);

      /* Create a pie chart */
      long[] v = new long[2];
      v[0] = used_space;
      v[1] = free_space;
      Color[] c = new Color[2];
      c[0] = TWGConsoleColor.getBlueColor();
      c[1] = TWGConsoleColor.getMagentaColor();
      chart = new PieChart(v,c);

      /* Add all of the view constructs to the display */
      Insets i = new Insets(0,5,0,5);
      makeGridBagLayoutItem (panel, new JPanel(),    0,0, 1,1,1,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    6,0, 1,1,1,0,i);
      makeGridBagLayoutItem (panel, icon_panel,      1,1, 2,3,0,0,i);
      makeGridBagLayoutItem (panel, label_text,      3,1, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, label_value,     4,1, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,2, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, type_text,       3,3, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, type_value,      4,3, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,4, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, filesys_text,    3,5, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, filesys_value,   4,5, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,6, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,7, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, used_space_color,2,8, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, used_space_text, 3,8, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, used_space_bytes,4,8, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, used_space_gb,   5,8, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,9, 1,1,0,0,i);
      makeGridBagLayoutItem (panel, free_space_color,2,10,1,1,0,0,i);
      makeGridBagLayoutItem (panel, free_space_text, 3,10,1,1,0,0,i);
      makeGridBagLayoutItem (panel, free_space_bytes,4,10,1,1,0,0,i);
      makeGridBagLayoutItem (panel, free_space_gb,   5,10,1,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,11,1,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,12,1,1,0,0,i);
      makeGridBagLayoutItem (panel, separator,       0,13,7,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,14,1,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,15,1,1,0,0,i);
      makeGridBagLayoutItem (panel, capacity_text,   3,16,1,1,0,0,i);
      makeGridBagLayoutItem (panel, capacity_bytes,  4,16,1,1,0,0,i);
      makeGridBagLayoutItem (panel, capacity_gb,     5,16,1,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,17,1,1,0,0,i);
      makeGridBagLayoutItem (panel, chart,           3,18,3,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,19,1,1,0,0,i);
      makeGridBagLayoutItem (panel, drive,           0,20,7,1,0,0,i);
      makeGridBagLayoutItem (panel, new JPanel(),    0,21,1,1,0,1,i);
      panel.setSize(panel.getPreferredSize());
      scroll_pane.setViewportView(panel);
      image_panel.add(scroll_pane,BorderLayout.CENTER);

      /* Repaint the window */
      image_panel.invalidate();
      image_panel.validate();
      image_panel.repaint();
   }

   /**
    * This is an internal method used to simplify the usage
    * of GridBagLayout.
    */
   private void makeGridBagLayoutItem (Container cont,
                                       Component comp,
                                       int       x,
                                       int       y,
                                       int       w,
                                       int       h,
                                       double    weightx,
                                       double    weighty,
                                       Insets    i)
   {
     GridBagLayout gbl = (GridBagLayout)cont.getLayout();
     GridBagConstraints c = new GridBagConstraints();
     c.anchor     = GridBagConstraints.NORTHWEST;
     c.fill       = GridBagConstraints.BOTH;
     c.gridx      = x;
     c.gridy      = y;
     c.gridwidth  = w;
     c.gridheight = h;
     c.weightx    = weightx;
     c.weighty    = weighty;
     if (i != null)
     {
        c.insets = i;
     }
     cont.add(comp);
     gbl.setConstraints(comp, c);
   }

   /**
    * This internal method is called from the response object
    * when the property data has been returned. 
    */
   private void showProperties ( aCIMProperty[] p )
   {
      /* Clear the response (since we don't need it again) */
      response = null;

      /* Set default values */
      drive_type   = 0;
      used_space   = 0L;
      free_space   = 0L;
      capacity     = 0L;
      file_sys     = new String();
      volume_label = new String();
      name         = new String();

      Object   obj;
      String   tmp;

      /* Get the returned drive type */
      if (p[0].GetRC())
      {
         obj = p[0].GetValueElement(0);
         if (obj instanceof Integer)
         {
            drive_type = ((Integer)obj).intValue();
         }
      }

      /* Get the returned file system */
      if (p[1].GetRC())
      {
         obj = p[1].GetValueElement(0);
         if (obj instanceof String)
         {
            file_sys = (String)obj;
         }
      }

      /* Get the returned free space */
      if (p[2].GetRC())
      {
         obj = p[2].GetValueElement(0);
         if (obj instanceof Long)
         {
            free_space = ((Long)obj).longValue();
         }
      }
      
      /* Get the returned drive name */
      if (p[3].GetRC())
      {
         obj = p[3].GetValueElement(0);
         if (obj instanceof String)
         {
            name = (String) obj;
         }
      }

      /* Get the returned volume label */
      if (p[4].GetRC())
      {
         obj = p[4].GetValueElement(0);
         if (obj instanceof String)
         {
            volume_label = (String) obj;
         }
      }

      /* Get the returned capacity */
      if (p[5].GetRC())
      {
         obj = p[5].GetValueElement(0);
         if (obj instanceof Long)
         {
            capacity = ((Long)obj).longValue();
         }
      }

      /* Calculate the used space */
      used_space = capacity - free_space;
      if (used_space < 0)
      { 
         used_space = 0;
      }

      /* Check for empty volume label */
      if (volume_label.length()==0)
      {
         volume_label = bundle.getString("NoLabel");
      }

      /* Update the display from the event dispatch thread */
      try
      {
         SwingUtilities.invokeLater(this);
      }
      catch (Exception x)
      {
      }
   }

   /* Method invoked on the event dispatch thread to update */
   /* the display after the properties have been retrieved. */
   public void run ()
   {
      updateView();
   }

   /**
    * LogicalDiskResponse is an internal class used to query
    * properites for an instance.
    */
   class LogicalDiskResponse implements iCIMAccessorResponse
   {

      aCIMInstance    inst;   
      aCIMProperty[]  p;
      boolean         synch     = false;
      boolean         cancelled = false;

      /*
       * Constructor.
       */
      public LogicalDiskResponse ( aCIMInstance i )
      {
         inst = i;
      }

      /**
       * This method is called when the requested property 
       * values have been returned.
       */
      public synchronized void GetResponse(iCIMAccessor Caller,aCIMAccessPair[] Values)
      {
         /* Check if we have been cancelled while we were */
         /* waiting                                       */
         if (!cancelled)
         {
            /* Get the properties */
            p = Values[0].GetProperty();

            /* If this is the result of our synchronous  */
            /* query to get the drive type, then wake    */
            /* up the caller.  Otherwise, it must be the */
            /* result of querying all the properties, so */
            /* update the display.                       */
            if (synch)
            {
               notify();
            }
            else
            {
               showProperties(p);
               /* Set the status to ready after updating */
               /* the display                            */
               setStatus(TWGCIMConstants.READY);
            }
         }
      }

      /**
       * This callback is called when an error occurs during the
       * response processing.
       */
      public synchronized void AccessorError(iCIMAccessor Caller,aCIMError Error)
      {
         /* If the error occurred as the result of our synchronous  */
         /* query to get the drive type, then wake up the caller.   */
         /* Otherwise, it must be the result of querying all the    */
         /* properties, so just set the status to ready.            */
         if (synch)
         {
            p = null;
            notify();
            synch = false;
         }
         else
         {
            setStatus(TWGCIMConstants.READY);
         }
      }

      /**
       * This callback is not used in this sample snap-in.
       */
      public void SetResponse(iCIMAccessor Caller,aCIMAccessPair[] Values)
      {
      }

      /**
       * This internal method is used to get a single property in
       * a synchronous fashion (instead of the normal asynchronous
       * operation).
       */
      public synchronized Object[] getPropertyValueArray (String propertyName)
      {

         /* Build CIM objects to get the property */
         CIMAccessor acc  = new CIMAccessor(((CIMInstance)inst).GetMOID());
         CIMProperty prop = new CIMProperty(inst.GetNameSpace(),propertyName);

         /* Set our synchronous flag */
         synch = true;

         /* Try to get the property */
         Object[] val = null;
         if (acc.GetValue(inst,prop,this))
         {
            try
            {
               /* Wait for the data to be ready */
               wait();

               /* If everything worked ok, then get the result */
               if ((p != null) && (p[0] != null) && p[0].GetRC())
               {
                  val = p[0].GetValueArray();
               }
            }
            catch (InterruptedException ie) 
            {
            }
         }

         /* Return the property value */
         return val;
      }

      /**
       * This method is used to cancel a response that is in-progress.
       */
      public synchronized void cancel ()
      {
         cancelled = true;
      }
   }
}
