package com.tivoli.twg.mscs;

import com.tivoli.twg.engine.*;
import java.util.Vector;

/**
 * <P>SQL database table for MSCS Cluster Info Table information
 *
 * <p>Primary key - MANAGED_OBJ_ID
 *
 * <p>
 * <table width=450 border=5 cols=14>
 * <!- Table Name>
 * <tr><td colspan=14 align=center><strong>MSCS_GROUPS</strong></td>
 * <!- Table description >
 * <tr><td><strong>Field Name</strong></td>
 * <td><strong>Data Type</strong></td>
 * <td><strong>Description</strong></td>
 * <td><strong>NT/2000</strong></td>
 * <td><strong>95/98</strong></td>
 * <td><strong>OS/2</strong></td>
 * <td><strong>Netware</strong></td>
 * <td><strong>AS/400</strong></td>
 * <td><strong>SNMP</strong></td>
 * <td><strong>AIX</strong></td>
 * <td><strong>HPUX</strong></td>
 * <td><strong>Solaris</strong></td>
 * <td><strong>Unixware</strong></td>
 * <td><strong>Linux</strong></td>
 * <!- Table rows >
 * <tr><td>MANAGED_OBJ_ID</td>         <td>INT</td>       <td>Managed object id (of cluster)</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * <tr><td>MSCS_NAME</td>              <td>CHAR(64)</td>  <td>Cluster Name</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * <tr><td>MSCS_MAJ_VER</td>           <td>INT</td>       <td>Cluster Major Version</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * <tr><td>MSCS_MIN_VER</td>           <td>INT</td>       <td>Cluster Minor Version</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * <tr><td>MSCS_BUILD</td>             <td>INT</td>       <td>Cluster Build Number</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * <tr><td>MSCS_VENDOR</td>            <td>CHAR(64)</td>  <td>Cluster Vendor</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * <tr><td>MSCS_CSD</td>               <td>CHAR(64)</td>  <td>Cluster CSD</td>
 *     <td>X</td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> <td> </td> 
 * </table>
 */
public class MSCSClusterInfoTable extends TWGDatabaseTable {
    /* SQL Table Name                                                                       */
    public static final String TABLE_NAME = "MSCS_CLUS_INFO";

    /* SQL Table Attributes                                                                 */
    public static final String MANAGED_OBJ_ID     = "MANAGED_OBJ_ID";
    public static final String MSCS_NAME          = "MSCS_NAME";
    public static final String MSCS_MAJ_VER       = "MSCS_MAJ_VER";
    public static final String MSCS_MIN_VER       = "MSCS_MIN_VER";
    public static final String MSCS_BUILD         = "MSCS_BUILD";
    public static final String MSCS_VENDOR        = "MSCS_VENDOR";
    public static final String MSCS_CSD           = "MSCS_CSD";

    /* table version                                                                        */
    public static final String TABLE_VERSION                = "1.0";
    /**
     * Default constructor for MSCS Cluster Info table
     */
    public MSCSClusterInfoTable() {
        super();
    }
    /**
     * Table create method
     * @exception TWGDbException if error creating table
     */
    public void create() throws TWGDbException {
        //-define SQL database name
        String db = TWGDatabase.TWG_DB_NAME;
        String tableName = TABLE_NAME;
        String tableTokenName = TABLE_NAME;
        //----------------------------------------------------------------------
        //-Create the columns
        //----------------------------------------------------------------------
        TWGDbColumn column;
        TWGDbIntegerColumn intColumn;
        TWGDbColumnData column_data = new TWGDbColumnData();

        intColumn = new TWGDbIntegerColumn(MANAGED_OBJ_ID, MANAGED_OBJ_ID);
        intColumn.setFilterColumn(false);
        column_data.addColumn(intColumn);

        column = new TWGDbCharColumn(MSCS_NAME, MSCS_NAME, 64);
        column.setDefaultValue("");
        column_data.addColumn(column);

        intColumn = new TWGDbIntegerColumn(MSCS_MAJ_VER, MSCS_MAJ_VER);
        column_data.addColumn(intColumn);

        intColumn = new TWGDbIntegerColumn(MSCS_MIN_VER, MSCS_MIN_VER);
        column_data.addColumn(intColumn);

        intColumn = new TWGDbIntegerColumn(MSCS_BUILD, MSCS_BUILD);
        column_data.addColumn(intColumn);

        column = new TWGDbCharColumn(MSCS_VENDOR, MSCS_VENDOR, 64);
        column.setDefaultValue("");
        column_data.addColumn(column);

        column = new TWGDbCharColumn(MSCS_CSD, MSCS_CSD, 64);
        column.setDefaultValue("");
        column_data.addColumn(column);

        //---------------------------------------------------------------------
        // parms object to hold optional parms (key, key unique, and additional indices
        //---------------------------------------------------------------------
        TWGDbTableParms parms = new TWGDbTableParms();
        parms.setKey(MANAGED_OBJ_ID);
        parms.setKeyUnique(true);
        parms.setGraphicName("com/tivoli/twg/inventory/images/database_group_s.gif");

        //---------------------------------------------------------------------
        // NLS bundle for the table
        //---------------------------------------------------------------------
        String tableBundle = "com.tivoli.twg.mscs.MSCSNLS";

        //---------------------------------------------------------------------
        // Negotiate the SQL database name
        //---------------------------------------------------------------------
        int i = 1;
        String saveTableName      = tableName;
        String saveTableTokenName = tableTokenName;

        boolean tableNegotiationComplete = false;

        while (tableNegotiationComplete == false) {
            try {
                this.TWGInitTable(db, tableTokenName, tableName, column_data, parms,
                    tableBundle);
                tableNegotiationComplete = true;
            } catch (TWGDbRealTableNameNotUniqueException e) {
                tableName = saveTableName + String.valueOf(i);
                tableTokenName = saveTableTokenName + String.valueOf(i);
                i++;
            } catch (TWGDbTokenTableNameNotUniqueException e) {
                tableName = saveTableName + String.valueOf(i);
                tableTokenName = saveTableTokenName + String.valueOf(i);
                i++;
            }
        }
    }
    /**
     * Get the version number for the table
     * @returns the table version
     */
    public String getTableVersion() {
      return TABLE_VERSION;
    }
}
