package com.tivoli.twg.mscs;

import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.log.TWGRas;
import java.util.Vector;
import java.util.Hashtable;
import com.tivoli.twg.inventory.TWGInvCollector;
import com.tivoli.twg.inventory.TWGInvCollectorConfig;

/**
 * Microsoft Cluster Service extension class 
 * @author Mike Primm
 * @version 1.1 - 6/8/05
 */
public class MSCSExtension extends TWGExtension {
   private static final String CLASSNAME     = "com.tivoli.twg.mscs.MSCSInvCollector";
   private static final String CLASSNAME_MO  = "com.tivoli.twg.mscs.MSCSManagedObject";
   private static final String CLASSNAME_MOF = "com.tivoli.twg.mscs.MSCSManagedObjectFactory";
    /**
     * Default constructor
     */
    public MSCSExtension() {
        super();
    }
    /**
     * First initialization method, intended to allow extensions to register
     * classes.
     *
     * @exception TWGExtensionInitException if a fatal error occurs during
     * the initialization method
     */
    public void InitClassRegistration() throws
        TWGExtensionInitException {
        String c = "";
        try {
            /* Register MO and MOF classes */
            RegisterClass(CLASSNAME_MO);
            RegisterClass(CLASSNAME_MOF);
            /* Register our inventory scanner and its database tables       */
            TWGInvCollector.registerCollector(this, CLASSNAME, CLASSNAME_MO);
        } catch (ClassNotFoundException cnfx) {
            throw new TWGExtensionInitException("Class '" + c + "' not found: " +
                cnfx);
        } catch (IllegalAccessException e) {
            throw new TWGExtensionInitException(e.toString());
        } catch (InstantiationException e) {
            throw new TWGExtensionInitException(e.toString());
        }
    }
    /**
     * Second initialization method, intended to allow extension to create
     * default instances of classes registered during InitClassRegistration().
     *
     * @exception TWGExtensionInitException if a fatal error occurs during
     * the initialization method
     */
    public void InitClassInstances() throws TWGExtensionInitException {
        String errmsg;
        /* create default inventory queries                                                 */
        String tableClass = null;
        try {
            if(TWGDatabase.isActive()) {
                TWGInvCollectorConfig.CreateDefaultQueries("com.tivoli.twg.mscs.MSCSInvCollector",
                                                         "com.tivoli.twg.mscs.MSCSNLS");
            }
            /* Initialize client service subscriber                         */
            MSCSManagedObjectFactory.initializeServiceSubscriber();
        } catch (TWGDbException e) {
            errmsg = "MSCSExtension.InitClassInstances: Error creating default query for "
                + tableClass;
            throw new TWGExtensionInitException(errmsg);
        }
    }
    /**
     * Final initialization method, intended to allow an extension to do any
     * actions which may have required the other extensions to have completed
     * the previous passes of initialization.
     * @exception TWGExtensionInitException if a fatal error occurs during
     * the initialization method
     */
    public void InitCompletion() throws TWGExtensionInitException {
        String errmsg;
        /* See if the inventory filter factory already exists.  If so, add to it            */
        try {
           // Add inventory collector's tables to base filter factory
           TWGInvCollectorConfig.UpdateBaseFilterFactory("com.tivoli.twg.mscs.MSCSInvCollector");
        } catch (TWGDbException dbx) {
            errmsg = "MSCSExtension: exception adding tables to inventory factory - " + dbx;
            throw new TWGExtensionInitException(errmsg);
        }
    }
    /**
     * First termination method, intended to allow an extension to begin
     * the process of termination.  The only termination processing which
     * should occur here is that which needs to be done before other
     * extensions have executed TermCompletion() pass.
     */
    public void TermBegin() {
        /* Terminate client service subscriber                                           */
        MSCSManagedObjectFactory.terminateServiceSubscriber();
    }
    /**
     * Final termination method, intended to allow an extension to terminate
     * any processing which may still be active.
     */
    public void TermComplete() {
    }
//    /**
//     * Build default queries
//     * @param Table - database table to be generated
//     * @param insertMoName - include managed object name in query
//     */
//    public static TWGDatabaseQuery ReturnDefaultQuery(TWGDatabaseTable Table, boolean insertMoName){
//        String temp = Table.getClass().getName();
//        temp = temp.substring(temp.lastIndexOf('.')+1);
//
//        // see if the query already exists.  No need to create it again
//        TWGDatabaseQuery Query = TWGDatabaseQuery.findQueryByIdString(temp);
//        if ( Query != null ) {
//            return Query;
//        }
//        //prepare to build query
//        Vector selectFields = new Vector();
//        if (insertMoName) {
//            selectFields.addElement(new TWGDbQueryField(
//                com.tivoli.twg.inventory.TWGManagedObjectTable.TABLE_NAME,
//                com.tivoli.twg.inventory.TWGManagedObjectTable.LABEL));
//        }
//        selectFields.addElement(new TWGDbQueryField(Table.getTokenName(),"*"));
//
//        String db =  Table.getDatabase().getDatabaseName();
//        //create parms to init query
//        TWGDbQueryParms qParms;
//        try {
//            qParms = new TWGDbQueryParms(selectFields,db);
//            qParms.setResourceBundleName("com.tivoli.twg.mscs.MSCSNLS");
//            qParms.setResourceBundleKey("TableName."+temp);
//            qParms.setIdString(temp);
//            qParms.setGraphic(com.tivoli.twg.inventory.TWGInvTaskFrame.STANDARD_ICON);
//            qParms.setRowDistinct(false);
//            qParms.setDefault(true);
//            Query = new TWGDatabaseQuery(qParms);
//        } catch (TWGDbException dbx) {
//            TWGRas.debug(TWGRas.ENGINE, "MSCSExtension ReturnDefaultQuery() error building query parms or database query");
//        }
//        if (Query != null) {
//            return Query;
//        } else {
//            System.out.println("Query Construction failed....");
//            return Query;
//        }
//    }
}
