/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.monitors;

import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeLocalImplFactory;

import com.tivoli.twg.monitor.api.*;

import java.lang.NumberFormatException;
import java.util.Locale;
import java.util.Enumeration;
import java.lang.Long;

/*******************************************************************************
 *                                                                             *
 * This is the Java sample for the Director Monitor Server APIs.            *
 *                                                                             *
 * @author William Trautman                                                    *
 * @version July 1999                                                          *
 *                                                                             *
 *******************************************************************************/
public class MonAPISample1 {
   // ----------------
   // class variables
   // ----------------

   static TWGMonAPI   api = null;

   public static void main(String argv[])
   {
      Long        lVal = null;
      long        moid = -1;


      // print out heading
      System.out.println("\nMonAPISample1 listing all Native MOIDs\n");

      // Enable Component tracing.
      TWGRas.initialize();

      // required initialization of the ServiceNode class with a service node factory
      ServiceNodeLocalImplFactory snlif = new ServiceNodeLocalImplFactory();
      ServiceNode.SetServiceNodeFactory(snlif);

      try {
         //--------------------------------------------------------------------
         // create a Monitor API object to communicate with the monitor server.
         // indicate that responses are expected in english and that we would like
         // the object to check for data updates every 5 seconds.
         //--------------------------------------------------------------------
         api = new TWGMonAPI( Locale.ENGLISH, 5000);
      } catch (com.tivoli.twg.monitor.api.TWGMonException e) {
         ;
      } // end try

      //--------------------------------------------------------------------
      // get the list of all native moids that are currently known
      //--------------------------------------------------------------------
      long moids[] = api.getAllNativeMOIDs();
      if ((moids != null) && (moids.length > 0)) {
         System.out.print("got Native MOIDs=");
         for (int i=0; i<moids.length; i++) {
            System.out.print(moids[i] + " ");
         }
         System.out.println("");
      } else {
         System.out.println("got zero Native MOIDs");
      } // end if we got a moid response

      // we have finished using the api object,  it is now time to tell it
      // we will not be using it anymore. this must be done before the
      // reference is released so that garbage collection will succeed in
      // reclaiming the object.
      System.out.println("shutdown API");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample1.main: shutdown API");
      api.shutdown();

      // free up the reference to the object to allow garbage collection
      // to clean up the remains of the object.
      System.out.println("nulling API");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample1.main: null API");
      api = null;

      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample1.main: exiting");
      System.out.println("exiting");
      return;
   } // end method main

} // end MonAPISample1 Class

