/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/
package com.BobCo.monitors;

import com.tivoli.twg.log.TWGRas;
import com.tivoli.twg.libs.LongValueSet;
import com.tivoli.twg.libs.ServiceNode;
import com.tivoli.twg.libs.ServiceNodeLocalImplFactory;

import com.tivoli.twg.monitor.api.*;

import java.lang.NumberFormatException;
import java.util.Locale;
import java.util.Enumeration;
import java.lang.Long;


/*******************************************************************************
 *                                                                             *
 * This is the Java sample for the Director Monitor Server APIs.            *
 *                                                                             *
 * @author William Trautman                                                    *
 * @version July 1999                                                          *
 *                                                                             *
 *******************************************************************************/
public class MonAPISample4a {
   // ----------------
   // class variables
   // ----------------

   static TWGMonAPI   api = null;


   public static void main(String argv[])
   {


      // print out heading and usage information
      System.out.println("\nMonAPISample4a - setting up alias on all current objects\n");

      // Enable Component tracing.
      TWGRas.initialize();

      // required initialization of the ServiceNode class with a service node factory
      ServiceNodeLocalImplFactory snlif = new ServiceNodeLocalImplFactory();
      ServiceNode.SetServiceNodeFactory(snlif);

      try {
         //--------------------------------------------------------------------
         // create a Monitor API object to communicate with the monitor server.
         // indicate that responses are expected in english and that we would like
         // the object to check for data updates every 5 seconds.
         //--------------------------------------------------------------------
         api = new TWGMonAPI( Locale.ENGLISH, 5000);
      } catch (com.tivoli.twg.monitor.api.TWGMonException e) {
         ;
      } // end try

      // set up the alias to the path,
      TWGMonPath  aliasPath = new TWGMonPath();
      aliasPath.addLevel("Director Sample Shortcut");

      // set up the real path path
      TWGMonPath  realPath = new TWGMonPath();
      realPath.addLevel("Director Agent");
      realPath.addLevel("MONCPU");

      /// set up the display strings for 2 locales
      TWGMonPath[] aliasPathDisplayStrings = new TWGMonPath[2];

      TWGMonPath englishPathStrings = new TWGMonPath();
      englishPathStrings.addLevel("CPU Shortcut");
      englishPathStrings.setLocale(Locale.ENGLISH);
      aliasPathDisplayStrings[0] = englishPathStrings;

      TWGMonPath frenchPathStrings = new TWGMonPath();
      frenchPathStrings.addLevel("Le CPU Shortcut");
      frenchPathStrings.setLocale(Locale.FRENCH);
      aliasPathDisplayStrings[1] = frenchPathStrings;

      // monAPIobject is an instance of the monitor Api object
      // this line will set the alias against all currently known Native
      // managed objects as a TREE based allias
      api.setAttributeAlias(api.getAllNativeMOIDs(),
                            aliasPath,
                            aliasPathDisplayStrings,
                            realPath,
                            true );

      // we have finished using the api object,  it is now time to tell it
      // we will not be using it anymore. this must be done before the
      // reference is released so that garbage collection will succeed in
      // reclaiming the object.
      System.out.println("shutdown API");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample4a.main: shutdown API");
      api.shutdown();

      // free up the reference to the object to allow garbage collection
      // to clean up the remains of the object.
      System.out.println("nulling API");
      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample4a.main: null API");
      api = null;

      if (TWGRas.isDebugEnabled(TWGRas.MONITORS))
         TWGRas.debug(TWGRas.MONITORS,
                      "MonAPISample4a.main: exiting");
      System.out.println("exiting");
      return;
   } // end method main

} // end MonAPISample4a Class

