/////////////////////////////////////////////////////////
// Module		OptionsPanel.java
// Version		3.10
// Language		Java
// Author		Scott Riley
// History
// 06/09/00    Release 3.10
// 10/26/99    Initial release
//
// Copyright (C) 1999-2000 Tivoli Systems, an IBM Company  
//
////////////////////////////////////////////////////////

import java.awt.*;
import java.awt.event.*;

public class OptionsPanel extends Panel implements ItemListener
{
	FindFrame myFrame;
	CheckboxGroup booleanCheckboxGroup;
	Checkbox andCheckbox, orCheckbox, asStringCheckbox;
	Checkbox caseSensitivity;
	Panel casePanel;
		
	// Constructor
	OptionsPanel(FindFrame mf)
	{
		myFrame = mf;
		Panel booleanPanel;

		// The three checkboxes are created and grouped together.
		booleanCheckboxGroup = new CheckboxGroup();
		andCheckbox = new Checkbox("all the words", booleanCheckboxGroup, false);
		orCheckbox = new Checkbox("any of the words", booleanCheckboxGroup, false);
		asStringCheckbox = new Checkbox("exact phrase", booleanCheckboxGroup, true);

		// For better layout, place groups of AWT components
		// in their own Panels
		booleanPanel = new Panel();
		casePanel = new Panel();

		// Lay out the boolean search switches in their panel.
		booleanPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		booleanPanel.add(andCheckbox);
		booleanPanel.add(orCheckbox);
		booleanPanel.add(asStringCheckbox);
		booleanPanel.validate();
		booleanPanel.setVisible(true);

		// Create the checkbox for caseSensitivity.
		caseSensitivity = new Checkbox("exact case");

		// Lay out the case sensititvity switch in its panel.
		casePanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		casePanel.add(caseSensitivity);
		casePanel.validate();
		casePanel.setVisible(true);

      //  Listen for events.
      andCheckbox.addItemListener(this);
      orCheckbox.addItemListener(this);
      asStringCheckbox.addItemListener(this);
      caseSensitivity.addItemListener(this);

		setLayout(new GridLayout(2,1));
		
		// Lay out the switch panels
		add(booleanPanel);
		add(casePanel);
		validate();
		setVisible(true);
	}

	// Method to find out if the caseSensitivity switch is on
	public boolean isCaseSensitiveSearch() 
	{
		return caseSensitivity.getState();
	}
	
	// Method to find out if the search is AND or OR
	public boolean isBooleanSearch()
	{
		if (andCheckbox.getState() || orCheckbox.getState()) {
			return true;
		} else {
			return false;
		}
	}

	// Method to find out if the AND switch is on
	public boolean isANDSearch() 
	{
		return andCheckbox.getState();
	}

	// Method to find out if the OR switch is on
	public boolean isORSearch()
	{
		return orCheckbox.getState();
	}

	// Paint a 3-d box around the panel.
	public void paint(Graphics g)
	{
		Dimension d = getSize();
		Color bg = getBackground();
		
		g.setColor(bg);
		g.draw3DRect(0,0,d.width-1,d.height-1,true);
		g.draw3DRect(2,2,d.width-5,d.height-5,false);
	}
	
	// The padding for the chrome
	public Insets getInsets()
	{
		return new Insets(3,3,3,3);
	}
	
	// The minimum size of this panel
	public Dimension getMinimumSize()
	{
		Dimension d = new Dimension(400,70);
		return d;
	}
		
	// Would actually prefer it to be of this size
	public Dimension getPreferredSize()
	{
		Dimension d = new Dimension(400,70);
		return d;
	}

	// Event handler
   public void itemStateChanged(ItemEvent evt) {
      }
}
