/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/

package com.BobCo.snapins;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * The PieChart class extends JPanel to draw a 3-D pie chart.
 * This is a simple implementation that only expects two
 * data values for the chart.
 *
 * @version 1.1 - 6/8/05
 */
public class PieChart extends JPanel
{
   /* Private data */
   private long[]    values;
   private Color[]   colors;
   private Dimension d;

   /* 
    * Constructor.  
    *
    * @param v - an array of two values to be charted
    * @param c - an array of two colors to be used for the char
    */
   public PieChart ( long[] v, Color[] c )
   {
      /* Save the arrays */
      values = v;
      colors = c;

      /* Always be sure that the first value is larger */
      if (values[1] > values[0])
      {
         /* We need to swap the value (and colors) */
         long tmp = values[0];
         values[0] = values[1];
         values[1] = tmp;
         Color ctmp = colors[0];
         colors[0] = colors[1];
         colors[1] = ctmp;
      }

      /* Calculate the preferred size */
      d = super.getPreferredSize();
      d.width = d.width * 20;
      d.height = d.height * 15;
   }

   /**
    * Update the window by painting the pie chart.
    */
   public void update (Graphics g)
   {
      paint( g );
   }

   /**
    * Retrieve the preferred size.
    */
   public Dimension getPreferredSize ()
   {
      return d;
   }

   /**
    * Retrieve the minimum size.
    */
   public Dimension getMinimumSize ()
   {
      return d;
   }

   /**
    * Draw the pie-chart.
    */
   public synchronized void paint (Graphics g)
   {

      Dimension size = d;
      g.setColor(getBackground());
      g.fillRect(0,0,size.width,size.height);
      int side_height = size.height / 8;
      int oval_height = size.height - side_height;
      int oval_width = size.width;
      Color c = new Color(colors[0].getRed()/2,colors[0].getGreen()/2,colors[0].getBlue()/2);
      g.setColor(c);
      g.fillArc(0,side_height,oval_width,oval_height,180,180);
      g.fillRect(0,oval_height/2,oval_width,side_height);
      int angle = 0;
      if (values[0] != 0)
      {
         angle = (int) ((values[1] * 360L) / (values[0] + values[1]));
      }
      g.setColor(colors[1]);
      g.fillArc(0,0,oval_width,oval_height,0,angle);
      g.setColor(colors[0]);
      g.fillArc(0,0,oval_width,oval_height,angle,360-angle);
   }
}
