/******************************************************************************
 * Licensed Materials - Property of IBM                            *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved. *
 *****************************************************************************/

package com.BobCo.snapins;

import javax.swing.*;
import com.tivoli.twg.cim.*;
import com.tivoli.twg.guilibs.*;

/**
 * ProgramGroupData is used to represent a program group 
 * within a UFTreeListView.  It inherits from the base
 * UFTreeListViewObject and add CIM instance information
 * to each node in the tree.
 *
 * @version 1.1 - 6/8/05
 */
public class ProgramGroupData extends UFTreeListViewObject
{
   /* Private data for each tree node */
   private aCIMInstance instance;
   private String       full_name;

    /**
     * Constructor for ProgramGroupData.  After calling the
     * inherited constructor, we save the CIM instance 
     * information in the object and shorten the name to be
     * more user-friendly.
     *
     * @param text        - full name of the program group
     * @param image       - icon image to display
     * @param cimInstance - CIM instance data
     */
   public ProgramGroupData ( String text, ImageIcon image, aCIMInstance cimInstance )
   {
      /* Call the inherited constructor */
      super(text,image);

      /* Save the full name and CIM instance data */
      full_name = text;
      instance = cimInstance;
                                                           
      /* Change the display caption to be a short name by */
      /* Removing the user and parent group names from    */
      /* the display string.                              */
      if (text.indexOf(':')!=-1)
      {
         text = text.substring(text.indexOf(':')+1);
      }
      if (text.lastIndexOf('\\')!=-1)
      {
         text = text.substring(text.lastIndexOf('\\')+1);
      }
      setCaption(text);
   }

   /**
    * Retrieve the CIM instance data for the program group.
    */
   public aCIMInstance getCIMInstance ()
   {
      return instance;
   }

   /**
    * Retrieve the full name of the program group.
    */
   public String getFullName ()
   {
      return full_name;
   }
}
