/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * PID (5697-TKT)                                                             *
 *                                                                            *
 * (C) Copyright by IBM 1999. All Rights Reserved.                            *
 *****************************************************************************/
package com.BobCo.associations;

import com.tivoli.twg.log.*;
import  com.tivoli.twg.engine.TWGDatabaseConstants;
import  com.tivoli.twg.engine.TWGDatabaseQuery;
import  com.tivoli.twg.engine.TWGDbCharColumn;
import  com.tivoli.twg.engine.TWGDbCreateQueryCmd;
import  com.tivoli.twg.engine.TWGDbException;
import  com.tivoli.twg.engine.TWGDbFindTableByClassnameCmd;
import  com.tivoli.twg.engine.TWGDbFindTableReply;
import  com.tivoli.twg.engine.TWGDbIntegerColumn;
import  com.tivoli.twg.engine.TWGDbResultSetMetaData;
import  com.tivoli.twg.engine.TWGDbQueryResults;
import  com.tivoli.twg.engine.TWGDbResultSet;
import  com.tivoli.twg.engine.TWGDbQueryField;
import  com.tivoli.twg.engine.TWGDbQueryParms;
import  com.tivoli.twg.engine.TWGDbRunQueryCmd;
import  com.tivoli.twg.libs.BadServiceNodeImplException;
import  com.tivoli.twg.libs.Command;
import  com.tivoli.twg.libs.ServiceNode;
import  com.tivoli.twg.libs.ServiceNodeClosedException;
import  com.tivoli.twg.libs.ServiceNodeException;
import  com.tivoli.twg.libs.ServiceNodeImplFactory;
import  com.tivoli.twg.libs.ServiceNodeLocalImplFactory;
import  java.io.NotSerializableException;
import  java.io.ObjectStreamException;
import  java.util.Vector;
import  java.util.Locale;




class RunQueryCmd extends Command
{
   private TWGDbResultSet resultSet;

   public TWGDbResultSet getResultSet(){
       /*
        *	Returns query resultSet as TWGDbResultSet. Accessor for
	* 		retrieving query data by Command sender.
	*/
       return resultSet;
   }

   public RunQueryCmd(int cc) {
      super(cc);
   }

   public void CommandComplete() {
   /*
    *	Populate the TWGDbResultSet resultSet with getObjectFromOutputParm(0)
    *		Store Command's return value for database query requester to
    *	  	retrieve.
    */
   try
   {
      TWGDbResultSet dbrs = null;
      if (ReturnCode() == 0)
      {
          dbrs = (TWGDbResultSet)this.GetObjectFromOutputParm(0);
          if (dbrs == null)
          {
             System.out.println(" query result set is null!");
             return;

          }
      }
      else
      {
         System.out.println("Return code from Create Run Query was " + ReturnCode() + "\n" +
                            "Create Run Query Cmd failed!");
         return;
      }

      if (dbrs.getNumOfRows() == 0)
      {
          System.out.println( "No information in this query result set!");
          return;
      }
      resultSet = dbrs;
   }
   catch (ObjectStreamException osx)
   {
      System.out.println(osx);
   }
   catch (ClassNotFoundException cnfx)
   {
      System.out.println(cnfx);
   }
  }
}

public class RunSimpleQuery
{

   /*
   * Create a query and run it.
   */

   static ServiceNode sn = null;
   
   public static TWGDbResultSet RunQuery(Vector selectFields)
   {
      /*
       *	Static function for building a simple database query and 
       *	returning results as a TWGDbResultSet. Database fields 
       *	specified by Vector selectFields. Accesses database by 
       *	sending Command dbCmd.
       */

      Command dbCmd = null;
      // Set ServiceNode support to use local IPC implementation
      ServiceNodeLocalImplFactory snlif = new ServiceNodeLocalImplFactory();
      ServiceNode.SetServiceNodeFactory(snlif);
      try
      {
         if(sn == null){
	    // Create service node
	    // no serviceNodeName needed since only sending commands
	    String serviceNodeName = null; 	// "RunSimpleQuery"; 
	    	    sn = new ServiceNode(serviceNodeName, true);
	    if (sn.Create() == false)
	    {
		System.out.println("Create() failed");
		return null;
	    }
	 }

         // Find out what the token name is for our table.  The token name for a given
         // table needs to be retrieved only once, which can be done when an application
         // is started because the token name for a given table does not change.

         // create a database command object to hold the table class name
         String tableTokenName = null;
         TWGDbFindTableByClassnameCmd dbFindTableByClassnameCmd =
	   		new TWGDbFindTableByClassnameCmd("com.BobCo.associations.BobCoAssociationsTable");
                        //new TWGDbFindTableByClassnameCmd("com.BobCo.database.BobCoDeviceTable");

         // create the IPC command and add the database command as its input parm
         dbCmd = new Command(TWGDbFindTableByClassnameCmd.TWG_DB_FIND_TABLE_BY_CLASSNAME_CMD);
         dbCmd.SetDestinationAddress(TWGDatabaseConstants.UPDATE_SERVICE_NODE_NAME);

         // add the database command as the input parm.
         dbCmd.AddObjectAsInputParm(dbFindTableByClassnameCmd);

         // send the command
         sn.SendCommand(dbCmd);

         if (dbCmd.ReturnCode() == 0)
         {
            TWGDbFindTableReply dbReply = (TWGDbFindTableReply)dbCmd.GetObjectFromOutputParm(0);

            if (dbReply == null)
            {
               System.out.println("Find Table reply is null");
               return null;
            }
            else
            {
               tableTokenName =  dbReply.getTableTokenName();
            }
         }
         else
         {
            System.out.println("Return code from Find Table was " + dbCmd.ReturnCode());
            return null;
         }
         System.out.println("table token is " +  tableTokenName);
         /*
         Create a user query with JOIN format.
         First builds query parameters and then sends these in a create run query command to the DB.
         */
         Long queryId = null;
         
         // Build a query parameter with the selected field
         TWGDbQueryParms qParms = new TWGDbQueryParms(selectFields);
         // The query will not be saved.
         qParms.setSavedQuery(false);
         // The query values will be translated.
         qParms.setTranslateValues(true);
         // Keep all rows in query even if duplicate
         qParms.setRowDistinct(false);
         // This is a user query)
         qParms.setDefault(false);
         // The query is JOIN format
         qParms.setFormat(TWGDatabaseQuery.JOIN_FORMAT);
         // set column fields by which to sort the result set
         Vector sortField = new Vector();
         // Build create query command
         TWGDbCreateQueryCmd cmdParm = new TWGDbCreateQueryCmd(qParms);
         // Sets the locale for the query result set.
         cmdParm.setLocale(Locale.getDefault());
         // Create a command to send create query request
         dbCmd = new RunQueryCmd(TWGDbCreateQueryCmd.TWG_DB_CREATE_RUN_QUERY_CMD);
         // set the destination to the database query service node
         dbCmd.SetDestinationAddress(TWGDatabaseConstants.QUERY_SERVICE_NODE_NAME);
         // add the create query database command object as an input parameter
         // to the IPC command
         dbCmd.AddObjectAsInputParm(cmdParm);
         // send the command and wait for returning.
         sn.SendCommand(dbCmd);
      }
      catch (TWGDbException dbx)
      {
         System.out.println(dbx.getMessage());
      }
      catch (NotSerializableException nsx)
      {
         System.out.println(nsx.getMessage());
      }
      catch (BadServiceNodeImplException snix)
      {
         System.out.println(snix);
      }
      catch (ServiceNodeClosedException sncx)
      {
         System.out.println(sncx);
      }
      catch (ServiceNodeException snx)
      {
         System.out.println(snx);
      }
      catch (InterruptedException ix)
      {
         System.out.println(ix);
      }
      catch (ClassNotFoundException cnfx)
      {
         System.out.println(cnfx);
      }
      catch (ObjectStreamException osx)
      {
         System.out.println(osx);
      }
      return ((RunQueryCmd)dbCmd).getResultSet();
   }
}



