/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/
package com.ibm.sysmgt.sdk.tools;

import java.util.*;
import java.io.*;
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import com.tivoli.twg.console.TWGAddMOFObjectDialog;
import com.tivoli.twg.console.TWGAddMOFObjectInterface;
import com.tivoli.twg.console.TWGProductInfo;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.log.TWGRas;
import com.tivoli.uif.controls.*;
import com.tivoli.twg.netipc.TWGSvcMgrServiceNodeConstants;
import com.tivoli.twg.netipc.TWGNetMgrServiceNodeConstants;

/**
* The SDKCLICreatePanel class is a panel loaded by the main console which
* allows a user to create a new user defined managed object.  To register
* the new managed object with the Director server, the user must enter the
* name of the managed object on the panel.
*
* @see com.tivoli.twg.console.TWGAddMOFObjectDialog
*/
public class SDKCLICreatePanel extends JPanel
                               implements TWGAddMOFObjectInterface,
                                          KeyListener
{
  private TWGAddMOFObjectDialog parentDialog = null;
  private ServiceNode serviceNode;
  private boolean Ready = false;
  private JTextField hostName;
  //private JTextField address;
  private JLabel hostLabel;
  //private JLabel addressLabel;
  private MultiLocaleBundle mlb = null;
  /* Name of our Resource Bundle class                               */
  private static final String OURRESOURCEBUNDLE = "com.ibm.sysmgt.sdk.tools.SDKCLIResourceBundle";




  /**
   * Default constructor for new object panel
   */
  public SDKCLICreatePanel()
  {
    /* Load our NLS resources                                 */
    TWGRas.entryExit(TWGRas.SDK, "SDKCLICreatePanel: entering constructor");

    try
    {
      serviceNode = new ServiceNode();
      if (serviceNode.Create() == false)
      {
        Ready = false;
      }
      else
      {
        Ready = true;
      }
    }
    catch (BadServiceNodeImplException bsnie) {Ready = false;}
    catch (ServiceNodeClosedException snce) {Ready = false;}
    catch (ServiceNodeException sne) {Ready = false;}

    try
    {
      mlb = MultiLocaleBundle.getSharedBundle(OURRESOURCEBUNDLE);
    }
    catch (MissingResourceException mrx)
    {
      mlb = null;
    }
    /* Build disabled panel : we will enable when we get IPC  */
    /* protocol data                                          */
    BuildView();
  }

  /**
   * Build panel view
   */
  private void BuildView()
  {
    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();

    setLayout(gbl); /* Use gridbag layout                               */
    gbc.fill = GridBagConstraints.BOTH;     /* Fill both vertical+horiz */
    gbc.ipadx = 5;
    gbc.ipady = 1;
    gbc.insets = new Insets(5, 5, 5, 5);

    /* Create fields for system name entry field                        */
    hostLabel = new JLabel("Host Name");
    hostLabel.setOpaque(false);
    gbc.gridwidth=GridBagConstraints.RELATIVE;
    gbl.setConstraints(hostLabel, gbc);
    add(hostLabel);
    hostName = new JTextField("");
    hostName.setEditable(true);
    hostName.addKeyListener( this );
    gbc.gridwidth=GridBagConstraints.REMAINDER;
    gbl.setConstraints(hostName, gbc);
    add(hostName);

    /* Set the number of columns for the hostName to be 12 characters. The
     * will control the width of the column.
     */
    hostName.setColumns (16 );

    /* Create fields for address entry field                            */
    //addressLabel = new JLabel("Network Address");
    //addressLabel.setOpaque(false);
    //gbc.gridwidth=GridBagConstraints.RELATIVE;
    //gbl.setConstraints(addressLabel, gbc);
    //add(addressLabel);
    //
    //address = new JTextField("");
    //address.setEditable(true);
    //address.addKeyListener( this );
    //gbc.gridwidth=GridBagConstraints.REMAINDER;
    //gbl.setConstraints(address, gbc);
    //add(address);
  }

  /**
   * Set the background color for this panel
   * @param c - the color to set as the background color
   */
  public void setBackground( Color c )
  {
    super.setBackground( c );
  }

  /**
   * Set the foreground color for this panel
   * @param c - the color to set as the foreground color
   */
  public void setForeground( Color c )
  {
    super.setForeground( c );
    if( hostLabel != null )
        hostLabel.setForeground( c );
    //if( addressLabel != null )
    //    addressLabel.setForeground( c );
  }

  /**
   * Set the parent dialog
   * @param parent - the parent to point to
   */
  public void setParentDialog( TWGAddMOFObjectDialog parent )
  {
    parentDialog = parent;
    okIsAllowed();
  }

  public boolean displayStatusBar()
  {
    return true;
  }

  public void setFocus()
  {
    hostName.requestFocus();
  }

  /**
   * Allows subclassing applications to verify information entered
   * before the dialog is dismissed and to prevent dismissal.
   *
   * @return Return true to allow the dialog to close.  Return false
   * to prevent the dialog from closing.
   */
  public boolean verifyOnClose()
  {
    /* Read fields from panel                               */
    String name = hostName.getText();
    //String addr = address.getText();

    //createDevice(name, addr);
    createDevice(name);

    return true;
  }

  /**
  * Sends command to server to create a new managed object
  *
  * @param hostName the name of the managed object to add
  // * @param address the address of the managed object to add
  */
  //private void createDevice(String hostName, String address)
  private void createDevice(String hostName)
  {
    boolean returnValue = false;
    Command cmd = null;
    TWGRas.entryExit(TWGRas.SDK, "SDKCLICreatePanel.createDevice(), hostName = "
                     + hostName);


    parentDialog.startStatusIndicator();
    /* create a new device creation cmd */
    cmd = new Command(SDKCLITaskServer.USER_DEFINED_SYSTEM_CREATION);
    /* Send it to the base here */
    cmd.SetDestinationAddress(SDKCLITaskServer.TASK_SERVER_CLASS);
    try
    {
      /* Add the new device information here */
      cmd.AddObjectAsInputParm(hostName);
      //cmd.AddObjectAsInputParm(address);
      cmd.setCommandCompleteListener(new AddComplete(parentDialog, mlb));

      /* Long timeout for this command */
      //cmd.SetTimeOut(60000);

      try
      {
        serviceNode.SendAsynchCommand(cmd);
      }
      catch (ServiceNodeClosedException exception)
      {
        TWGRas.error(TWGRas.SDK, "SDKCLICreatePanel.createDevice(): servicenode exception "
                     + exception);
      }
      catch (InterruptedException exception)
      {
        TWGRas.error(TWGRas.SDK, "SDKCLICreatePanel.createDevice(): servicenode exception "
                     + exception);
      }
      catch (ServiceNodeException exception)
      {
        TWGRas.error(TWGRas.SDK, "SDKCLICreatePanel.createDevice(): servicenode exception "
                     + exception);
      }
    }
    catch (NotSerializableException exception) {}
  }

  public void pCleanUp()
  {
    if (serviceNode != null)
    {
      try
      {
        serviceNode.Close();
      }
      catch (ServiceNodeException sne) {}

      serviceNode = null;
    }

    if( hostName != null )
    {
      hostName.removeKeyListener( this );
      //address.removeKeyListener( this );
      removeAll();
      hostName  = null;
      //address  = null;
    }
  }
  // End Required for TWGAddMOFObjectInterface

  /**
   * Convenience method for enabling/disabling the OK button on the
   * dialog based on the current contents of the entry fields.
   */
  private void okIsAllowed()
  {
    // If we have access to our parent, and its a TWGAddMOFObjectFrame, then
    // enable/disable the button based on the data
    if (parentDialog != null)
    {
      boolean goodinput = checkInputData();
      parentDialog.getOKButton().setEnabled( goodinput );
      UFButton button = null;
      if (goodinput)
      {
        button = parentDialog.getOKButton();
      }
      else
      {
        button = parentDialog.getCancelButton();
      }
      if (button != null)
      {
        parentDialog.getRootPane().setDefaultButton(button);
      }
    }
  }

  /**
   * Convenience method for checking the current contents of the entry fields.
   * @return Return true if both entry fields contain data. Otherwise
   *         return false if either of them are empty.
   */
  private boolean checkInputData()
  {
    boolean brc = false;
    if ( hostName != null )
    {
      String sysn = hostName.getText();
      //String addr = address.getText();
      //brc = (!(( sysn == null || sysn.trim().equals( "" )) ||
      //         ( addr == null || addr.trim().equals( "" ))));
      brc = (!( sysn == null || sysn.trim().equals( "" )));
    }
    return brc;
  }

  /**
   * Implement a key listener to update okay button
   */
  public void keyPressed(KeyEvent e) {}
  public void keyTyped(KeyEvent e) {}
  public void keyReleased( KeyEvent e ) { okIsAllowed(); }



  // begin private inner class AddComplete
  /** Completion notification listener object for add command   */
  private static class AddComplete implements CommandCompleteListener, Runnable
  {
    int result;
    MultiLocaleBundle mlb;
    TWGAddMOFObjectDialog parent = null;

    AddComplete(TWGAddMOFObjectDialog dlg, MultiLocaleBundle nls)
    {
      mlb = nls;
      parent = dlg;
    }

    /**
     * Command complete notification
     * @param cmd - completed command
     */
    public void CommandComplete(Command cmd)
    {
      TWGRas.entryExit(TWGRas.SDK, "SDKCLICreatePanel.AddComplete.CommandComplete()");

      try
      {
        // Check result of command
        result = ((Integer)cmd.GetObjectFromOutputParm(0)).intValue();
      }
      catch (ObjectStreamException ose)
      {
        result = SDKCLITaskServer.BADOTHER;
        TWGRas.error(TWGRas.SDK, "SDKCLICreatePanel.AddComplete.CommandComplete(): Cannot retrieve output from command: " + ose);
      }
      catch (ClassNotFoundException cnfe)
      {
        result = SDKCLITaskServer.BADOTHER;
        TWGRas.error(TWGRas.SDK, "SDKCLICreatePanel.AddComplete.CommandComplete(): Cannot retrieve output from command: " + cnfe);
      }

      /* If parent window found, display result dialog        */
      if (parent != null)
      {
        Thread thrd = new Thread(this);
        thrd.start();
      }
      else
      {
        // If the status indicator is running, parent will stop
        // it prior to clean up.
        parent.dismissDialog();
      }
    }

    /**
     * Run method : displays result dialog box
     */
    public void run()
    {
      TWGRas.entryExit(TWGRas.SDK, "SDKCLICreatePanel.Addcomplete.run(): entering");

      /* If good, inform user that object may not appear
       * immediately                                          */
      if (result == SDKCLITaskServer.GOOD)
      {
          String goodtitle = getString("addobject.goodresult.title");
          String goodmsg = TWGProductInfo.formatMessage(getString("addobject.goodresult.msg"),
                                                        TWGProductInfo.PRODUCT_NAME);
          parent.setDismissDialog( true );
          parent.displayMessage(goodmsg, goodtitle, JOptionPane.INFORMATION_MESSAGE);
      }
      /* If bad address, display error                        */
      else if (result == SDKCLITaskServer.BADADDR)
      {
          String badaddrtitle = getString("addobject.badaddrresult.title");
          String badaddrmsg = getString("addobject.badaddrresult.msg");
          parent.displayMessage(badaddrmsg, badaddrtitle, JOptionPane.WARNING_MESSAGE);
      }
      else if (result == SDKCLITaskServer.DUPLICATE)
      {
          String duplicatetitle = getString("addobject.duplicateresult.title");
          String duplicatemsg = getString("addobject.duplicateresult.msg");
          parent.displayMessage(duplicatemsg, duplicatetitle, JOptionPane.ERROR_MESSAGE);
      }
      /* Else, some other error occurred                      */
      else
      {
          String badtitle = getString("addobject.badotherresult.title");
          String badmsg = getString("addobject.badotherresult.msg");
          parent.displayMessage(badmsg, badtitle, JOptionPane.ERROR_MESSAGE);
      }
      // stop the status indicator
      parent.stopStatusIndicator();
    }

    public String getString(String id)
    {
      String name = id;
      if(mlb != null)
      {
          name = mlb.getString(Locale.getDefault(), id, id);
      }
      return name;
    }
  }
  // end private inner class AddComplete


}
