/*
 * SDKCLIExtension.java
 *
 * (c) Copyright International Business Machines Corp. 2001.
 * All rights reserved.
 *
 * U.S. Government Users Restricted Rights.
 *
 * Use, duplication or disclosure restricted
 * by GSA ADP Schedule Contract with IBM Corp.
 *
 * Licensed Material - Property of IBM.
 */
package com.ibm.sysmgt.sdk.tools;

import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.log.TWGRas;
import java.util.*;
import com.ibm.sysmgt.sdk.CmdTask.CmdTaskData;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * This director server extension supports APIs to create and delete groups,
 * managed objects, and tasks.
 *
 * @version 1.0 - 5/06/2002
 */
public class SDKCLIExtension extends TWGExtension
{
  /**
   * Default constructor
   */
  public SDKCLIExtension()
  {
    super();
    TWGRas.debug(TWGRas.SDK, "SDKCLIExtension.SDKCLIExtension()");
    Properties p = getPropertySet();
  }

  /**
   * TWGExtension - First initialization method
   * @exception TWGExtensionInitException if a fatal error occurs
   */
  public void InitClassRegistration() throws TWGExtensionInitException
  {
    TWGRas.debug(TWGRas.SDK, "SDKCLIExtension.InitClassRegistration()");
    String c = "";

    /* Now register all of our persistent object classes here */
    try
    {
      c = "com.ibm.sysmgt.sdk.tools.SDKCLITaskServer";
      RegisterClass( c );
      c = "com.ibm.sysmgt.sdk.tools.UserDefinedManagedSystem";
      RegisterClass(c);
      c = "com.ibm.sysmgt.sdk.tools.UserDefinedManagedSystemFactory";
      RegisterClass(c);
    }
    catch (ClassNotFoundException cnfx)
    {
        TWGRas.debug(TWGRas.SDK,"Class '"+ c +"' not found: throwing ClassNotFoundException");
        throw new TWGExtensionInitException("Class '" + c + "' not found: " + cnfx);
    }
  }

  /**
   * TWGExtension - Second initialization method
   * @exception TWGExtensionInitException if a fatal error occurs
   */
  public void InitClassInstances() throws TWGExtensionInitException
  {
    TWGRas.debug(TWGRas.SDK, "SDKCLIExtension.InitClassInstances()");

    try
    {
      if (!TWGTask.isTaskID("SDK|CLI"))
      {
        try
        {
          TWGDefaultTask testTask =
               new TWGDefaultTask("com/ibm/sysmgt/sdk/tools/SDKCLI.properties");
        }
        catch (TWGTaskCreationException tce)
        {
          tce.printStackTrace ();
          TWGRas.error(TWGRas.SDK,
                       "SDKCLIExtension.InitClassInstances(): TWGTaskCreationException "
                       + tce);
          throw new TWGExtensionInitException("Error creating SDKCLI default task");
        }
      }
    }
    catch (Exception e)
    {
      TWGRas.error(TWGRas.SDK, "SDKCLIExtension.InitClassInstances()", e);
      throw new TWGExtensionInitException(e.getMessage());
    }
  }

  /**
   * TWGExtension - Final initialization method
   * @exception TWGExtensionInitException if a fatal error occurs
   */
  public void InitCompletion() throws TWGExtensionInitException
  {
    TWGRas.debug(TWGRas.SDK, "SDKCLIExtension.InitCompletion()" );
  }

  /**
   * TWGExtension - First termination method
   * @exception TWGExtensionTermException if an error occurs
   */
  public void TermBegin() throws TWGExtensionTermException
  {
    TWGRas.debug(TWGRas.SDK, "SDKCLIExtension.TermBegin()" );
  }

  /**
   * TWGExtension - Final termination method
   * @exceptio n TWGExtensionTermException if an error occurs
   */
  public void TermComplete() throws TWGExtensionTermException
  {
    TWGRas.debug(TWGRas.SDK, "SDKCLIExtension.TermComplete()" );
  }

}
