/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * 09N6996
 *
 * (C) Copyright IBM Corp. 1999, 2004 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
package com.BobCo.snmp;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import com.tivoli.twg.console.TWGMainGUI;
import com.tivoli.twg.console.TWGTaskFrame;
import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.snmp.*;
import com.tivoli.twg.snmputils.*;

/**
 * SNMPSample the usage of the SNMP PDU functions.
 */
public class SNMPSample extends TWGTaskFrame implements SNMPRemoteResponse
{
   private JTextArea textArea;

   /**
    * Constructor
    */
   public SNMPSample()
   {
      super(true);
   }

   /**
    * Create the client pane and return it.
    */
   public Container buildView()
   {
      setTitle("SNMP Sample");

      JPanel returnPanel = new JPanel();

      // create the text message and add it to the center
      returnPanel.setLayout(new BorderLayout());

      textArea = new JTextArea("", 10, 10);
      returnPanel.add("Center", textArea);

      long moid = getTaskActivator().getMoid().GetValue(0);


      try {
         // Construct our remote SNMP object
         SNMPRemoteRequest remoteRequest = new SNMPRemoteRequest(SNMPRemoteRequest.SNMP_V1);

         String[] oidArray = new String[2];
         oidArray[0] = "1.3.6.1.2.1.1.1.0"; // sysDescr
         oidArray[1] = "1.3.6.1.2.1.1.2.0"; // sysObjectID
         //oidArray[2] = "1.3.6.1.2.1.1.3.0"; // sysUpTime
         //oidArray[3] = "1.3.6.1.2.1.1.4.0"; // sysContact
         //oidArray[4] = "1.3.6.1.2.1.1.5.0"; // sysName
         //oidArray[5] = "1.3.6.1.2.1.1.6.0"; // sysLocation
         //oidArray[6] = "1.3.6.1.2.1.1.7.0"; // sysServices

         textArea.append("Get Request on object ids:\n");

         // Attempt a Get request on oids
         for (int i = 0; i < oidArray.length; ++i) {
            textArea.append("   " + oidArray[i] + "\n");
         }

         long rc = remoteRequest.sendGetRequest(moid, oidArray, this);
         //boolean rc = remoteSNMP.sendGetNextRequest(moid, oidArray, this);
         //boolean rc = remoteSNMP.sendGetBulkRequest(moid, oidArray, 1, 2, this);
         //boolean rc = remoteSNMP.sendInformRequest(moid, oidArray, this);
         if (rc < 0) {
            textArea.append("SNMP Get Request failed!\n");
         }

         textArea.append("---------------------------------------------------\n");

      } catch (ServiceNodeException sne) {
         textArea.append("Service Node Exception during Remote SNMP communication!\n");
      }

      return returnPanel;
   }

   /**
    * Initial frame size
    */
   public Dimension getFrameSize()
   {
      return new Dimension(300, 300);
   }

   public void response(SNMPRemoteRequest caller, long moid, int requestType, long requestID, VariableBinding[] vbl, long errorIndex)
   {
      if (errorIndex < 0) {
         textArea.append("Error " + errorIndex + "\n");
      } else {
         printInfo(vbl);
      }
   }

   private void printInfo(VariableBinding[] vbl)
   {
      for (int i = 0; i < vbl.length; ++i) {
         if (vbl[i] instanceof CounterVbl) {
            textArea.append("   oid = " + ((CounterVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((CounterVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((CounterVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof Counter64Vbl) {
            textArea.append("   oid = " + ((Counter64Vbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((Counter64Vbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((Counter64Vbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof GaugeVbl) {
            textArea.append("   oid = " + ((GaugeVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((GaugeVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((GaugeVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof IntegerVbl) {
            textArea.append("   oid = " + ((IntegerVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((IntegerVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((IntegerVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof IpAddressVbl) {
            textArea.append("   oid = " + ((IpAddressVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((IpAddressVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((IpAddressVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof NsapVbl) {
            textArea.append("   oid = " + ((NsapVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((NsapVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((NsapVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof OidVbl) {
            textArea.append("   oid = " + ((OidVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((OidVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((OidVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof OpaqueVbl) {
            textArea.append("   oid = " + ((OpaqueVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((OpaqueVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((OpaqueVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof StringVbl) {
            textArea.append("   oid = " + ((StringVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((StringVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((StringVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof TimeticksVbl) {
            textArea.append("   oid = " + ((TimeticksVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((TimeticksVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((TimeticksVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof UnsignedVbl) {
            textArea.append("   oid = " + ((UnsignedVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((UnsignedVbl)vbl[i]).getType() + "\n");
            textArea.append("   value = " + ((UnsignedVbl)vbl[i]).getValue() + "\n");
         } else if (vbl[i] instanceof NoSuchObjectVbl) {
            textArea.append("   oid = " + ((NoSuchObjectVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((NoSuchObjectVbl)vbl[i]).getType() + "\n");
         } else if (vbl[i] instanceof NoSuchInstanceVbl) {
            textArea.append("   oid = " + ((NoSuchInstanceVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((NoSuchInstanceVbl)vbl[i]).getType() + "\n");
         } else if (vbl[i] instanceof EndOfMibViewVbl) {
            textArea.append("   oid = " + ((EndOfMibViewVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((EndOfMibViewVbl)vbl[i]).getType() + "\n");
         } else if (vbl[i] instanceof EmptyVbl) {
            textArea.append("   oid = " + ((EmptyVbl)vbl[i]).getOid() + "\n");
            textArea.append("   type = " + ((EmptyVbl)vbl[i]).getType() + "\n");
         } else {
            if (vbl[i] == null) {
               textArea.append("VBL is null\n");
            }
         }
      }
   }

}


