package com.tivoli.twg.mscs;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.libs.TWGConVersion;
import com.tivoli.twg.mscs.*;

/** 
 * Object to represent a group in the cluster. Main difference between
 * this and the base object is that this adds an owner field
 */
public class TWGMSCSClusterGroupObject extends TWGMSCSClusterObject 
                                       implements ActionListener
                          {
    private JMenu         changemenu=null;
    private JMenuItem     fPopupOnline = null;
    private JMenuItem     fPopupOffline = null;
    private JMenuItem     fPopupDelete = null;
    private JMenuItem     fPopupRename = null;
    private Vector        faPopupNodes = null;
    private Vector        faPopupNodeNames = null;
    private Vector        faPopupNodeTags = null;
    private String        owner;

    public TWGMSCSClusterGroupObject(String nname, String nstate, String ndesc, String nowner) {
        super(nname,nstate,ndesc);
        owner=nowner;
    }

    public void setOwner(String nowner) {
        int i;

        // if we're too low a version, skip this whole thing
        if ((TWGConVersion.getServerVersion()/100000)>3000 && (TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
            // fix the menus, if they exist
            if (faPopupNodeNames!=null && !faPopupNodeNames.isEmpty()) {
                for (i=0;i<faPopupNodeNames.size() && faPopupNodeNames.elementAt(i)!=null;i++) {
                    if (owner.equals((String)faPopupNodeNames.elementAt(i))) {
                        // found the old node, grey it in
                        if (faPopupNodes!=null && !faPopupNodes.isEmpty()){
                            if (faPopupNodes.elementAt(i)!=null) {
                                ((JMenuItem)faPopupNodes.elementAt(i)).setEnabled(true);
                            }
                        }
                    }
                    if (nowner.equals((String)faPopupNodeNames.elementAt(i))) {
                        // found the new node, grey it out
                        if (faPopupNodes!=null && !faPopupNodes.isEmpty()){
                            if (faPopupNodes.elementAt(i)!=null) {
                                ((JMenuItem)faPopupNodes.elementAt(i)).setEnabled(false);
                            }
                        }
                    }
                }
            }
        }
        
        owner=nowner;
    }
    
    public void setOwner(TWGMSCSClusterNodeObject nowner) {
        int i;

        if (nowner!=null) {
            // if we're too low a version, skip this whole thing
            if ((TWGConVersion.getServerVersion()/100000)>3000 && (TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                // fix the menus, if they exist
                if (faPopupNodeNames!=null && !faPopupNodeNames.isEmpty()) {
                    for (i=0;i<faPopupNodeNames.size() && faPopupNodeNames.elementAt(i)!=null;i++) {
                        if (owner.equals((String)faPopupNodeNames.elementAt(i))) {
                            // found the old node, grey it in
                            if (faPopupNodes!=null && !faPopupNodes.isEmpty()){
                                if (faPopupNodes.elementAt(i)!=null) {
                                    ((JMenuItem)faPopupNodes.elementAt(i)).setEnabled(true);
                                }
                            }
                        }
                        if (nowner.getName().equals((String)faPopupNodeNames.elementAt(i))) {
                            // found the new node, grey it out
                            if (faPopupNodes!=null && !faPopupNodes.isEmpty()){
                                if (faPopupNodes.elementAt(i)!=null) {
                                    ((JMenuItem)faPopupNodes.elementAt(i)).setEnabled(false);
                                }
                            }
                        }
                    }
                }
            }

            owner=nowner.getName();
        } else {
            owner=null;
        }
    }

    public String getOwner() {
        return owner;
    }

    public  String  getIconName() {
        return (new String("/com/tivoli/twg/engine/images/ggdy16.gif"));
    }

    public int      getColumns() {
        return 4;
    }

    public String   getColumnHeader(int column) {
        switch (column) {
            case 0:
                return getNlsString("colnameName");
            case 1:
                return getNlsString("colnameState");
            case 2:
                return getNlsString("colnameOwner");
            case 3:
                return getNlsString("colnameDesc");
            default:
                return getNlsString("stateUnknown");
        }
    }

    public String   getColumnString(int column) {
        switch (column) {
            case 0:
                if (getName()!=null) {
                    return getName();
                }
                break;
            case 1:
                if (getState()!=null) {
                    return getState();
                }
                break;
            case 2:
                if (getOwner()!=null) {
                    return getOwner();
                }
                break;
            case 3:
                if (getDescription()!=null) {
                    return getDescription();
                } else {
                    return new String("");
                }
            default:
                break;
        }
        return getNlsString("stateUnknown");
    }

    public void setState(String nstate) {
        // have my superclass update my state
        super.setState(nstate);

        // if we're too low a version, skip this whole thing
        if ((TWGConVersion.getServerVersion()/100000)>3000 && (TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
            // update my menu items
            // turn all three off, then turn back on the right ones
            if (fPopupOnline!=null) {
                fPopupOnline.setEnabled(false);
            }
            if (fPopupOffline!=null) {
                fPopupOffline.setEnabled(false);
            }
            if (nstate.equals(getNlsString("stateOffline"))) {
                if (fPopupOnline!=null) {
                    fPopupOnline.setEnabled(true);
                }
            } else if (nstate.equals(getNlsString("stateOnline")) ||
                       nstate.equals(getNlsString("statePartial"))) {
                if (fPopupOffline!=null) {
                    fPopupOffline.setEnabled(true);
                }
            }
        }
    }

    public int      getPopupItemCount() {
        if (super.getPopupItemCount()>0) {
            return super.getPopupItemCount()+7;
        } else {
            return 6;
        }
    }

    public JMenuItem getPopupItem(int index) {
        int                       i;
        JMenuItem                 tNodeMI;
        Object[]                  nameargs = { null };
        String                    tNodeMIAction;
        TWGMSCSClusterObject      co;

        switch (index) {
            case 0:
                if (fPopupOnline==null) {
                    fPopupOnline=new JMenuItem(getNlsString("miBringonline"));
                    fPopupOnline.setActionCommand("bringonline");
                    fPopupOnline.addActionListener(this);
                    if (!(getState().equals(getNlsString("stateOffline")))) {
                        fPopupOnline.setEnabled(false);
                    }
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupOnline.setEnabled(false);
                    }
                }
                return fPopupOnline;
            case 1:
                if (fPopupOffline==null) {
                    fPopupOffline=new JMenuItem(getNlsString("miTakeoffline"));
                    fPopupOffline.setActionCommand("takeoffline");
                    fPopupOffline.addActionListener(this);
                    if (!(getState().equals(getNlsString("stateOnline"))) &&
                           !(getState().equals(getNlsString("statePartial")))) {
                        fPopupOffline.setEnabled(false);
                    }
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupOffline.setEnabled(false);
                    }
                }
                return fPopupOffline;
            case 2:
                // add the change groups submenu
                // build the list of groups and groupnames
                // first, clear out anything already there
                if (faPopupNodes==null) {
                    faPopupNodes=new Vector();
                } else {
                    faPopupNodes.removeAllElements();
                }
                if (faPopupNodeNames==null) {
                    faPopupNodeNames=new Vector();
                } else {
                    faPopupNodeNames.removeAllElements();
                }
                if (faPopupNodeTags==null) {
                    faPopupNodeTags=new Vector();
                } else {
                    faPopupNodeTags.removeAllElements();
                }
                // we need to loop through the child objects of the cluster to find the nodes
                co=(TWGMSCSClusterObject)getParentWithServiceNode().getChild();
                if (co != null) {
                    i=0;
                    // loop through them and add them to the vectors
                    do {
                        if (co instanceof TWGMSCSClusterNodeObject) {
                            nameargs[0] = co.getName();
                            tNodeMIAction=MessageFormat.format(new String("movegroup{0}"),nameargs);
                            tNodeMI=new JMenuItem(co.getName());
                            tNodeMI.setActionCommand(tNodeMIAction);
                            tNodeMI.addActionListener(this);
                            if (co.getName().equals(getOwner())) {
                                tNodeMI.setEnabled(false);
                            }
                            if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                                tNodeMI.setEnabled(false);
                            }
                            faPopupNodes.insertElementAt(tNodeMI, i);
                            faPopupNodeNames.insertElementAt(co.getName(), i);
                            faPopupNodeTags.insertElementAt(tNodeMIAction, i);
                            i++;
                        }
                        co=(TWGMSCSClusterObject)co.getSibling();
                    } while ((co!=null) && (co!=(TWGMSCSClusterObject)getParentWithServiceNode().getChild()));
                }

                // now create the menu itself, and populate it
                changemenu=new JMenu(getNlsString("miMovegroup"));
                for (i=0;i<faPopupNodes.size() && faPopupNodes.elementAt(i)!=null;i++) {
                    changemenu.add((JMenuItem)faPopupNodes.elementAt(i));
                }
                // if there are less than two nodes, disable the whole menu
                if (i<2) {
                    changemenu.setEnabled(false);
                }
                return changemenu;

            case 3:
            case 6:
                // return null to have the adder add a separator instead
                return null;
            case 4:
                if (fPopupDelete==null) {
                    fPopupDelete=new JMenuItem(getNlsString("miDelete"));
                    fPopupDelete.setActionCommand("delete");
                    fPopupDelete.addActionListener(this);
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupDelete.setEnabled(false);
                    }
                }
                return fPopupDelete;
            case 5:
                if (fPopupRename==null) {
                    fPopupRename=new JMenuItem(getNlsString("miRename"));
                    fPopupRename.setActionCommand("rename");
                    fPopupRename.addActionListener(this);
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupRename.setEnabled(false);
                    }
                }
                return fPopupRename;
            default:
                break;
        }
        if (index<super.getPopupItemCount()+7) {
            return super.getPopupItem(index-7);
        } else {
            // don't know what to add, so add a separator, I guess
            return null;
        }
    }

    public void actionPerformed( ActionEvent evt )
    {
      // Place menu and/or button handling code here.
      // Remove listener interface if neither of these are required.
      //         ....................
      String                               cmd = evt.getActionCommand();
      TWGMSCSClusterObjectWithServiceNode  owsn = null;
      ServiceNode                          sn = null;
      MSCSAgentCommand                     agtcmd = null;
      TWGMSCSClusterResourceObject         childRes;
      TWGMSCSClusterObject                 coTmp = null;
      int                                  i;
      boolean                              found;

      owsn=getParentWithServiceNode();
      if (owsn!=null) {
          sn=owsn.getServiceNode();
      }

      if (cmd.equals("bringonline")) {
          // bring the group online
          if (owsn != null) {
              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_ONLINE_GROUP, owsn.GetMyDestination());
              agtcmd.addCompUnicodeInputParam(getName());
              if (sn != null) {
                  // Send command to server
                  try{
                      sn.SendAsynchCommand(agtcmd);
                  }
                  catch(InterruptedException x){
                      return;
                  } 
                  catch(ServiceNodeClosedException x){
                      return;
                  }
                  catch(ServiceNodeException x){
                      return;
                  }
              }

              // now bring the resources that belong to this group online
              // we can't be certain that we have children (some instances of the group object don't)
              // so go find one that does.  Find the "groups" group
              coTmp=owsn.findChild(getNlsString("objnameGroups"));
              if (coTmp!=null) {
                  // find my instance under the list of groups
                  coTmp=coTmp.findChild(getName());
                  if (coTmp!=null) {
                      childRes=(TWGMSCSClusterResourceObject)coTmp.getChild();
                      if (childRes!=null) {
                          do {
                              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_ONLINE_RESOURCE, owsn.GetMyDestination());
                              agtcmd.addCompUnicodeInputParam(childRes.getName());
                              if (sn != null) {
                                  // Send command to server
                                  try{
                                      sn.SendAsynchCommand(agtcmd);
                                  }
                                  catch(InterruptedException x){
                                      return;
                                  } 
                                  catch(ServiceNodeClosedException x){
                                      return;
                                  }
                                  catch(ServiceNodeException x){
                                      return;
                                  }
                              }
                              childRes=(TWGMSCSClusterResourceObject)childRes.getSibling();
                          } while ((childRes!=null) && (childRes!=coTmp.getChild()));
                      }
                  }
              }
          }

      } else if (cmd.equals("takeoffline")) {
          // take the group offline
          if (owsn != null) {
              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_OFFLINE_GROUP, owsn.GetMyDestination());
              agtcmd.addCompUnicodeInputParam(getName());
              if (sn != null) {
                  // Send command to server
                  try{
                      sn.SendAsynchCommand(agtcmd);
                  }
                  catch(InterruptedException x){
                      return;
                  } 
                  catch(ServiceNodeClosedException x){
                      return;
                  }
                  catch(ServiceNodeException x){
                      return;
                  }
              }
          }
      
      } else if (cmd.equals("delete")) {
          // delete the group
          if (owsn != null) {
              // first verify that the user Really wants to do this
              Object[] nameargs = {getName()};
              if (JOptionPane.showConfirmDialog(null,
                                            MessageFormat.format(getNlsString("areYouSureDelG"), nameargs),
                                            getNlsString("TWGMSCSBrowseFrameTitle"),
                                            JOptionPane.YES_NO_OPTION,
                                            JOptionPane.QUESTION_MESSAGE) ==
                      JOptionPane.YES_OPTION) {
                  agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_DELETE_GROUP, owsn.GetMyDestination());
                  agtcmd.addCompUnicodeInputParam(getName());
                  if (sn != null) {
                      // Send command to server
                      try{
                          sn.SendAsynchCommand(agtcmd);
                      }
                      catch(InterruptedException x){
                          return;
                      } 
                      catch(ServiceNodeClosedException x){
                          return;
                      }
                      catch(ServiceNodeException x){
                          return;
                      }
                  }
              }
          }

      } else if (cmd.equals("rename")) {
          // rename the group
          String newname;

          newname=JOptionPane.showInputDialog(null,
                                              getNlsString("renameNewName"),
					      getNlsString("renameTitle"),
                                              JOptionPane.QUESTION_MESSAGE);
	  if (newname !=null) {
	      if (!isNameValid(newname)) {
		  Object[] badname = {getName(), newname};
		  JOptionPane.showMessageDialog(null,
						MessageFormat.format(getNlsString("errNameBad"),badname),
						getNlsString("TWGMSCSBrowseFrameTitle"),
						JOptionPane.ERROR_MESSAGE);
	      } else {
		  Object[] nameargs = {getName(), newname};
		  // verify that they Really want to rename the group
		  if (JOptionPane.showConfirmDialog(null,
						    MessageFormat.format(getNlsString("areYouSureRename"),nameargs),
						    getNlsString("TWGMSCSBrowseFrameTitle"),
						    JOptionPane.YES_NO_OPTION,
						    JOptionPane.QUESTION_MESSAGE) ==
                      JOptionPane.YES_OPTION) {
		      // they said yes, do the rename
		      agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_SET_GROUP_NAME, owsn.GetMyDestination());
		      agtcmd.addCompUnicodeInputParam(getName());
		      agtcmd.addCompUnicodeInputParam(newname);
		      if (sn != null) {
			  // Send command to server
			  try{
			      sn.SendAsynchCommand(agtcmd);
			  }
			  catch(InterruptedException x){
			      return;
			  } 
			  catch(ServiceNodeClosedException x){
			      return;
			  }
			  catch(ServiceNodeException x){
			      return;
			  }
		      }
		  }
	      }
	  }
      } else {
          // check the movegroup list
          found=false;
          if (faPopupNodeTags!=null && !faPopupNodeTags.isEmpty()) {
              for (i=0;i<faPopupNodeTags.size() && faPopupNodeTags.elementAt(i)!=null;i++) {
                  if (cmd.equals((String)faPopupNodeTags.elementAt(i))) {
                      // found the element
                      found=true;
                      // send the command to switch the group to the new node
                      if ((owsn != null) && (faPopupNodeNames.elementAt(i)!=null)) {
                          Object[] nameargs = {getName(), getOwner(), faPopupNodeNames.elementAt(i)};
                          if (JOptionPane.showConfirmDialog(null,
                                                        MessageFormat.format(getNlsString("areYouSureGN"), nameargs),
                                                        getNlsString("TWGMSCSBrowseFrameTitle"),
                                                        JOptionPane.YES_NO_OPTION,
                                                        JOptionPane.QUESTION_MESSAGE) ==
                                  JOptionPane.YES_OPTION) {
                              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_MOVE_GROUP, owsn.GetMyDestination());
                              agtcmd.addCompUnicodeInputParam(getName());
                              agtcmd.addCompUnicodeInputParam((String)faPopupNodeNames.elementAt(i));
                              if (sn != null) {
                                  // Send command to server
                                  try{
                                      sn.SendAsynchCommand(agtcmd);
                                  }
                                  catch(InterruptedException x){
                                      return;
                                  } 
                                  catch(ServiceNodeClosedException x){
                                      return;
                                  }
                                  catch(ServiceNodeException x){
                                      return;
                                  }
                              }
                          }
                      }
                      break;
                  }
              }
          }
          if (!found) {
              super.actionPerformed(evt);
          }
      }
    } // end actionPerformed
}

