package com.tivoli.twg.mscs;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.beans.*;

import com.tivoli.twg.mscs.*;

/** 
 * Object to represent a network interface for the cluster. The name
 * field is used for what MSCS calles "adapter", state and description
 * stay the same, and node, network, and address are added.
 */
public class TWGMSCSClusterNetInterfaceObject extends TWGMSCSClusterObject {
    String  address;
    String  network;
    String  node;

    public TWGMSCSClusterNetInterfaceObject(String nname, String nstate, String ndesc, 
                                            String naddress, String nnetwork, String nnode) {
        super(nname,nstate,ndesc);
        address=naddress;
        network=nnetwork;
        node=nnode;
    }

    public String getAdapter() {
        return getName();
    }

    public void setAddress(String naddress) {
        address=naddress;
    }

    public String getAddress() {
        return address;
    }

    public void setNetwork(String nnetwork) {
        network=nnetwork;
    }

    public void setNetwork(TWGMSCSClusterNetworkObject nnetwork) {
        if (nnetwork != null) {
            network=nnetwork.getName();
        } else {
            network=null;
        }
    }

    public String getNetwork() {
        return network;
    }

    public void setNode(String nnode) {
        node=nnode;
    }

    public void setNode(TWGMSCSClusterNodeObject nnode) {
        if (nnode != null) {
            node=nnode.getName();
        } else {
            node=null;
        }
    }

    public String getNode() {
        return node;
    }

    public int      getColumns() {
        return 6;
    }

    public String   getColumnHeader(int column) {
        switch (column) {
            case 0:
                return getNlsString("colnameNode");
            case 1:
                return getNlsString("colnameNetwork");
            case 2:
                return getNlsString("colnameState");
            case 3:
                return getNlsString("colnameAdapter");
            case 4:
                return getNlsString("colnameAddress");
            case 5:
                return getNlsString("colnameDesc");
            default:
                return getNlsString("stateUnknown");
        }
    }

    public String   getColumnString(int column) {
        switch (column) {
            case 0:
                if (getNode()!=null) {
                    return getNode();
                }
                break;
            case 1:
                if (getNetwork()!=null) {
                    return getNetwork();
                }
                break;
            case 2:
                if (getState()!=null) {
                    return getState();
                }
                break;
            case 3:
                if (getAdapter()!=null) {
                    return getAdapter();
                }
                break;
            case 4:
                if (getAddress()!=null) {
                    return getAddress();
                }
                break;
            case 5:
                if (getDescription()!=null) {
                    return getDescription();
                } else {
                    return new String("");
                }
            default:
                break;
        }
        return getNlsString("stateUnknown");
    }
}

