package com.tivoli.twg.mscs;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import com.tivoli.twg.engine.*;
import com.tivoli.twg.libs.*;
import com.tivoli.twg.libs.TWGConVersion;
import com.tivoli.twg.mscs.*;

/** 
 * Object to represent a resource in the cluster.
 */
public class TWGMSCSClusterResourceObject extends TWGMSCSClusterObject
                                          implements ActionListener
                           {
    private String  owner;  /* the node that currently owns this resource */
    private String  group;  /* the group that this resource is part of */
    private String  restype;/* what type of resource is this? */
    private JMenu         changemenu=null;
    private JMenuItem     fPopupOnline = null;
    private JMenuItem     fPopupOffline = null;
    private JMenuItem     fPopupFailure = null;
    private JMenuItem     fPopupDelete = null;
    private JMenuItem     fPopupRename = null;
    private Vector        faPopupGroups = null;
    private Vector        faPopupGroupNames = null;
    private Vector        faPopupGroupTags = null;

    public TWGMSCSClusterResourceObject(String nname, String nstate, String ndesc, String nowner,
                                        String ngroup, String nrestype) {
        super(nname,nstate,ndesc);
        owner=nowner;
        group=ngroup;
        restype=nrestype;
    }

    public void setOwner(String nowner) {
        owner=nowner;
    }
    
    public void setOwner(TWGMSCSClusterNodeObject nowner) {
        if (nowner != null) {
            owner=nowner.getName();
        } else {
            owner=null;
        }
    }

    public String getOwner() {
        return owner;
    }

    public void setGroup(String ngroup) {
        int i;

        group=ngroup;
        // if we're too low a version, skip this whole thing
        if ((TWGConVersion.getServerVersion()/100000)>3000 && (TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
            // fix the menus, if they exist
            if (faPopupGroupNames!=null && !faPopupGroupNames.isEmpty()) {
                for (i=0;i<faPopupGroupNames.size() && faPopupGroupNames.elementAt(i)!=null;i++) {
                    if (group.equals((String)faPopupGroupNames.elementAt(i))) {
                        // found the new group, grey it out
                        if (faPopupGroups!=null && !faPopupGroups.isEmpty()){
                            if (faPopupGroups.elementAt(i)!=null) {
                                ((JMenuItem)faPopupGroups.elementAt(i)).setEnabled(false);
                            }
                        }
                    } else {
                        // this is not the new group, grey it in
                        if (faPopupGroups!=null && !faPopupGroups.isEmpty()){
                            if (faPopupGroups.elementAt(i)!=null) {
                                ((JMenuItem)faPopupGroups.elementAt(i)).setEnabled(true);
                            }
                        }
                    }
                }
            }
        }
    }

    public void setGroup(TWGMSCSClusterGroupObject ngroup) {
        int i;

        if (ngroup != null) {
            group=ngroup.getName();
            // if we're too low a version, skip this whole thing
            if ((TWGConVersion.getServerVersion()/100000)>3000 && (TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                // fix the menus, if they exist
                if (faPopupGroupNames!=null && !faPopupGroupNames.isEmpty()) {
                    for (i=0;i<faPopupGroupNames.size() && faPopupGroupNames.elementAt(i)!=null;i++) {
                        if (group.equals((String)faPopupGroupNames.elementAt(i))) {
                            // found the new group, grey it out
                            if (faPopupGroups!=null && !faPopupGroups.isEmpty()){
                                if (faPopupGroups.elementAt(i)!=null) {
                                    ((JMenuItem)faPopupGroups.elementAt(i)).setEnabled(false);
                                }
                            }
                        } else {
                            // this is not the new group, grey it in
                            if (faPopupGroups!=null && !faPopupGroups.isEmpty()){
                                if (faPopupGroups.elementAt(i)!=null) {
                                    ((JMenuItem)faPopupGroups.elementAt(i)).setEnabled(true);
                                }
                            }
                        }
                    }
                }
            }
        } else {
            group=null;
        }
    }

    public String getGroup() {
        return group;
    }

    public void setResType(String nrestype) {
        restype=nrestype;
    }

    public void setResType(TWGMSCSClusterResTypeObject nrestype) {
        if (nrestype != null) {
            restype=nrestype.getName();
        } else {
            restype=null;
        }
    }

    public String getResType() {
        return restype;
    }

    public int      getColumns() {
        return 6;
    }

    public String   getColumnHeader(int column) {
        switch (column) {
            case 0:
                return getNlsString("colnameName");
            case 1:
                return getNlsString("colnameState");
            case 2:
                return getNlsString("colnameOwner");
            case 3:
                return getNlsString("colnameGroup");
            case 4:
                return getNlsString("colnameResType");
            case 5:
                return getNlsString("colnameDesc");
            default:
                return getNlsString("stateUnknown");
        }
    }

    public String   getColumnString(int column) {
        switch (column) {
            case 0:
                if (getName()!=null) {
                    return getName();
                }
                break;
            case 1:
                if (getState()!=null) {
                    return getState();
                }
                break;
            case 2:
                if (getOwner()!=null) {
                    return getOwner();
                }
                break;
            case 3:
                if (getGroup()!=null) {
                    return getGroup();
                }
                break;
            case 4:
                if (getResType()!=null) {
                    return getResType();
                }
                break;
            case 5:
                if (getDescription()!=null) {
                    return getDescription();
                } else {
                    return new String("");
                }
            default:
                break;
        }
        return getNlsString("stateUnknown");
    }

    public int      getPopupItemCount() {
        if (super.getPopupItemCount()>0) {
            return super.getPopupItemCount()+8;
        } else {
            return 7;
        }
    }

     public void setState(String nstate) {
         // have my superclass update my state
         super.setState(nstate);

         // if we're too low a version, skip this whole thing
         if ((TWGConVersion.getServerVersion()/100000)>3000 && (TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
             // update my menu items
             // turn all three off, then turn back on the right ones
             if (fPopupOnline!=null) {
                 fPopupOnline.setEnabled(false);
             }
             if (fPopupOffline!=null) {
                 fPopupOffline.setEnabled(false);
             }
             if (fPopupFailure!=null) {
                 fPopupFailure.setEnabled(false);
             }
             if (nstate.equals(getNlsString("stateOffline"))) {
                 if (fPopupOnline!=null) {
                     fPopupOnline.setEnabled(true);
                 }
             } else if (nstate.equals(getNlsString("stateOnline"))) {
                 if (fPopupOffline!=null) {
                     fPopupOffline.setEnabled(true);
                 }
                 if (fPopupFailure!=null) {
                     fPopupFailure.setEnabled(true);
                 }
             } else if (nstate.equals(getNlsString("stateFailed"))) {
                 if (fPopupOnline!=null) {
                     fPopupOnline.setEnabled(true);
                 }
                 if (fPopupOffline!=null) {
                     fPopupOffline.setEnabled(true);
                 }
             }
         }
     }

    public JMenuItem getPopupItem(int index) {
        int                       i;
        JMenuItem                 tGroupMI;
        Object[]                  nameargs = { null };
        String                    tGroupMIAction;
        TWGMSCSClusterObject      tfolder;
        TWGMSCSClusterGroupObject cgo;

        switch (index) {
            case 0:
                if (fPopupOnline==null) {
                    fPopupOnline=new JMenuItem(getNlsString("miBringonline"));
                    fPopupOnline.setActionCommand("bringonline");
                    fPopupOnline.addActionListener(this);
                    if (!(getState().equals(getNlsString("stateOffline"))) &&
                            !(getState().equals(getNlsString("stateFailed")))) {
                        fPopupOnline.setEnabled(false);
                    }
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupOnline.setEnabled(false);
                    }
                }
                return fPopupOnline;
            case 1:
                if (fPopupOffline==null) {
                    fPopupOffline=new JMenuItem(getNlsString("miTakeoffline"));
                    fPopupOffline.setActionCommand("takeoffline");
                    fPopupOffline.addActionListener(this);
                    if (!(getState().equals(getNlsString("stateOnline"))) &&
                            !(getState().equals(getNlsString("stateFailed")))) {
                        fPopupOffline.setEnabled(false);
                    }
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupOffline.setEnabled(false);
                    }
                }
                return fPopupOffline;
            case 2:
                if (fPopupFailure==null) {
                    fPopupFailure=new JMenuItem(getNlsString("miInitfail"));
                    fPopupFailure.setActionCommand("initfail");
                    fPopupFailure.addActionListener(this);
                    if (!(getState().equals(getNlsString("stateOnline")))) {
                        fPopupFailure.setEnabled(false);
                    }
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupFailure.setEnabled(false);
                    }
                }
                return fPopupFailure;
            case 3:
                // add the change groups submenu
                // build the list of groups and groupnames
                // first, clear out anything already there
                if (faPopupGroups==null) {
                    faPopupGroups=new Vector();
                } else {
                    faPopupGroups.removeAllElements();
                }
                if (faPopupGroupNames==null) {
                    faPopupGroupNames=new Vector();
                } else {
                    faPopupGroupNames.removeAllElements();
                }
                if (faPopupGroupTags==null) {
                    faPopupGroupTags=new Vector();
                } else {
                    faPopupGroupTags.removeAllElements();
                }
                // get a pointer to the groups folder
                tfolder=getParentWithServiceNode().findChild(getNlsString("objnameGroups"));
                cgo=(TWGMSCSClusterGroupObject)tfolder.getChild();
                if (cgo != null) {
                    i=0;
                    // loop through them and add them to the vectors
                    do {
                        nameargs[0] = cgo.getName();
                        tGroupMIAction=MessageFormat.format(new String("changegroup{0}"),nameargs);
                        tGroupMI=new JMenuItem(cgo.getName());
                        tGroupMI.setActionCommand(tGroupMIAction);
                        tGroupMI.addActionListener(this);
                        if (cgo.getName().equals(getGroup())) {
                            tGroupMI.setEnabled(false);
                        }
                        if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                            tGroupMI.setEnabled(false);
                        }
                        faPopupGroups.insertElementAt(tGroupMI, i);
                        faPopupGroupNames.insertElementAt(cgo.getName(), i);
                        faPopupGroupTags.insertElementAt(tGroupMIAction, i);
                        i++;
                        cgo=(TWGMSCSClusterGroupObject)cgo.getSibling();
                    } while ((cgo!=null) && (cgo!=tfolder.getChild()));
                }

                // now create the menu itself, and populate it
                changemenu=new JMenu(getNlsString("mChangegroup"));
                for (i=0;i<faPopupGroups.size() && faPopupGroups.elementAt(i)!=null;i++) {
                    changemenu.add((JMenuItem)faPopupGroups.elementAt(i));
                }
                return changemenu;

            case 4:
            case 7:
                // return null to have the adder add a separator instead
                return null;
            case 5:
                if (fPopupDelete==null) {
                    fPopupDelete=new JMenuItem(getNlsString("miDelete"));
                    fPopupDelete.setActionCommand("delete");
                    fPopupDelete.addActionListener(this);
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupDelete.setEnabled(false);
                    }
                }
                return fPopupDelete;
            case 6:
                if (fPopupRename==null) {
                    fPopupRename=new JMenuItem(getNlsString("miRename"));
                    fPopupRename.setActionCommand("rename");
                    fPopupRename.addActionListener(this);
                    if ((TWGConVersion.getServerVersion()/100000)<3000 || !(TWGConUser.currentHasPrivilege(TWGMSCSConstants.MODIFY_CLUSTER_ACCESS_TOKEN))) {
                        fPopupRename.setEnabled(false);
                    }
                }
                return fPopupRename;
            default:
                break;
        }
        if (index<super.getPopupItemCount()+8) {
            return super.getPopupItem(index-8);
        } else {
            // don't know what to add, so add a separator, I guess
            return null;
        }
    }

    public void actionPerformed( ActionEvent evt )
    {
      // Place menu and/or button handling code here.
      // Remove listener interface if neither of these are required.
      //         ....................
      String                               cmd = evt.getActionCommand();
      TWGMSCSClusterObjectWithServiceNode  owsn = null;
      ServiceNode                          sn = null;
      MSCSAgentCommand                     agtcmd = null;
      int                                  i;
      boolean                              found;

      owsn=getParentWithServiceNode();
      if (owsn!=null) {
          sn=owsn.getServiceNode();
      }

      if (cmd.equals("bringonline")) {
          // bring the resource online
          if (owsn != null) {
              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_ONLINE_RESOURCE, owsn.GetMyDestination());
              agtcmd.addCompUnicodeInputParam(getName());
              if (sn != null) {
                  // Send command to server
                  try{
                      sn.SendAsynchCommand(agtcmd);
                  }
                  catch(InterruptedException x){
                      return;
                  } 
                  catch(ServiceNodeClosedException x){
                      return;
                  }
                  catch(ServiceNodeException x){
                      return;
                  }
              }
          }

      } else if (cmd.equals("takeoffline")) {
          // take the resource offline
          if (owsn != null) {
              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_OFFLINE_RESOURCE, owsn.GetMyDestination());
              agtcmd.addCompUnicodeInputParam(getName());
              if (sn != null) {
                  // Send command to server
                  try{
                      sn.SendAsynchCommand(agtcmd);
                  }
                  catch(InterruptedException x){
                      return;
                  } 
                  catch(ServiceNodeClosedException x){
                      return;
                  }
                  catch(ServiceNodeException x){
                      return;
                  }
              }
          }
      
      } else if (cmd.equals("initfail")) {
          // initiates a resource failure
          if (owsn != null) {
              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_FAIL_RESOURCE, owsn.GetMyDestination());
              agtcmd.addCompUnicodeInputParam(getName());
              if (sn != null) {
                  // Send command to server
                  try{
                      sn.SendAsynchCommand(agtcmd);
                  }
                  catch(InterruptedException x){
                      return;
                  } 
                  catch(ServiceNodeClosedException x){
                      return;
                  }
                  catch(ServiceNodeException x){
                      return;
                  }
              }
          }

      } else if (cmd.equals("delete")) {
          // delete the resource
          if (owsn != null) {
              Object[] nameargs = {getName()};
              if (JOptionPane.showConfirmDialog(null,
                                            MessageFormat.format(getNlsString("areYouSureDelRes"), nameargs),
                                            getNlsString("TWGMSCSBrowseFrameTitle"),
                                            JOptionPane.YES_NO_OPTION,
                                            JOptionPane.QUESTION_MESSAGE) ==
                      JOptionPane.YES_OPTION) {
                  agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_DELETE_RESOURCE, owsn.GetMyDestination());
                  agtcmd.addCompUnicodeInputParam(getName());
                  if (sn != null) {
                      // Send command to server
                      try{
                          sn.SendAsynchCommand(agtcmd);
                      }
                      catch(InterruptedException x){
                          return;
                      } 
                      catch(ServiceNodeClosedException x){
                          return;
                      }
                      catch(ServiceNodeException x){
                          return;
                      }
                  }
              }
          }

      } else if (cmd.equals("rename")) {
          // rename the resource
          String newname;

          newname=JOptionPane.showInputDialog(null,
					      getNlsString("renameNewName"),
                                              getNlsString("renameTitle"),
                                              JOptionPane.QUESTION_MESSAGE);
          if (newname != null) { // no? then the user hit "cancel"
	      if (!isNameValid(newname)) {
		  Object[] badname = {getName(), newname};
		  JOptionPane.showMessageDialog(null,
						MessageFormat.format(getNlsString("errNameBad"),badname),
						getNlsString("TWGMSCSBrowseFrameTitle"),
						JOptionPane.ERROR_MESSAGE);
	      } else {
		  Object[] nameargs = {getName(), newname};
		  // verify that they Really want to rename the resource
		  if (JOptionPane.showConfirmDialog(null,
						    MessageFormat.format(getNlsString("areYouSureRename"),nameargs),
						    getNlsString("TWGMSCSBrowseFrameTitle"),
						    JOptionPane.YES_NO_OPTION,
						    JOptionPane.QUESTION_MESSAGE) ==
                      JOptionPane.YES_OPTION) {
		      // they said yes, do the rename
		      agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_SET_RESOURCE_NAME, owsn.GetMyDestination());
		      agtcmd.addCompUnicodeInputParam(getName());
		      agtcmd.addCompUnicodeInputParam(newname);
		      if (sn != null) {
			  // Send command to server
			  try{
			      sn.SendAsynchCommand(agtcmd);
			  }
			  catch(InterruptedException x){
			      return;
			  } 
			  catch(ServiceNodeClosedException x){
			      return;
			  }
			  catch(ServiceNodeException x){
			      return;
			  }
		      }
		  }
	      }
	  }

      } else {
          // check the changegroup list
          found=false;
          if (faPopupGroupTags!=null && !faPopupGroupTags.isEmpty()) {
              for (i=0;i<faPopupGroupTags.size() && faPopupGroupTags.elementAt(i)!=null;i++) {
                  if (cmd.equals((String)faPopupGroupTags.elementAt(i))) {
                      // found the element
                      found=true;
                      // send the command to switch the resource to the new group
                      if ((owsn != null) && (faPopupGroupNames.elementAt(i)!=null)) {
                          Object[] nameargs = {getName(), getGroup(), faPopupGroupNames.elementAt(i)};
                          if (JOptionPane.showConfirmDialog(null,
                                                        MessageFormat.format(getNlsString("areYouSureRG"), nameargs),
                                                        getNlsString("TWGMSCSBrowseFrameTitle"),
                                                        JOptionPane.YES_NO_OPTION,
                                                        JOptionPane.QUESTION_MESSAGE) ==
                                  JOptionPane.YES_OPTION) {
                              agtcmd = new MSCSAgentCommand(MSCSAgentCommand.MSCSAGT_CHANGE_RESOURCE_GROUP, owsn.GetMyDestination());
                              agtcmd.addCompUnicodeInputParam(getName());
                              agtcmd.addCompUnicodeInputParam((String)faPopupGroupNames.elementAt(i));
                              if (sn != null) {
                                  // Send command to server
                                  try{
                                      sn.SendAsynchCommand(agtcmd);
                                  }
                                  catch(InterruptedException x){
                                      return;
                                  } 
                                  catch(ServiceNodeClosedException x){
                                      return;
                                  }
                                  catch(ServiceNodeException x){
                                      return;
                                  }
                              }
                          }
                      }
                      break;
                  }
              }
          }
          if (!found) {
              super.actionPerformed(evt);
          }
      }
    } // end actionPerformed
}

