/*
 * Created on Nov 12, 2003
 *
 * 
 */
package com.ibm.sysmgt.sdk.remote.test;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import junit.framework.Assert;
import junit.framework.TestCase;

import com.ibm.sysmgt.sdk.remote.Attribute;
import com.ibm.sysmgt.sdk.remote.DirectorEvent;
import com.ibm.sysmgt.sdk.remote.DirectorEventFilter;
import com.ibm.sysmgt.sdk.remote.DirectorEventLog;
import com.ibm.sysmgt.sdk.remote.DirectorRemoteSdk;
import com.ibm.sysmgt.sdk.remote.DirectorServer;
import com.ibm.sysmgt.sdk.remote.Log;
import com.ibm.sysmgt.sdk.remote.LoginSession;
import com.ibm.sysmgt.sdk.remote.ManagedGroup;
import com.ibm.sysmgt.sdk.remote.ManagedSystem;
import com.ibm.sysmgt.sdk.remote.Task;
import com.ibm.sysmgt.sdk.remote.UnknownManagerException;
import com.tivoli.twg.libs.DataValue;
import com.tivoli.twg.libs.TWGByteArray;
import com.tivoli.twg.libs.TWGEnvironment;
import com.tivoli.twg.libs.TWGLong;

/**
 * @author Dave Wormald
 *
 * 
 */
public class TestDirectorEvents extends SetupToUseDirectorServer {

	/**
	 * 
	 */
	public TestDirectorEvents() {
		super();

	}

	/**
	 * @param arg0
	 */
	public TestDirectorEvents(String arg0) {
		super(arg0);

	}

	public void testgetEventsForSystemUsingFilters() {
		try {
			int i = 0;

			List systems = ManagedSystem.getAllSystems(s);
			assertNotNull(systems);
			assertTrue(systems.size() > 0);

			DirectorEventLog log =
				(DirectorEventLog) Log.getLog(DirectorEventLog.EVENT_LOG_NAME);
			assertNotNull(log);

			List filters = DirectorEventFilter.getAllFilters(s);
			assertNotNull(filters);
			assertTrue(filters.size() > 0);

			DirectorEventFilter filterForAllEvents = null;
			// find the All Events Filter
			for (i = 0; i < filters.size(); i++) {
				if (((DirectorEventFilter) filters.get(i))
					.getName()
					.equals("All Events")) {
					filterForAllEvents = (DirectorEventFilter) filters.get(i);
				}

			}
			assertNotNull(filterForAllEvents);

			// process through all the systems
			for (i = 0; i < systems.size(); i++) {
				ManagedSystem currentSystem = (ManagedSystem) systems.get(i);
				assertNotNull(currentSystem);

				// get events using the filter
				List eventListFromFilter =
					log.getEventsForSystemUsingFilter(
						s,
						filterForAllEvents,
						currentSystem);

				// get all events
				List allEventsforSys = log.getEventsForSystem(s, currentSystem);

				//make sure the filter returned all events
				assertTrue(
					eventListFromFilter.size() == allEventsforSys.size());

				println(
					""
						+ eventListFromFilter.size()
						+ " events for "
						+ currentSystem);

			}

		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testgetEventsUsingFilters() {
		try {
			DirectorEventLog log =
				(DirectorEventLog) Log.getLog(DirectorEventLog.EVENT_LOG_NAME);
			assertNotNull(log);

			List filters = DirectorEventFilter.getAllFilters(s);
			assertNotNull(filters);
			assertTrue(filters.size() > 0);

			DirectorEventFilter filterForAllEvents = null;
			// find the All Events Filter
			for (int i = 0; i < filters.size(); i++) {
				if (((DirectorEventFilter) filters.get(i))
					.getName()
					.equals("All Events")) {
					filterForAllEvents = (DirectorEventFilter) filters.get(i);
				}

			}
			assertNotNull(filterForAllEvents);

			// get events using the filter
			List eventListFromFilter =
				log.getEventUsingFilter(s, filterForAllEvents);

			// get all events
			List allEvents = log.getAllEvents(s);

			//make sure the filter returned all events
			assertTrue(eventListFromFilter.size() == allEvents.size());

		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testgetAllEvents() {
		try {
			/* 
			 * Static methods in the RemoteSdk classes return specific 
			 * instances of objects, like the Event Log 
			 * 
			 */
			DirectorEventLog evlog = (DirectorEventLog) Log.getLog("EventLog");
			List eventList = evlog.getAllEvents(s);

			if (eventList.size() > 0) {
				DirectorEvent dirEvent = (DirectorEvent) eventList.get(0);

				Map eventDetails = dirEvent.getDetails(s);
				println(
					"Events Details for Event'"
						+ dirEvent.getEventType()
						+ "': "
						+ eventDetails);

			}

		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testgetFilters() {
		try {
			DirectorEventLog log =
				(DirectorEventLog) Log.getLog(DirectorEventLog.EVENT_LOG_NAME);
			assertNotNull(log);

			List filters = DirectorEventFilter.getAllFilters(s);
			assertNotNull(filters);
			assertTrue(filters.size() > 0);
			System.out.println(filters);

		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testgetLog() {
		try {
			DirectorEventLog log =
				(DirectorEventLog) Log.getLog(DirectorEventLog.EVENT_LOG_NAME);
			assertNotNull(log);
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testgetLogList() {
		try {
			List logs = Log.getLogList();
			assertNotNull(logs);
			assertTrue(logs.size() > 0);
			println("Log list=" + logs);
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

}
