/******************************************************************************
 * Licensed Materials - Property of IBM                                       *
 *                                                                            *
 * (C) Copyright by IBM 2003.  All Rights Reserved.                           *
 *****************************************************************************/

package com.ibm.sysmgt.sdk.tools;

import com.tivoli.twg.engine.InvalidObjectIDException;
import com.tivoli.twg.engine.ManagedObjectFactoryActivateException;
import com.tivoli.twg.engine.ManagedObjectFactoryDiscoverException;
import com.tivoli.twg.engine.TWGByteArrayInputStream;
import com.tivoli.twg.engine.TWGByteArrayOutputStream;
import com.tivoli.twg.engine.TWGImageSet;
import com.tivoli.twg.engine.TWGManagedObject;
import com.tivoli.twg.engine.TWGManagedObjectFactory;
import com.tivoli.twg.engine.TWGManagedObjectFingerprint;
import com.tivoli.twg.engine.TWGMofInstanceOf;
import com.tivoli.twg.engine.TWGPersistentObjectDictionary;
import com.tivoli.twg.engine.TWGPersistentObjectSaveException;
import com.tivoli.twg.engine.TWGPersistentObjectRestoreException;
import com.tivoli.twg.engine.TWGPing;
import com.tivoli.twg.libs.QueueBatch;
import com.tivoli.twg.log.TWGRas;


/**
 * Factory class for user defined managed objects.
 *
 * @see com.tivoli.twg.engine.TWGManagedObjectFactory
 */
public class UserDefinedManagedSystemFactory extends TWGManagedObjectFactory
{
   /* Our classname */
   private static final String CLASSNAME =
       "com.ibm.sysmgt.sdk.tools.UserDefinedManagedSystemFactory";

   /* Name of TWGManagedObject class supported by factory */
   public static final String MO_CLASS =
       "com.ibm.sysmgt.sdk.tools.UserDefinedManagedSystem";

   private static final int CLASS_VERSION = 1;

   /* Name of our NLS resources class */
   private static final String ADDRESSNLSCLASS =
     "com.ibm.sysmgt.sdk.tools.SDKCLIResourceBundle";

   /* Name ID for our native managed objects */
   private static final String MONAME = "User defined managed systems";

   /* Reference to TWGNativeManagedObjectFactory instance */
   private static UserDefinedManagedSystemFactory factory = null;

   /* Flag indicating if we're active */
   private boolean is_active;

   /* Current version stamp */
   private int current_version;

   ///* The queue that handles the ping requests we need to process */
   //private static QueueBatch queue = null;

   /* our list of images for the group filter */
   protected static final String[] FILTER_SET =
   {
     /** Small filter icon */
     "/com/ibm/sysmgt/sdk/tools/images/user16on.gif",
     /** Large filter icon */
     "/com/BobCo/webobject/images/web32on.gif"
   };

   /** our list of images for the User Defined Managed Systems */
   protected static final String[] IMAGE_SET =
   {
     /** User Defined Managed System Image set icon index - indeterminent (large)         */
     "/com/BobCo/webobject/images/web32ind.gif",
     /** User Defined Managed System Image set icon index - on-line (large)               */
     "/com/BobCo/webobject/images/web32on.gif",
     /** User Defined Managed System Image set icon index - off-line (large)              */
     "/com/BobCo/webobject/images/web32off.gif",
     /** User Defined Managed System Image set icon index - on-line/error (large)         */
     "/com/BobCo/webobject/images/web32one.gif",
     /** User Defined Managed System Image set icon index - off-line/error (large)        */
     "/com/BobCo/webobject/images/web32offe.gif",
     /** User Defined Managed System Image set icon index - unlicensed device (large)     */
     "/com/BobCo/webobject/images/web32unl.gif",
     /** User Defined Managed System Image set icon index - indeterminent (small)         */
     "/com/ibm/sysmgt/sdk/tools/images/user16ind.gif",
     /** User Defined Managed System Image set icon index - on-line (small)               */
     "/com/ibm/sysmgt/sdk/tools/images/user16on.gif",
     /** User Defined Managed System Image set icon index - off-line (small)              */
     "/com/ibm/sysmgt/sdk/tools/images/user16off.gif",
     /** User Defined Managed System Image set icon index - on-line/error (small)         */
     "/com/ibm/sysmgt/sdk/tools/images/user16on.gif",
     /** User Defined Managed System Image set icon index - off-line/error (small)        */
     "/com/ibm/sysmgt/sdk/tools/images/user16off.gif",
     /** User Defined Managed System Image set icon index - unlicensed device (small)     */
     "/com/ibm/sysmgt/sdk/tools/images/user16ind.gif",
   };



   /**
    * INTERNAL USE ONLY
    * <p>
    * Default constructor
    * @exception ClassNotFoundException if MO class not loaded
    */
   public UserDefinedManagedSystemFactory() throws ClassNotFoundException
   {
      this(MO_CLASS);
   }

   /**
    * INTERNAL USE ONLY
    * <p>
    * Constructor for creating a subclass of this factory.
    * @param managedObjectClassName - TWGManagedObject classname associated with factory
    * @exception ClassNotFoundException if MO class not loaded
    */
   public UserDefinedManagedSystemFactory(String managedObjectClassName)
       throws ClassNotFoundException
   {
      super(managedObjectClassName);
      TWGRas.entryExit(TWGRas.SDK, "UserDefinedManagedSystemFactory: constructor entry");
      factory = this;
      is_active = false;
      current_version = 0;

      /* Set discovery supported and setup discovery queue */
      SetDiscoverySupported(false);
      //SetDiscoverySupported(true);

      //create the queue that will handle discovery requests */
      //if (queue == null)
      //{
      //    queue = new QueueBatch("UserDefinedDiscoveryQueue");
      //}

      // Set the ping type to use.
      setPingType(TWGPing.PING_TYPE_NOT_SUPPORTED);

      // /* 8 hrs */
      // setMorphInterval(8 * 60 * 60 * 1000);

      setAddObjectClassname("com.ibm.sysmgt.sdk.tools.SDKCLICreatePanel");
      // setAddObjectClassname("null");
      setEditObjectClassname(null);

      try
      {
         // set up our image set
         TWGImageSet userDefinedFilterImageSet =
             TWGImageSet.FindImageSet(UserDefinedManagedSystemFactory.FILTER_SET);
         TWGImageSet userDefinedObjectImageSet =
             TWGImageSet.FindImageSet(UserDefinedManagedSystemFactory.IMAGE_SET);
         try
         {
            setImageSet((int)userDefinedObjectImageSet.PersistID());
            setFilterImageSet((long)userDefinedFilterImageSet.PersistID());
         }
         catch (InvalidObjectIDException exception)
         {
           TWGRas.error(TWGRas.SDK,
                        "UserDefinedManagedSystemFactory constructor: error setting image set.");
         }
      }
      catch (TWGPersistentObjectSaveException exception)
      {
        TWGRas.error(TWGRas.SDK,
                     "UserDefinedManagedSystemFactory constructor: error saving image set.");
      }

      setName(MONAME, ADDRESSNLSCLASS);
   }

   /**
    * Get instance of native managed object factory
    * @return TWGSNMPDeviceObjectFactory instance
    */
   public static UserDefinedManagedSystemFactory GetFactory()
   {
      return factory;
   }

   /**
    * INTERNAL USE ONLY
    * <p>
    * Activate the topology processing.
    * @exception ManagedObjectFactoryActivateException if error
    */
   protected void Activate() throws ManagedObjectFactoryActivateException
   {
      if (is_active == false)
      {
        // do something here, then set is_active to true
        is_active = true;
      }

      if (is_active == false)
         //throw new ManagedObjectFactoryActivateException("User defined factory activate failed - " + exception);
         throw new ManagedObjectFactoryActivateException("User defined factory activate failed - ");
   }

   /**
    * Deactivate the topology processing
    */
   public void Deactivate()
   {
     if (is_active)
     {
       is_active = false;
     }
   }

   /**
    * Start ping
    * @return boolean TRUE: Ping function supported
    */
   public boolean startPing()
   {
      return true;
   }

   /**
    * Test a managed object instance for inclusion into a managed object
    * factory class.  This method is used during managed object morphing.
    * It is used by TWGEngine processes to determine the lowest level
    * managed object factory class to which a managed object instace belongs.
    * This method should be inplemented by all managed object factory
    * extension classes.
    * @param mofInstanceOf Reference to TWGMofInstanceOf callback interface object.
    * @param mo Reference to managed object instance to be tested.
    * @return byte see TWGManagedObjectFactory.
    */
   public byte instanceOf(TWGMofInstanceOf mofInstanceOf,
                          TWGManagedObject managedObject)
   {
      if (managedObject instanceof UserDefinedManagedSystem)
      {
         /* It is one in our chain... */
         // we don't need to do anything fancy here... we are the top of
         // the extension chain - ie. we are directly subclassed from
         // TWGManagedObject.  Classes lower on the extension chain would
         // have to query the device to check for the correct support.
         return TWGManagedObjectFactory.INSTANCE_OF_YES;
      }
      else
      {
         /* It is not in our inheritance tree */
         return TWGManagedObjectFactory.INSTANCE_OF_NO;
      }
   }

   /**
    * Get managed object by managed object fingerprint
    * @param fingerprint - managed object fingerprint
    * @return managed object, or null if not found
    */
   protected TWGManagedObject getManagedObjectByFingerprint(TWGManagedObjectFingerprint fingerprint)
   {
      TWGManagedObject managedObject = null;
      return managedObject;
   }

   /**
    * Start a discovery pass: operation should return quickly, with
    * actual discovery logic not being completed before the method
    * returns.  The default method does nothing.
    * @exception ManagedObjectFactoryDiscoverException if error starting
    *   discovery pass
    */
   public void StartDiscoveryPass() throws ManagedObjectFactoryDiscoverException
   {
     TWGRas.entryExit(TWGRas.SDK, "UserDefinedManagedSystemFactory.StartDiscoveryPass(): entry");


     //UserDefinedDiscoveryRequest qe =
     //  new UserDefinedDiscoveryRequest(TWGManagedObject.getAllManagedObjects());
     //queue.enqueue(qe);
   }


}
