/* file:  monitors\com\apidll.cpp */
// Licensed Materials - Property of IBM
//
// OCO Source Materials
// 
// 09N6996
//
// (C) Copyright IBM Corp. 1999, 2002 All Rights Reserved
//
// The source code for this program is not published or other-
// wise divested of its trade secrets, irrespective of what has
// been deposited with the U.S. Copyright Office.
// #include <stdio.h>
#include "apidll.hpp"         // include our TWGMonAPIDLL header

//====================
// TWGMonAPIDLL class
//====================
TWGMonAPIDLL::TWGMonAPIDLL(char *dllName)
   :DynamicLinkLibrary(dllName),
   DllInit(NULL),
   DllTerm(NULL),
   DllGetNodeList(NULL),
   DllFreeNodeList(NULL),
   DllGetHandle(NULL),
   DllReleaseHandle(NULL),
   DllGetData(NULL),
   DllFreeData(NULL),
   DllGetMetaData(NULL),
   DllFreeMetaData(NULL),
   DllLogDebug(NULL),
   lInit(RC_INVALID_OUTPUT)
{
   // If loaded successfully, do initialization
   if (IsLoaded())
   {
      DllInit = (long (APIENTRY *)( ))
                        LoadProcAddr("TWGMonAPIInit");
      DllTerm = (long (APIENTRY *)( ))
                        LoadProcAddr("TWGMonAPITerm");
      DllGetNodeList = (AttributeNodeList MONFAR *(APIENTRY *)(AttributePath MONFAR *))
                        LoadProcAddr("TWGMonAPIGetNodeList");
      DllFreeNodeList = (long (APIENTRY *)(AttributeNodeList MONFAR *))
                        LoadProcAddr("TWGMonAPIFreeNodeList");
      DllGetHandle = (MonHandle MONFAR *(APIENTRY *)(AttributePath MONFAR *))
                        LoadProcAddr("TWGMonAPIGetHandle");
      DllReleaseHandle = (long (APIENTRY *)(MonHandle MONFAR *))
                        LoadProcAddr("TWGMonAPIReleaseHandle");
      DllGetData = (MonData MONFAR *(APIENTRY *)(ULONG))
                        LoadProcAddr("TWGMonAPIGetData");
      DllFreeData = (long (APIENTRY *)(MonData MONFAR *))
                        LoadProcAddr("TWGMonAPIFreeData");
      DllGetMetaData = (MonMetaData MONFAR *(APIENTRY *)(ULONG))
                        LoadProcAddr("TWGMonAPIGetMetaData");
      DllFreeMetaData = (long (APIENTRY *)(MonMetaData MONFAR *))
                        LoadProcAddr("TWGMonAPIFreeMetaData");
      DllLogDebug = (BOOL (APIENTRY *)(char MONFAR *))
                        LoadProcAddr("TWGMonAPILogDebug");

      // check to see that all of out entry points were found
      if (DllInit && DllTerm && DllGetNodeList && DllFreeNodeList &&
          DllGetHandle && DllReleaseHandle && DllGetData &&
          DllFreeData && DllGetMetaData && DllFreeMetaData && DllLogDebug)
      {
#ifdef _DEBUG
         printf("%s - entry points were loaded\n", dllName);
#endif
         // besure we are using the locally defined timezone info
         tzset();
         // initialize the Monitor Agent APIs
         lInit = init();
         // printf("TWGMonAPIInit() returned %d\n", lInit);
      }
#ifdef _DEBUG
      else
      {
         printf("%s - entry points were not loaded\n", dllName);
         printf("TWGMonAPIInit = %p\n", DllInit);
         printf("TWGMonAPITerm = %p\n", DllTerm);
         printf("TWGMonAPIGetNodeList  = %p\n", DllGetNodeList);
         printf("TWGMonAPIFreeNodeList = %p\n", DllFreeNodeList);
         printf("TWGMonAPIGetHandle = %p\n", DllGetHandle);
         printf("TWGMonAPIReleaseHandle = %p\n", DllReleaseHandle);
         printf("TWGMonAPIGetData = %p\n", DllGetData);
         printf("TWGMonAPIFreeData = %p\n", DllFreeData);
         printf("TWGMonAPIGetMetaData = %p\n", DllGetMetaData);
         printf("TWGMonAPIFreeMetaData = %p\n", DllFreeMetaData);
         printf("TWGMonAPILogDebug = %p\n", DllLogDebug);
      }
#endif
   }
#ifdef _DEBUG
   else
   {
      printf("%s - module was not loaded\n", dllName);
   }
#endif
}

TWGMonAPIDLL::~TWGMonAPIDLL()
{
   term();
}

long TWGMonAPIDLL::init( )
{
   if (DllInit)
   {
      return (*DllInit)( );
   }
   else
   {
      return RC_INVALID_CMD;
   }
}

long TWGMonAPIDLL::term( )
{
   if (DllTerm)
   {
      return (*DllTerm)( );
   }
   else
   {
      return RC_INVALID_CMD;
   }
}

BOOL TWGMonAPIDLL::isOK()
{
   return (lInit == RC_OK);
}

AttributeNodeList MONFAR *TWGMonAPIDLL::getNodeList( AttributePath MONFAR *pAPath)
{
   if (isOK() && (DllGetNodeList))
   {
      return (*DllGetNodeList)(pAPath);
   }
   else
   {
      return NULL;
   }
}

long MONFAR TWGMonAPIDLL::freeNodeList( AttributeNodeList MONFAR *pANList)
{
   if (isOK() && (DllFreeNodeList))
   {
      return (*DllFreeNodeList)(pANList);
   }
   else
   {
      return RC_INVALID_CMD;
   }
}

MonHandle MONFAR *TWGMonAPIDLL::getHandle( AttributePath MONFAR *pAPath)
{
   if (isOK() && (DllGetHandle))
   {
      return (*DllGetHandle)(pAPath);
   }
   else
   {
      return NULL;
   }
}

long TWGMonAPIDLL::releaseHandle( MonHandle MONFAR *mHandle)
{
   if (isOK() && (DllReleaseHandle))
   {
      return (*DllReleaseHandle)(mHandle);
   }
   else
   {
      return RC_INVALID_CMD;
   }
}

MonData MONFAR *TWGMonAPIDLL::getData( ULONG handle)
{
   if (isOK() && (DllGetData))
   {
      return (*DllGetData)(handle);
   }
   else
   {
      return NULL;
   }
}

long TWGMonAPIDLL::freeData( MonData MONFAR *mData)
{
   if (isOK() && (DllFreeData))
   {
      return (*DllFreeData)(mData);
   }
   else
   {
      return RC_INVALID_CMD;
   }
}

MonMetaData MONFAR *TWGMonAPIDLL::getMetaData( ULONG handle)
{
   if (isOK() && (DllGetMetaData))
   {
      return (*DllGetMetaData)(handle);
   }
   else
   {
      return NULL;
   }
}

long TWGMonAPIDLL::freeMetaData( MonMetaData MONFAR *mData)
{
   if (isOK() && (DllFreeMetaData))
   {
      return (*DllFreeMetaData)(mData);
   }
   else
   {
      return RC_INVALID_CMD;
   }
}

BOOL TWGMonAPIDLL::logDebug( char MONFAR *str)
{
   if (isOK() && (DllLogDebug))
   {
      return (*DllLogDebug)(str);
   }
   else
   {
      return FALSE;
   }
}


