#ifndef BOBCBASE_HPP
#define BOBCBASE_HPP

/****************************************************************************/
/* Name of service node for base service                                    */
/****************************************************************************/
#define BOBCO_SVCNODE_NAME    "BobCoB"

/****************************************************************************/
/* BobCo's command codes                                                    */
/****************************************************************************/
#define BOBCO_GET_LOCAL_FIXED_DRIVES 0xB0BC0001L /* Get all local fixed drvs */
#define BOBCO_GET_DIRECTORY_CONTENTS 0xB0BC0002L /* Get directory contents   */

#define RETURN_OKAY                      0
#define RETURN_UNRECOGNIZED_COMMAND_CODE 1
#define RETURN_FIND_FAILED               2

#ifdef _Windows
/* Provide FTIME and FDATE compatable structures                            */
typedef struct _FTIME {        
   USHORT   twosecs : 5;
   USHORT   minutes : 6;
   USHORT   hours   : 5;
} FTIME;

typedef struct _FDATE {        
   USHORT   day     : 5;
   USHORT   month   : 4;
   USHORT   year    : 7;
} FDATE;

class FileDateTime {
public:
   FTIME ftime;
   FDATE fdate;

   FileDateTime(FILETIME *ft) {
      setDateTime(ft);
   }

   FileDateTime() {}

   void setDateTime(FILETIME *ft) {
      FILETIME tft;
      FileTimeToLocalFileTime(ft, &tft);
      FileTimeToDosDateTime(&tft, (LPWORD)&fdate, (LPWORD)&ftime);
   }
};
#endif

#ifdef __NETWARE_386__
// redefine these since WATCOM puts them as 4 bytes instead of 2!!!
#define FTIME USHORT
#define FDATE USHORT
#endif

#define MAXPATH 260

#pragma pack(1)

// Structure for putting file information into a Command output parm
typedef struct _CMD_FILEINFO {
   DWORD  fileAttributes;
   DWORD  sizeLow;
   DWORD  sizeHigh;
   USHORT dosDate;
   USHORT dosTime;
   CHAR   fileName[(MAXPATH+1) * 2];
} CMD_FILEINFO;

#pragma pack()

// Class for creating file information 
class FileInfo {
public:
   enum {
      READONLY  = 0x00000001, // read only attribute
      HIDDEN    = 0x00000002, // hidden attribute 
      SYSTEM    = 0x00000004, // system attribute 
      DIRECTORY = 0x00000010, // directory attribute 
      ARCHIVE   = 0x00000020, // archive attribute 
      FIXED_FILE_INFO_LENGTH = (sizeof(DWORD)*3) + (sizeof(USHORT)*2)
   };

   ULONG        fileInfoLength;
   CMD_FILEINFO cmdFileInfo;

   DWORD        fileAttributes;
   FileDateTime creationTime;
   FileDateTime lastAccessTime;
   FileDateTime lastWriteTime;
   DWORD        fileSizeHigh;
   DWORD        fileSizeLow;
   CHAR         fileName[ MAXPATH ];

#ifdef _Windows
   FileInfo() {}

   FileInfo(WIN32_FIND_DATA *fd) {
      setInfo(fd);
   }

   void setInfo(WIN32_FIND_DATA *fd);
#endif
};

#define FIXED_CMD_FILEINFO_LENGTH (sizeof(DWORD)*3 + sizeof(USHORT)*2)

#endif
