############################################################################
# Makefile Includes for Watcom 10.6
############################################################################


!ifndef WATCOM106DIR
!ERROR WATCOM106DIR environment variable needs to be set to Watcom C/C++ V10.6 directory
!endif


############################################################################
# Definitions for tool flags and macros:
#     In this section, we define the common flags and macros to help
# maintain some consistency on the compiling and linking of the code in
# the makefile.  In general, this area will support a command-line
# providable options "db" in order to select to build the code with
# "debug" information, or without it (the default).  To active this
# using nmake, just enter "nmake db=".
############################################################################
LINKNLM=preplnk.exe
LINK=wlink option quiet
LIBMAN=wlib.exe
CC=wpp386.exe
CC_ONLY=wcc386.exe
HC=hcrtf.exe -xn
RC=$(WATCOM106DIR)\binp\rc.exe -r

# WATCOM C++32 Optimizing Compiler  Version 10.6
#    -bm             build target is a multi-thread environment
#    -bd             build target is a dynamic link library (DLL)
#    -bw             build target is a default windowing application
#    -bt[=<os>]      build target is operating system <os>
#    -db             generate browsing information
#    -d1             line number debugging information
#    -d2             full symbolic debugging information
#    -d3             full symbolic debugging with unreferenced type names
#    -d+             allow extended -d macro definitions
#    -d<name>[=text] precompilation #define name [text]
#    -e<number>      set error limit number
#    -ei             force enums to be type int
#    -en             emit routine names in the code segment
#    -ep[<number>]   call prologue hook routine with number stack bytes available
#    -ee             call epilogue hook routine
#    -ew             alternate error message formatting
#    -ez             generate PharLap EZ-OMF object files
#    -fh             use pre-compiled headers
#    -fhq            do not display pre-compiled header activity warnings
#    -fh=<file_name> set pre-compiled header file name
#    -fi=<file_name> force file_name to be included
#    -fo=<file_name> set object or preprocessor output file name
#    -fr=<file_name> set error file name
#    -ft             check for truncated versions of file names
#    -fx             do not check for truncated versions of file names
#    -fpc            calls to floating-point library
#    -fpd            enable Pentium FDIV check
#    -fpi            inline 80x87 instructions with emulation
#    -fpi87          inline 80x87 instructions
#    -fpr            generate backward compatible 80x87 code
#    -fp2            generate 287 floating-point code
#    -fp3            generate 387 floating-point code
#    -fp5            optimize floating-point for Pentium
#    -g=<codegroup>  set code group name
#    -h{w,d,c}       set debug format (Watcom,Dwarf,Codeview)
#    -i=<directory>  another include directory
#    -j              change char default from unsigned to signed
#    -m{s,m,c,l,f}   memory model (Small,Medium,Compact,Large,Flat)
#    -3r             386 register calling conventions
#    -3s             386 stack calling conventions
#    -4r             486 register calling conventions
#    -4s             486 stack calling conventions
#    -5r             Pentium register calling conventions
#    -5s             Pentium stack calling conventions
#    -7              inline 387 instructions
#    -nc=<name>      set code class name
#    -nd=<name>      set data segment name
#    -nm=<name>      set module name
#    -nt=<name>      set name of text segment
#    -o{a,c,d,e,f[+],i,l[+],m,n,o,p,r,s,t,u,x,z} control optimization
#        a           -> relax aliasing constraints
#        c           -> disable <call followed by return> to <jump> optimization
#        d           -> disable all optimizations
#        e[=<num>]   -> expand user functions inline. <num> controls max size
#        f           -> generate traceable stack frames as needed
#        f+          -> always generate traceable stack frames
#        i           -> expand intrinsic functions inline
#        l           -> enable loop optimizations
#        l+          -> enable loop unrolling optimizations
#        m           -> generate inline 80x87 code for math functions
#        n           -> allow numerically unstable optimizations
#        o           -> continue compilation if low on memory
#        p           -> generate consistent floating-point results
#        r           -> reorder instructions for best pipeline usage
#        s           -> favor code size over execution time in optimizations
#        t           -> favor execution time over code size in optimizations
#        u           -> all functions must have unique addresses
#        x           -> equivalent to -omilr -s
#        z           -> NULL points to valid memory in the target environment
#    -p{e,l,c,w=<num>} preprocess file
#        e           -> encrypt identifiers
#        l           -> insert #line directives
#        c           -> preserve comments
#        w=<num>     -> wrap output lines at <num> columns. Zero means no wrap.
#    -r              save/restore segment registers across calls
#    -ri             promote to int all function arguments and returns
#    -s              remove stack overflow checks
#    -sg             generate calls to grow the stack
#    -st             touch stack through SS first
#    -t<number>      set width of a tab stop
#    -u<name>        undefine macro name
#    -v              output function declarations to .def
#    -w<number>      set warning level number
#    -we             treat all warnings as errors
#    -wx             set warning level to maximum setting
#    -xd             disable exception handling (default)
#    -xdt            disable exception handling (same as -xd)
#    -xds            disable exception handling (table-driven destructors)
#    -xs             enable exception handling
#    -xst            enable exception handling (direct calls for destruction)
#    -xss            enable exception handling (table-driven destructors)
#    -za             disable extensions (i.e., accept only ISO/ANSI C++)
#    -ze             enable extensions (i.e., near, far, export, etc.)
#    -zc             place literal strings in the code segment
#    -zdf            DS floats i.e. not fixed to DGROUP
#    -zdl            Load DS directly from DGROUP
#    -zdp            DS is pegged to DGROUP
#    -zff            FS floats i.e. not fixed to a segment
#    -zfp            FS is pegged to a segment
#    -zgf            GS floats i.e. not fixed to a segment
#    -zgp            GS is pegged to a segment
#    -zg             generate function prototypes using base types
#    -zk{0,1,2}      double-byte char support: 0=Kanji,1=Chinese/Taiwanese,2=Korean
#    -zk0u           translate double-byte Kanji to UNICODE
#    -zku=<codepage> load UNICODE translate table for specified code page
#    -zl             remove default library information
#    -zld            remove file dependency information
#    -zm             place each function in separate segment
#    -zo             use exception-handling for a specific operating system
#    -zp{1,2,4,8}    pack structure members
#    -zq             operate quietly
#    -zs             syntax check only
#    -zt<number>     set far data threshold (i.e., larger objects go in far memory)
#    -zu             SS != DGROUP
#    -zw             generate code for Microsoft Windows
#    -zz             remove "@size" from __stdcall function names (10.0 compatible)
#
# common compiler flags FOR NETWARE (independent of debug / nondebug)
#       /3s             386 Stack calling conventions
#       /bm             build target is a multi-thread environment
#       /bt=NETWARE     build target is operating system <os>
#       /d2             full symbolic debugging information
#       /e10            set error limit number
#       /mf             Flat memory model
#       /r              save/restore segment registers across calls
#       /s              remove stack overflow checks
#       /zq             operate quietly
#


TWGFLG= -D__TWG__
CFLGSCOM=/mf/zq/s/e10/bt=NETWARE -D__NETWARE_386__
!IFDEF db
#   /d2 /w4             Full Symbolic and Wider error count
CFLGS=/3s $(CFLGSCOM)/bm /d2/w4$(TWGFLG) -D_DEBUG
CFLGSD=/3s $(CFLGSCOM)/bm /d2/w4$(TWGFLG) -D_DEBUG
CFLAGS=/r /4s $(CFLGSCOM)$(TWGFLG) -D_DEBUG
DEBUGALL=debug all
!else
#   /w1 Error Count
CFLGS=/3s $(CFLGSCOM)/bm/w1$(TWGFLG)
CFLGSD=/3s $(CFLGSCOM)/bm/w1$(TWGFLG)
CFLAGS=/r /4s $(CFLGSCOM)$(TWGFLG)
DEBUGALL=
!endif

# common linker flags (independent of debug / nondebug)
#   /nologo = "quiet" link...no copyright statements

LFLGS=option quiet

# Conditional for debug-versus-nondebug-specific flags
!IFDEF db
CFLGS2=/d2/w4 $(CFLGS)
CFLGS2D=/d2/w4 $(CFLGSD)
!else
CFLGS2=/w1 $(CFLGS)
CFLGS2D=/w1 $(CFLGSD)
!endif


WATCOM=$(WATCOM106DIR)

PATH=$(WATCOM106DIR)\binnt;$(WATCOM106DIR)\binp;$(WATCOM106DIR)\binb;$(PATH)
INCLUDE=$(WATCOM106DIR);$(WATCOM106DIR)\novh;$(WATCOM106DIR)\H;$(WATCOM106DIR)\h\os2
LIB=$(WATCOM106DIR)\lib386\netware;$(WATCOM106DIR)\lib386;$(WATCOM106DIR)\lib386\os2

