/*****************************************************************************
 * Licensed Materials - Property of IBM                            
 * PID (5697-TKT)                                                             
 * (C) Copyright by IBM 1999. All Rights Reserved.
 ****************************************************************************/
class KeyInfoBlock;
class MonitorLibrary;

/**             
 * This class is a the base for ValueGenerators. This class is intended to be 
 * overriden so that the setValueFor and addNodes member functions can be 
 * implemented. 
 */
class ValueGenerator
{
public:
  /**
   * Constructor for the ValueGenerator objects.
   */
  ValueGenerator();
  /**
   * Destructor for the ValueGenerator objects.
   */
  virtual
   ~ValueGenerator();

  /**
   * Abstract member function intended to be implemented in a derived class. 
   * Used to set the value into the KeyInfoBlock object.
   *
   * @param fBlock - KeyInfoBlock object that needs its value set.   
   */
  virtual void
    setValueFor( KeyInfoBlock& fBlock) = 0;

  /**
   * Abstract member function intended to be implemented in a derived class. 
   * Used to add nodes, KeyInfoBlock objects, to the MonitorLibrary object.
   *
   * @param fMonitor - MonitorLibrary object that the KeyInfoBlocks will be 
   *                   added to.   
   */
  virtual void
    addNodesTo( MonitorLibrary& fMonitor ) = 0;

  /**
   * Virtual member function that may be implemented in a derived class. 
   * Used to modify or add nodes to the KeyInfoBlock passed in. This is to allow
   * dynamic Monitors.
   *
   * @param fBlock - KeyInfoBlock object that needs its value set.   
   */
  virtual void
    updateNodesFor( KeyInfoBlock& fBlock,
                    MonitorLibrary& fMonitor);
};
